@@ -16,6 +16,8 @@ GNU General Public License for more details.
 #include "molchrg.h"
 #include "phmodel.h"
 
+using namespace std;
+
 namespace OpenBabel {
 
 extern OBAromaticTyper  aromtyper;
@@ -188,6 +190,17 @@ int OBAtom::CountBondsOfOrder(unsigned int order)
   return(count);
 }
 
+bool OBAtom::HasNonSingleBond()
+{
+  OBBond *bond;
+  vector<OBEdgeBase*>::iterator i;
+  for (bond = BeginBond(i);bond;bond = NextBond(i))
+    if (bond->GetBO() != 1)
+      return(true);
+
+  return(false);
+}
+
 bool OBAtom::IsPolarHydrogen()
 {
   if (!IsHydrogen()) return(false);
@@ -290,7 +303,7 @@ char *OBAtom::GetType()
       OBTypeTable tempTable;
       tempTable.SetFromType("ATN");
       tempTable.SetToType("INT");
-      sprintf(num, "%d", GetAtomicNum());
+      snprintf(num, 6, "%d", GetAtomicNum());
       tempTable.Translate(_type, num);
     }
 
