@@ -20,6 +20,8 @@
 #include "chains.h"
 #include <map>
 
+using namespace std;
+
 ////////////////////////////////////////////////////////////////////////////////
 // Preprocessor Definitions
 ////////////////////////////////////////////////////////////////////////////////
@@ -353,8 +355,8 @@ static ByteCode *AllocateByteCode(int type)
     result = (ByteCode*)malloc(sizeof(ByteCode));
     if( !result )
     {   
-		cerr << "Error: Unable to allocate byte codes in chains.cpp!" << endl;
-        exit(1);
+      cerr << "Error: Unable to allocate byte codes in chains.cpp!" << endl;
+      exit(1);
     }
     result->type = type;
     
@@ -363,8 +365,8 @@ static ByteCode *AllocateByteCode(int type)
 
 static void FatalMemoryError(void)
 {
-	cerr << "Error: Fatal memory allocation error in chains.cpp!" << endl;
-    exit(1);
+  cerr << "Error: Fatal memory allocation error in chains.cpp!" << endl;
+  exit(1);
 }
 
 void GenerateByteCodes(ByteCode **node, int resid, int curr, int prev, int bond)
@@ -556,7 +558,7 @@ void GenerateByteCodes(ByteCode **node, int resid, int curr, int prev, int bond)
 	} 
 	else if( count )
 	{   
-		fputs("Error: Maximum Monomer Fanout Exceeded!\n",stderr);
+	  cerr << "Error: Maximum Monomer Fanout Exceeded!" << endl;
 		fprintf(stderr,"Residue %s atom %d\n",ChainsResName[resid],curr);
 		fprintf(stderr,"Previous = %d  Fanout = %d\n",prev,count);
 		exit(1);
@@ -689,25 +691,15 @@ void OBChainsParser::SetupMol(OBMol &mol)
 
 void OBChainsParser::CleanupMol(void)
 {
-	if (bitmasks != NULL) delete bitmasks;
-	if (hetflags != NULL) delete hetflags;
-	if (atomids  != NULL) delete atomids;
-	if (resids   != NULL) delete resids;
-	if (resnos   != NULL) delete resnos;
-	if (sernos   != NULL) delete sernos;
-	if (hcounts  != NULL) delete hcounts;
-	if (chains   != NULL) delete chains;
-	if (flags    != NULL) delete flags;
-
-	bitmasks = NULL;
-	hetflags = NULL;
-	atomids  = NULL;
-	resids   = NULL;
-	resnos   = NULL;
-	sernos   = NULL;
-	hcounts  = NULL;
-	chains   = NULL;
-	flags    = NULL;
+  if (bitmasks != NULL) {delete bitmasks; bitmasks = NULL;}
+  if (hetflags != NULL) {delete hetflags; hetflags = NULL;}
+  if (atomids  != NULL) {delete atomids; atomids = NULL;}
+  if (resids   != NULL) {delete resids; resids = NULL;}
+  if (resnos   != NULL) {delete resnos; resnos = NULL;}
+  if (sernos   != NULL) {delete sernos; sernos = NULL;}
+  if (hcounts  != NULL) {delete hcounts; hcounts = NULL;}
+  if (chains   != NULL) {delete chains; chains = NULL;}
+  if (flags    != NULL) {delete flags; flags = NULL;}
 }
 
 void OBChainsParser::ClearResidueInformation(OBMol &mol)
