@@ -15,6 +15,8 @@ GNU General Public License for more details.
 #include "binary.h"
 #include "phmodel.h"
 
+using namespace std;
+
 namespace OpenBabel {
 
 extern bool SwabInt;
@@ -675,21 +677,27 @@ OBMol &OBMol::operator=(const OBMol &source)
 
       //Set reference array
       unsigned char *ref = new unsigned char [rml->NumRotors()*4];
-      rml->GetReferenceArray(ref);
-      cp_rml->Setup((*this),ref,rml->NumRotors());
-      delete [] ref;
+      if (ref)
+	{
+	  rml->GetReferenceArray(ref);
+	  cp_rml->Setup((*this),ref,rml->NumRotors());
+	  delete [] ref;
+	}
 
       //Set Rotamers
       unsigned char *rotamers = new unsigned char [(rml->NumRotors()+1)*rml->NumRotamers()];
-      vector<unsigned char*>::iterator kk;
-      unsigned int idx=0;
-      for (kk = rml->BeginRotamer();kk != rml->EndRotamer();kk++)
-        {
-          memcpy(&rotamers[idx],(const unsigned char*)*kk,sizeof(unsigned char)*(rml->NumRotors()+1));
-          idx += sizeof(unsigned char)*(rml->NumRotors()+1);
-        }
-      cp_rml->AddRotamers(rotamers,rml->NumRotamers());
-      delete [] rotamers;
+      if (rotamers)
+	{
+	  vector<unsigned char*>::iterator kk;
+	  unsigned int idx=0;
+	  for (kk = rml->BeginRotamer();kk != rml->EndRotamer();kk++)
+	    {
+	      memcpy(&rotamers[idx],(const unsigned char*)*kk,sizeof(unsigned char)*(rml->NumRotors()+1));
+	      idx += sizeof(unsigned char)*(rml->NumRotors()+1);
+	    }
+	  cp_rml->AddRotamers(rotamers,rml->NumRotamers());
+	  delete [] rotamers;
+	}
       SetData(cp_rml); 
     }
 
@@ -875,12 +883,20 @@ OBBond *OBMol::CreateBond(void)
 
 void OBMol::DestroyAtom(OBNodeBase *atom)
 {
-  delete atom;
+  if (atom)
+    {
+      delete atom;
+      atom = NULL;
+    }
 }
 
 void OBMol::DestroyBond(OBEdgeBase *bond)
 {
-  delete bond;
+  if (bond)
+    {    
+      delete bond;
+      bond = NULL;
+    }
 }
 
 OBAtom *OBMol::NewAtom()
@@ -1302,7 +1318,7 @@ bool OBMol::AddHydrogens(bool polaronly,bool correctForPH)
 
   //Just delete the pose coordinate array.  It will automatically
   //be reallocated when SetPose(unsigned int) is called.
-  if (_xyz_pose) delete [] _xyz_pose; _xyz_pose = NULL;
+  if (_xyz_pose) {delete [] _xyz_pose; _xyz_pose = NULL;}
 
   IncrementMod();
 
@@ -1382,7 +1398,7 @@ bool OBMol::AddHydrogens(OBAtom *atom)
 
   //Just delete the pose coordinate array.  It will automatically
   //be reallocated when SetPose(unsigned int) is called.
-  if (_xyz_pose) delete [] _xyz_pose; _xyz_pose = NULL;
+  if (_xyz_pose) {delete [] _xyz_pose; _xyz_pose = NULL;}
 
   IncrementMod();
 
@@ -1849,7 +1865,11 @@ bool OBMol::DeleteResidue(OBResidue *residue)
 
     _residue.erase(_residue.begin() + idx);
 
-    delete residue;
+    if (residue)
+      {
+	delete residue;
+	residue = NULL;
+      }
 
     return(true);
 }
@@ -1858,8 +1878,9 @@ bool OBMol::AddBond(int first,int second,int order,int stereo,int insertpos)
 {
   BeginModify();
 
-  if ((unsigned)first <= NumAtoms() && (unsigned)second <= NumAtoms()) 
-    //atoms exist
+  if ((unsigned)first <= NumAtoms() && (unsigned)second <= NumAtoms()
+      && !GetBond(first, second))
+    //atoms exist and bond doesn't
     {
       OBBond *bond = CreateBond();
       if (!bond)
@@ -2243,7 +2264,7 @@ OBMol::~OBMol()
   _vconf.clear();
 
   //Deallocate the pose coordinate array if necessary
-  if (_xyz_pose) delete [] _xyz_pose; _xyz_pose = NULL;
+  if (_xyz_pose) {delete [] _xyz_pose; _xyz_pose = NULL;}
 
   if (!_vdata.empty())
   {
@@ -2640,6 +2661,8 @@ void OBMol::ConnectTheDots(void)
   delete [] c;
 }
 
+// From current connectivity (but little or no atom types)
+// assign multiple bonds to fill out valences, etc.
 void OBMol::PerceiveBondOrders()
 {
   OBAtom *atom, *b, *c;
@@ -2710,6 +2733,54 @@ void OBMol::PerceiveBondOrders()
 
   // Pass 4: Check for known functional group patterns and assign bonds
   //         to the canonical form
+  //      Currently we have explicit code to do this, but a "bond typer"
+  //      is in progress to make it simpler to test and debug.
+  OBBond *b1,*b2,*b3;
+  OBAtom *a1,*a2,*a3,*a4;
+  vector<vector<int> > mlist;
+  vector<vector<int> >::iterator l;
+
+  // azide -N=N=N
+  OBSmartsPattern azide; azide.Init("[#7D2][#7D2][#7D1]");
+  if (azide.Match(*this))
+    {
+      mlist = azide.GetUMapList();
+      for (l = mlist.begin(); l != mlist.end(); l++)
+        {
+          a1 = GetAtom((*l)[0]);
+          a2 = GetAtom((*l)[1]);
+          a3 = GetAtom((*l)[2]);
+          b1 = a2->GetBond(a1); b2 = a2->GetBond(a3);
+
+          if (!b1 || !b2) continue;
+          b1->SetBO(2);
+          b2->SetBO(2);
+        }
+    } // Azide
+
+  // Sulphone -SO2-
+  OBSmartsPattern sulphone; sulphone.Init("[#16D4]([#8D1])([#8D1])(*)(*)");
+  if (sulphone.Match(*this))
+    {
+      mlist = sulphone.GetUMapList();
+      for (l = mlist.begin(); l != mlist.end(); l++)
+        {
+          a1 = GetAtom((*l)[0]);
+          a2 = GetAtom((*l)[1]);
+          a3 = GetAtom((*l)[2]);
+	  a4 = GetAtom((*l)[3]);
+          b1 = a1->GetBond(a2); b2 = a1->GetBond(a3); b3 = a1->GetBond(a4);
+
+          if (!b1 || !b2 || !b3) continue;
+          b1->SetBO(2);
+          b2->SetBO(2);
+	  b3->SetBO(1);
+	  a4 = GetAtom((*l)[4]);
+	  b3 = a1->GetBond(a4);
+	  if (!b3) continue;
+	  b3->SetBO(1);
+        }
+    } // Sulfone
 
   // Pass 5: Check for aromatic rings and assign bonds as appropriate
 
@@ -2738,6 +2809,9 @@ void OBMol::PerceiveBondOrders()
 	  // loop through the neighbors looking for a hybrid or terminal atom
 	  // (and pick the one with highest electronegativity first)
 	  // *or* pick a neighbor that's a terminal atom
+	  if (atom->HasNonSingleBond())
+	    continue;
+
 	  maxElNeg = 0.0f;
 	  shortestBond = 5000.0f;
 	  c = NULL;
@@ -2750,9 +2824,12 @@ void OBMol::PerceiveBondOrders()
 		       (currentElNeg == maxElNeg
 			&& (atom->GetBond(b))->GetLength() < shortestBond)) )
 		{
-		      shortestBond = (atom->GetBond(b))->GetLength();
-		      maxElNeg = etab.GetElectroNeg(b->GetAtomicNum());
-		      c = b; // save this atom for later use
+		  if (atom->HasNonSingleBond())
+		    continue;
+
+		  shortestBond = (atom->GetBond(b))->GetLength();
+		  maxElNeg = etab.GetElectroNeg(b->GetAtomicNum());
+		  c = b; // save this atom for later use
 		}
 	    }
 	  if (c)
@@ -2762,6 +2839,9 @@ void OBMol::PerceiveBondOrders()
 		&& atom->BOSum() < etab.GetMaxBonds(atom->GetAtomicNum()) )
 	{
 	  // as above
+	  if (atom->HasNonSingleBond())
+	    continue;
+	  
 	  maxElNeg = 0.0f;
 	  shortestBond = 5000.0f;
 	  c = NULL;
@@ -2774,6 +2854,9 @@ void OBMol::PerceiveBondOrders()
                       (currentElNeg == maxElNeg
                        && (atom->GetBond(b))->GetLength() < shortestBond)) )
 		{
+		  if (atom->HasNonSingleBond())
+		    continue;
+		
 		  shortestBond = (atom->GetBond(b))->GetLength();
 		  maxElNeg = etab.GetElectroNeg(b->GetAtomicNum());
 		  c = b; // save this atom for later use
@@ -3038,7 +3121,7 @@ void OBMol::DeletePoses()
       }
 
     //Free the pose coordinate array
-    if (_xyz_pose) delete [] _xyz_pose; _xyz_pose = NULL;
+    if (_xyz_pose) {delete [] _xyz_pose; _xyz_pose = NULL;}
 
     _pose.clear();
 
