@@ -1,5 +1,9 @@
 /**********************************************************************
 Copyright (C) 1998-2001 by OpenEye Scientific Software, Inc.
+Some portions Copyright (c) 2001-2002 by Geoffrey R. Hutchison
+
+This file is part of the Open Babel project.
+For more information, see <http://openbabel.sourceforge.net/>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,8 +15,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
-#ifndef OBIFSTREAM_H
-#define OBIFSTREAM_H
+#ifndef OB_IFSTREAM_H
+#define OB_IFSTREAM_H
 
 #ifdef __sgi
 #include <iostream.h>
@@ -22,7 +26,7 @@ GNU General Public License for more details.
 #include <fstream>
 #endif
 
-class obifstream : public ifstream
+class obifstream : public std::ifstream
 {
 public:
   bool SafeGetline(char *buffer,int size)
@@ -48,4 +52,4 @@ public:
     }
 };
 
-#endif //OBIFSTREAM_H
+#endif //OB_IFSTREAM_H
