@@ -17,14 +17,14 @@
 // RELEASEDATE should be either __DATE__ or a fixed string like "2000-11-02"
 
 //#define RELEASEDATE __DATE__
-#define RELEASEDATE "2002-02-22"
+#define RELEASEDATE "2003-12-23"
 
 #define WEBSITE "http://www.bioinformatics.org/ghemical"
 
 // could we keep VERSION and RELEASEDATE here???
 
 #ifndef VERSION		// we need this for the old targets also... TH
-#define VERSION "0.82"
+#define VERSION "1.01"
 #endif	// VERSION
 
 /*################################################################################################*/
@@ -35,16 +35,16 @@
 
 /*################################################################################################*/
 
-#ifndef PRIMARY_PATH	// version number and subdirectory (e.g. parameters) in code
+#ifndef PRIMARY_PATH	// the install path of shared files (without version number).
 #define PRIMARY_PATH "/usr/local/share/ghemical/"	// the default for old makefiles...
 #endif
 
-#define SECONDARY_PATH "./"
+#define SECONDARY_PATH "./"	// DO NOT CHANGE! only for testing in the ghemical/bin directory.
 
 // some additional macros here...
 
-#define MM1_AMINO_BUILDER_FILE "builder/amino.txt"
-#define MM1_NUCLEIC_BUILDER_FILE "builder/nucleic.txt"
+#define MM1_AMINO_BUILDER_FILE "parameters/builder/amino.txt"
+#define MM1_NUCLEIC_BUILDER_FILE "parameters/builder/nucleic.txt"
 
 // directory and extension separators in filenames...
 
@@ -80,6 +80,7 @@
 #define LABEL_ELEMENT		0x03
 #define LABEL_ATOMTYPE		0x04
 #define LABEL_BONDTYPE		0x05
+#define LABEL_RESIDUE		0x06
 
 /*################################################################################################*/
 
