@@ -328,7 +328,9 @@ OBMol * file_trans::CopyAll(mm1_mdl * mdl)
 	return Copy(mdl, mdl->atom_list.begin(), mdl->atom_list.end());
 }
 
-void file_trans::Synchronize(void)
+// if you only want to add atoms and bonds (and not change existing ones)
+// set the parameter to true
+void file_trans::Synchronize(bool onlyAdd)
 {
 	// create a new tagtab for the current situation (for bonds)...
 	
@@ -375,7 +377,7 @@ void file_trans::Synchronize(void)
 	// this seems to crash -> disabled!!!!!!!!!!
 	//		atmr->charge = oba->GetPartialCharge();
 		}
-		else
+		else if (!onlyAdd)
 		{
 			atmr->el = element(oba->GetAtomicNum());
 			
@@ -412,7 +414,8 @@ void file_trans::Synchronize(void)
 		tagtab[n1].atmr = NULL;
 		tagtab[n1].oba = NULL;
 		
-		mdl->RemoveAtom(it1);
+		if (!onlyAdd)
+		  mdl->RemoveAtom(it1);
 	}
 	
 	// now we should have the atoms in sync; do the same for bonds.
@@ -454,7 +457,8 @@ void file_trans::Synchronize(void)
 
 			if (obb->IsAromatic())	bt = BONDTYPE_CNJGTD;
 			
-			bndr->bt = bondtype(bt);
+			if (!onlyAdd)
+			  bndr->bt = bondtype(bt);
 			
 			break;
 		}
