@@ -84,7 +84,8 @@ class file_trans
   OBMol * CopyAll(mm1_mdl *);
   
   /// This will syncronize the model and the OBMol.
-  void Synchronize(void);
+  // (if you only wish to add new bonds and atoms, set the parameter)
+  void Synchronize(bool onlyAdd = false);
   
 };
 
