@@ -63,9 +63,19 @@ class intcrd
 	
 	void UpdateCartesian(void);
 	
+	/// the "##variable" functions try to retain the torsions of non-rotating bonds constant.
+	
 	i32s GetVariableCount(void);
+	i32s FindVariable(fGL *, fGL *);
 	void SetVariable(i32s, fGL);
 	fGL GetVariable(i32s);
+	
+	/// the "##torsion" functions give access to all torsions. USE VERY CAREFULLY!!!
+	
+	i32s GetTorsionCount(void);
+	i32s FindTorsion(fGL *, fGL *);
+	void SetTorsion(i32s, fGL);
+	fGL GetTorsion(i32s);
 };
 
 /*################################################################################################*/
