@@ -13,6 +13,7 @@
 class all_atoms_interface;
 class trajectory_interface;
 
+#include "model.h"
 #include "typedef.h"
 #include "utility.h"
 
@@ -133,6 +134,10 @@ class trajectory_interface
 	
 	public:
 	
+	model_simple * ms;
+	
+	public:
+	
 	trajectory_interface(void);
 	virtual ~trajectory_interface(void);
 	
