@@ -21,6 +21,7 @@ class mm1_moldyn_param;
 #define UPDATEFRQ 25
 class mm1_random_search;	// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
 class mm1_systematic_search;	// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
+class mm1_monte_carlo_search;	// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
 
 /*################################################################################################*/
 
@@ -175,6 +176,8 @@ class mm1_random_search
 	mm1_random_search(mm1_mdl *, i32s, i32s, i32s, i32s, i32s);
 	~mm1_random_search(void);
 	
+	f64 GetMinEnergy(void) { return min_energy; }
+	
 	// the search is split in small steps to make it "look nice" in graphic environment.
 	// you should just keep callig TakeStep() until it returns a negative value...
 	
@@ -195,11 +198,69 @@ class mm1_systematic_search
 	i32s divisions;
 	i32s optsteps;
 	
+	mm1_intcrd * ic;
+	
+	mm1_eng * eng;
+	mm1_geomopt * go;
+	
+	i32s nvar;
+	i32s * counter1;
+	i32s counter2;
+	
+	f64 min_energy;
+	
 	public:
 	
 	mm1_systematic_search(mm1_mdl *, i32s, i32s, i32s, i32s, i32s);
 	~mm1_systematic_search(void);
 	
+	f64 GetMinEnergy(void) { return min_energy; }
+	
+	// the search is split in small steps to make it "look nice" in graphic environment.
+	// you should just keep callig TakeStep() until it returns a negative value...
+	
+	i32s TakeStep(void);
+};
+
+/*################################################################################################*/
+
+/**	A monte carlo search (intcrd based). both for conformational search and for collecting 
+	a set of probable states for a molecule.
+*/
+
+class mm1_monte_carlo_search
+{
+	protected:
+
+	mm1_mdl * mdl; i32s molnum;
+	i32s in_crdset; i32s out_crdset;
+	
+	i32s n_init_steps;
+	i32s n_simul_steps;
+	i32s optsteps;
+	
+	mm1_intcrd * ic;
+	
+	mm1_eng * eng;
+	mm1_geomopt * go;
+	
+	i32s counter1;
+	i32s counter2;
+	
+	i32s nvar;
+	f64 * curr_ic1;
+	f64 * curr_ic2;
+	f64 curr_energy;
+	
+	f64 min_energy;
+	
+	public:
+	
+	mm1_monte_carlo_search(mm1_mdl *, i32s, i32s, i32s, i32s, i32s, i32s);
+	~mm1_monte_carlo_search(void);
+	
+	f64 GetMinEnergy(void) { return min_energy; }
+	
 	// the search is split in small steps to make it "look nice" in graphic environment.
 	// you should just keep callig TakeStep() until it returns a negative value...
 	
