@@ -16,6 +16,12 @@ using namespace std;
 
 void CopyCRD(mm1_mdl * p1, mm1_eng * p2, i32u p3)
 {
+	if (p3 >= p1->cs_vector.size())
+	{
+		cout << "cs overflow at mm1 CopyCRD()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
 	i32u n1 = 0; iter_mm1al it1;
 	for (it1 = p1->atom_list.begin();it1 != p1->atom_list.end();it1++)
 	{
@@ -29,6 +35,12 @@ void CopyCRD(mm1_mdl * p1, mm1_eng * p2, i32u p3)
 
 void CopyCRD(mm1_eng * p1, mm1_mdl * p2, i32u p3)
 {
+	if (p3 >= p2->cs_vector.size())
+	{
+		cout << "cs overflow at mm1 CopyCRD()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
 	i32u n1 = 0; iter_mm1al it1;
 	for (it1 = p2->atom_list.begin();it1 != p2->atom_list.end();it1++)
 	{
@@ -70,7 +82,7 @@ mm1_eng::mm1_eng(mm1_mdl & p1) : engine(), mdl(p1)
 		range_cr1[(* it1).index] = cr1.size();
 		range_cr2[(* it1).index] = cr2.size();
 		
-		SearchCR1a(it1);
+		SearchCR1a(& (* it1));
 	}
 	
 	range_cr1[GetModel()->atom_list.size()] = cr1.size();		// this is just the end mark...
@@ -117,6 +129,7 @@ void mm1_eng::Compute(i32s p1)
 	ComputeNBT1(p1);
 	
 	energy = energy_bt1 + energy_bt2 + energy_bt3 + energy_bt4 + energy_nbt1;
+//cout << "energy = " << energy_bt1 << " " << energy_bt2 << " " << energy_bt3 << " " << energy_bt4 << " " << energy_nbt1 << endl;
 }
 
 void mm1_eng::Check(i32s)
@@ -140,7 +153,7 @@ void mm1_eng::Check(i32s)
 			cout << "a = " << d1[n1][n2] << " ";
 			cout << "n = " << tmp2 << endl;
 			
-			if ((n1 % 5) == 4) cin >> old;
+			if ((n1 % 5) == 4) cin >> old;		// uses console I/O, but this function is not relevant to users...
 		}
 	}
 }
@@ -161,54 +174,39 @@ f64 mm1_eng::GetGradientVectorLength(void)
 	return sqrt(sum);
 }
 
-void mm1_eng::SearchCR1a(iter_mm1al it1)
+void mm1_eng::SearchCR1a(mm1_atom * atm1)
 {
-	for (iter_mm1cl iter2 = (* it1).cr_list.begin();iter2 != (* it1).cr_list.end();iter2++)
+	for (iter_mm1cl itc = atm1->cr_list.begin();itc != atm1->cr_list.end();itc++)
 	{
-		for (i32s dir = 0;dir < 2;dir++)
-		{
-			if ((* iter2).bndr->atmr[dir] == & (* it1))
-			{
-				mm1_atom * ref2 = (* iter2).bndr->atmr[!dir];
-				cr1.push_back(ref2->index);
-				
-				SearchCR1b(iter2, ref2);
-			}
-		}
+		mm1_atom * atm2 = (* itc).atmr;
+		cr1.push_back(atm2->index);
+		
+		SearchCR1b(atm2, (* itc).bndr);
 	}
 }
 
-void mm1_eng::SearchCR1b(iter_mm1cl iter1, mm1_atom * ref1)
+void mm1_eng::SearchCR1b(mm1_atom * atm2, mm1_bond * reserved1)
 {
-	for (iter_mm1cl iter2 = ref1->cr_list.begin();iter2 != ref1->cr_list.end();iter2++)
+	for (iter_mm1cl itc = atm2->cr_list.begin();itc != atm2->cr_list.end();itc++)
 	{
-		if (& (* iter2) == & (* iter1)) break;
-		for (i32s dir = 0;dir < 2;dir++)
-		{
-			if ((* iter2).bndr->atmr[dir] == ref1)
-			{
-				mm1_atom * ref2 = (* iter2).bndr->atmr[!dir];
-				cr1.push_back(ref2->index);
-				
-				SearchCR2(iter2, ref2);
-			}
-		}
+		if ((* itc).bndr == reserved1) continue;
+		
+		mm1_atom * atm3 = (* itc).atmr;
+		cr1.push_back(atm3->index);
+		
+		SearchCR2(atm3, reserved1, (* itc).bndr);
 	}
 }
 
-void mm1_eng::SearchCR2(iter_mm1cl iter1, mm1_atom * ref1)
+void mm1_eng::SearchCR2(mm1_atom * atm3, mm1_bond * reserved1, mm1_bond * reserved2)
 {
-	for (iter_mm1cl iter2 = ref1->cr_list.begin();iter2 != ref1->cr_list.end();iter2++)
+	for (iter_mm1cl itc = atm3->cr_list.begin();itc != atm3->cr_list.end();itc++)
 	{
-		if (& (* iter2) == & (* iter1)) break;
-		for (i32s dir = 0;dir < 2;dir++)
-		{
-			if ((* iter2).bndr->atmr[dir] == ref1)
-			{
-				mm1_atom * ref2 = (* iter2).bndr->atmr[!dir];
-				cr2.push_back(ref2->index);
-			}
-		}
+		if ((* itc).bndr == reserved1) continue;
+		if ((* itc).bndr == reserved2) continue;
+		
+		mm1_atom * atm4 = (* itc).atmr;
+		cr2.push_back(atm4->index);
 	}
 }
 
