@@ -98,9 +98,9 @@ class mm1_eng : public engine
 	
 	private:
 	
-	void SearchCR1a(iter_mm1al);
-	void SearchCR1b(iter_mm1cl, mm1_atom *);
-	void SearchCR2(iter_mm1cl, mm1_atom *);
+	void SearchCR1a(mm1_atom *);
+	void SearchCR1b(mm1_atom *, mm1_bond *);
+	void SearchCR2(mm1_atom *, mm1_bond *, mm1_bond *);
 	
 	protected:
 	
