@@ -11,6 +11,7 @@
 #include "mm1tab1.h"
 
 #include <algorithm>
+#include <strstream>
 using namespace std;
 
 /*################################################################################################*/
@@ -259,17 +260,16 @@ mm1_eng_exp1a::mm1_eng_exp1a(mm1_mdl & p1) : mm1_eng(p1)
 	
 /*##############################################*/
 /*##############################################*/
-	
-	// report possible errors... THIS IS NOT GOOD IN TARGET3 -> let the GUI handle this!!!
+
+	// report possible errors...
 	
 	i32s total_err = bt1_err + bt2_err + bt3_err;
 	if (total_err)
 	{
-		cout << "there were total of " << total_err << " bt-errors." << endl;
-		cout << "answer Q to quit or any other character to go on ??? ";
-		
-		char eee; cin >> eee;
-		if (eee == 'q' || eee == 'Q') exit(EXIT_FAILURE);
+		char mbuff1[256];
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
+		GetModel()->PrintToLog(mbuff1);
 	}
 }
 
@@ -694,7 +694,7 @@ mm1_eng_exp1b1::mm1_eng_exp1b1(mm1_mdl & p1) : mm1_eng(p1)
 			
 			if (test == range_cr1[ind1 + 1])
 			{
-				test = range_cr1[ind1];
+				test = range_cr2[ind1];
 				while (test < range_cr2[ind1 + 1])
 				{
 					if (cr2[test] == ind2) break;
@@ -713,10 +713,15 @@ mm1_eng_exp1b1::mm1_eng_exp1b1(mm1_mdl & p1) : mm1_eng(p1)
 		}
 	}
 	
-	if (GetModel()->ostr != NULL)
+	// report possible errors...
+	
+	i32s total_err = nbt1_err;
+	if (total_err)
 	{
-		(* GetModel()->ostr) << nbt1_vector.size() << " terms, ";
-		(* GetModel()->ostr) << nbt1_err << " errors." << endl;
+		char mbuff1[256];
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		GetModel()->PrintToLog(mbuff1);
 	}
 }
 
@@ -895,7 +900,7 @@ void mm1_eng_exp1b2::ComputeNBT1(i32s p1)
 			
 			if (test == range_cr1[ind1 + 1])
 			{
-				test = range_cr1[ind1];
+				test = range_cr2[ind1];
 				while (test < range_cr2[ind1 + 1])
 				{
 					if (cr2[test] == ind2) break;
@@ -1138,7 +1143,7 @@ void mm1_eng_exp1b3::UpdateTerms(void)
 				
 				if (t1b > limit) continue;
 				
-				test = range_cr1[ind1];
+				test = range_cr2[ind1];
 				while (test < range_cr2[ind1 + 1])
 				{
 					if (cr2[test] == ind2) break;
@@ -1157,10 +1162,15 @@ void mm1_eng_exp1b3::UpdateTerms(void)
 		}
 	}
 	
-	if (GetModel()->ostr != NULL)
+	// report possible errors...
+	
+	i32s total_err = nbt1_err;
+	if (total_err)
 	{
-		(* GetModel()->ostr) << nbt1_vector.size() << " terms, ";
-		(* GetModel()->ostr) << nbt1_err << " errors." << endl;
+		char mbuff1[256];
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		GetModel()->PrintToLog(mbuff1);
 	}
 }
 
