@@ -22,8 +22,8 @@ using namespace std;
 
 /*################################################################################################*/
 
-mm1_sequencebuilder mm1_mdl::amino_builder(MM1_AMINO_BUILDER_FILE);
-mm1_sequencebuilder mm1_mdl::nucleic_builder(MM1_NUCLEIC_BUILDER_FILE);
+mm1_sequencebuilder mm1_mdl::amino_builder(mm1_chn_info::amino_acid, MM1_AMINO_BUILDER_FILE);
+mm1_sequencebuilder mm1_mdl::nucleic_builder(mm1_chn_info::nucleic_acid, MM1_NUCLEIC_BUILDER_FILE);
 
 mm1_mdl::mm1_mdl(ostream * p1, class_factory & p2) :
 	model_extended(p1, p2), model_simple(p1, p2), all_atoms_interface(), trajectory_interface()
@@ -35,6 +35,8 @@ mm1_mdl::mm1_mdl(ostream * p1, class_factory & p2) :
 	box_hdim[0] = 0.5; box_fdim[0] = 2.0 * box_hdim[0];
 	box_hdim[1] = 1.0; box_fdim[1] = 2.0 * box_hdim[1];
 	box_hdim[2] = 1.5; box_fdim[2] = 2.0 * box_hdim[2];
+	
+	ms = this;	// trajectory_interface
 }
 
 mm1_mdl::~mm1_mdl(void)
@@ -43,7 +45,7 @@ mm1_mdl::~mm1_mdl(void)
 
 const char * mm1_mdl::GetProjectFileNameExtension(void)
 {
-	static const char ext[] = "mm1gp";
+	static const char ext[] = "gpr";
 	return ext;
 }
 
@@ -52,8 +54,9 @@ const char * mm1_mdl::GetProjectFileNameExtension(void)
 
 const char * mm1_mdl::engtab1[] =
 {
-	"mm1_eng_exp1 : the normal engine",
+	"mm1_eng_exp1 : the default engine",
 	"mm1_eng_exp1_mim : the periodic engine (minimum image model)",
+//	"mm1_eng_prmfit : this_is_not_ready_yet!!!",
 	NULL
 };
 
@@ -61,21 +64,30 @@ const i32s mm1_mdl::engtab2[] =
 {
 	ENG_MM1_NORMAL,
 	ENG_MM1_PERIODIC,
+//	ENG_MM1_EXPERIMENTAL,
 	NOT_DEFINED
 };
 
 mm1_eng * mm1_mdl::CreateDefaultEngine(void)
 {
+	prmfit_tables * tab = NULL; mm1_eng_prmfit * eng = NULL;
+	const char forcefield_path[] = "/home/thassine/DATABASE-mm1/organic_molecules/parameters";
+	
 	switch (engtab2[default_eng])
 	{
 		case ENG_MM1_PERIODIC:
 		periodic = true; GatherGroups();
 		return new mm1_eng_exp1_mim(* this);
+
+		case ENG_MM1_EXPERIMENTAL:
+		periodic = false;
+		tab = new prmfit_tables(forcefield_path);	// experimental!!! for debugging only!!!
+		eng = new mm1_eng_prmfit(* this, * tab);	// experimental!!! for debugging only!!!
+		delete tab; return eng;				// experimental!!! for debugging only!!!
 		
 		default:	// ENG_MM1_NORMAL
 		periodic = false;
 		return new mm1_eng_exp1(* this);
-		//return new mm1_eng_exp9(* this);	// experimental!!! for debugging only!!!
 	}
 }
 
@@ -84,9 +96,7 @@ mm1_eng * mm1_mdl::CreateDefaultEngine(void)
 
 void mm1_mdl::PushCRDSets(i32u p1)
 {
-//cs_vector.resize(cs_vector.size() + p1);				// more convenient way?!?!?!
-//for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();			// more crash-proof way?!?!?!?
-	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
+	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(new crd_set());
 	
 	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
 	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
@@ -204,7 +214,7 @@ void mm1_mdl::RemoveBond(iter_mm1bl it1)
 /*##############################################*/
 /*##############################################*/
 
-bool mm1_mdl::ReadStream_OLD(istream & p1)
+bool mm1_mdl::ReadStream_OLD(istream & p1, bool, bool)
 {
 	static const char message[] = "Hmmm. It seems that we are trying to read a VERY OLD version of this file.\nIt's not possible to verify the type and version of the file, so this\noperation may fail. Are you sure you wish to try (Y/N) ??? ";
 	
@@ -242,39 +252,38 @@ cout << "mm1_mdl::ReadStream_OLD() : starting file input." << endl;
 	return true;
 }
 
-bool mm1_mdl::ReadStream(istream & p1, bool selected)
+bool mm1_mdl::ReadStream(istream & p1, bool selected, bool skipheader)
 {
-	int version;
-	
-	if (bond_list.empty() && atom_list.empty())
-	  selected = false;
-
-	// first we have to identify the file version, and then call the matching function,
-	// if necessary. the very first file format did not use the "!Section" idea, so
-	// we must skip the test if no such traces are seen...
-	
-	if (p1.peek() != '!') version = -1;
-	else
+	if (!skipheader)
 	{
-		// now we expect to see "!Header" section.
-		// next we will check the file type and version information...
+		// first we have to identify the file version, and then call the matching function,
+		// if necessary. the very first file format did not use the "!Section" idea, so
+		// we must skip the test if no such traces are seen...
 		
-		p1 >> buffer;		// read "!Header".
-		p1 >> buffer;		// read the file type.
-		p1 >> version;		// read the version number;
+		int version;
+		if (p1.peek() != '!') version = -1;
+		else
+		{
+			// now we expect to see "!Header" section.
+			// next we will check the file type and version information...
+			
+			p1 >> buffer;		// read "!Header".
+			p1 >> buffer;		// read the file type.
+			p1 >> version;		// read the version number;
+			
+			p1.getline(buffer, sizeof(buffer));
+		}
 		
-		p1.getline(buffer, sizeof(buffer));
-	}
-	
-	// now look at the version number, and pick a correct function if needed.
-	
-	if (version < 100) return ReadStream_OLD(p1);
-	else if (version > 100)
-	{
-		static const char message[] = "Ooops. It seems that we are trying to read a FUTURE version of this file.\nIt's not sure if we can handle that. Are you sure you wish to try (Y/N) ??? ";
+		// now look at the version number, and pick a correct function if needed.
 		
-		bool result = err->Question(message);
-		if (!result) return false;
+		if (version < 100) return ReadStream_OLD(p1, selected, true);
+		else if (version > 100)
+		{
+			static const char message[] = "Ooops. It seems that we are trying to read a FUTURE version of this file.\nIt's not sure if we can handle that. Are you sure you wish to try (Y/N) ??? ";
+			
+			bool result = err->Question(message);
+			if (!result) return false;
+		}
 	}
 	
 	// the "!Header" section is now processed,
@@ -302,7 +311,7 @@ cout << "mm1_mdl::ReadStream() : starting file input." << endl;
 			if (n2 > 0)
 			{
 				i32u n3 = cs_vector.size(); PushCRDSets((i32u) n2);
-				for (i32u n4 = n3;n4 < cs_vector.size();n4++) cs_vector[n4].visible = true;
+				for (i32u n4 = n3;n4 < cs_vector.size();n4++) SetCRDSetVisible(n4, true);
 			}
 			
 #ifdef VERBOSE
@@ -618,7 +627,7 @@ bool mm1_mdl::ImportFile(const char *filename, int index)
     }
 
   istrstream interInput(intermed.str());
-  return ReadStream(interInput);
+  return ReadStream(interInput, false);
 }
 
 void mm1_mdl::AddHydrogens(void)
@@ -628,11 +637,14 @@ void mm1_mdl::AddHydrogens(void)
 	
   	file_trans ft;
   	OBMol *obm = ft.CopyAll(this);
-	obm->AddHydrogens(false, false);
-  	ft.Synchronize(); return;
+	obm->AddHydrogens(false, true);
+	// we only want to add atoms and bonds, not change existing ones
+  	ft.Synchronize(true);
 	
 	// The old version can be found in the CVS tree
 	// cvs update -r 1.36 mm1mdl.cpp
+	
+	PrintToLog("Hydrogens added.\n");
 }
 
 void mm1_mdl::RemoveHydrogens(void)
@@ -652,6 +664,8 @@ void mm1_mdl::RemoveHydrogens(void)
 		iter_mm1al it3 = it2++;
 		if ((* it3).el.GetAtomicNumber() == 1) RemoveAtom(it3);
 	}
+	
+	PrintToLog("Hydrogens removed.\n");
 }
 
 iter_mm1al mm1_mdl::FindAtomByIndex(i32s index)
@@ -1115,7 +1129,7 @@ void mm1_mdl::readpdb_ReadData(const char * filename, mm1_readpdb_mdata * mdata,
 	
 	PushCRDSets(new_csets);
 	
-	for (i32u n1 = 0;n1 < cs_vector.size();n1++) cs_vector[n1].visible = true;
+	for (i32u n1 = 0;n1 < cs_vector.size();n1++) SetCRDSetVisible(n1, true);
 	
 	// count the chains and relate them to mdata, create new records to mdata if necessary...
 	
@@ -1876,15 +1890,22 @@ void mm1_mdl::DoEnergy(void)
 {
 	mm1_eng * eng = CreateDefaultEngine();
 	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Calculating Energy ";
+	str1 << "(molecular mechanics";
+	str1 << ", engine = " << engtab1[default_eng];
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, eng, 0);
 	eng->Compute(0);
 	
 	char buffer[128];
 	ostrstream str(buffer, sizeof(buffer)); str.setf(ios::fixed); str.precision(8);
+	str << "Energy = " << eng->energy << " kJ/mol" << endl << ends;
 	
-	str << "Energy = " << eng->energy << " kJ/mol" << ends;
-	
-	err->Message(buffer);
+	PrintToLog(buffer);
 	delete eng;
 }
 
@@ -1894,6 +1915,15 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 	if (!param.confirm) return;
 	
 	mm1_eng * eng = CreateDefaultEngine();
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Starting Geometry Optimization ";
+	str1 << "(molecular mechanics";
+	str1 << ", engine = " << engtab1[default_eng];
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, eng, 0);
 	
 	mm1_geomopt * opt = new mm1_geomopt(eng, 100, 0.025);		// optimal settings?!?!?
@@ -1901,8 +1931,7 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 	char buffer[1024];
 	f64  last_energy = 0.0;		// this is for output and delta_e test...
 	
-	cout << "Cycle     Energy   Gradient     Step      Delta E" << endl;
-	cout << "--------------------------------------------------" << endl;
+	PrintToLog("Cycle    Energy    Gradient       Step        Delta E\n");
 	
 	i32s n1 = 0;	// n1 counts the number of steps...
 	while (true)
@@ -1916,17 +1945,17 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 		
 		if (n1 != 0)
 		  {
-		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e", n1,
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e \n", n1,
 			    opt->optval, eng->GetGradientVectorLength(),
 			    opt->optstp, last_energy - opt->optval);
 		  }
 		else
 		  {
-		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e **********", n1,
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e ********** \n", n1,
 			    opt->optval, eng->GetGradientVectorLength(),
 			    opt->optstp);
 		  }
-		cout << buffer << endl;
+		PrintToLog(buffer);
 		
 		bool terminate = false;
 		
@@ -1935,7 +1964,7 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 			if (n1 >= param.treshold_nsteps)
 			{
 				terminate = true;
-				cout << "the nsteps termination test was passed." << endl;
+				PrintToLog("the nsteps termination test was passed.\n");
 			}
 		}
 		
@@ -1944,7 +1973,7 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 			if (eng->GetGradientVectorLength() < param.treshold_grad)
 			{
 				terminate = true;
-				cout << "the grad termination test was passed." << endl;
+				PrintToLog("the grad termination test was passed.\n");
 			}
 		}
 		
@@ -1957,7 +1986,7 @@ void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 			if (flag)
 			{
 				terminate = true;
-				cout << "the delta_e termination test was passed." << endl;
+				PrintToLog("the delta_e termination test was passed.\n");
 			}
 		}
 		
@@ -2001,6 +2030,14 @@ void mm1_mdl::DoMolDyn(mm1_moldyn_param & param)
 	mm1_eng_exp1_mim * xxx = dynamic_cast<mm1_eng_exp1_mim *>(eng);
 	if (xxx != NULL) zzz = yyy = xxx;
 	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Starting Molecular Dynamics ";
+	str1 << "(molecular mechanics";
+	str1 << ", engine = " << engtab1[default_eng];
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, eng, 0);
 	
 	mm1_moldyn * dyn = new mm1_moldyn(eng, param.temperature, param.timestep);
@@ -2026,8 +2063,10 @@ void mm1_mdl::DoMolDyn(mm1_moldyn_param & param)
 		dyn->TakeMDStep();
 		if (!(n1 % 50))
 		{
-			cout << n1 << " T = " << dyn->ConvEKinTemp(dyn->kin);
-			cout << " pot = " << dyn->pot << " tot = " << (dyn->kin + dyn->pot) << endl;
+			ostrstream str2(mbuff1, sizeof(mbuff1));
+			str2 << "step " << n1 << " T = " << dyn->ConvEKinTemp(dyn->kin);
+			str2 << " Epot = " << dyn->pot << " Etot = " << (dyn->kin + dyn->pot) << endl << ends;
+			PrintToLog(mbuff1);
 		}
 		
 		if (n1 < START)
@@ -2045,8 +2084,13 @@ void mm1_mdl::DoMolDyn(mm1_moldyn_param & param)
 					sum += control[n9];
 				}
 				
-				sum /= (f64) ccnt; ccnt = 0; f64 rtemp = (f64) (n1 + CFRQ) / (f64) START;
-				cout << "average = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl;
+				sum /= (f64) ccnt; ccnt = 0;
+				f64 rtemp = (f64) (n1 + CFRQ) / (f64) START;
+				
+				ostrstream str2(mbuff1, sizeof(mbuff1));
+				str2 << "average Ekin = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl << ends;
+				PrintToLog(mbuff1);
+				
 				dyn->SetEKin(dyn->kin + (dyn->ConvTempEKin(dyn->temp * rtemp) - sum));
 			}
 		}
@@ -2088,20 +2132,12 @@ void mm1_mdl::MolDynGetParam(mm1_moldyn_param & param)
 	param.confirm = true;
 }
 
-void mm1_mdl::DoRandomSearch(void)
+void mm1_mdl::DoRandomSearch(i32s cycles, i32s optsteps)
 {
-	i32s cycles; i32s optsteps;
-	
-	cout << "give number of cycles (100 would be ok) or -1 to cancel : ";
-	cin >> cycles; if (cycles < 0) return;
-	
-	cout << "give number of optimization steps in a cycle (250 would be ok) : ";
-	cin >> optsteps;
-	
 	if (cs_vector.size() < 2)
 	{
 		PushCRDSets(1);
-		cs_vector[1].visible = false;
+		SetCRDSetVisible(1, false);
 	}
 	
 	mm1_random_search rs(this, 0, 0, 1, cycles, optsteps);
@@ -2109,14 +2145,89 @@ void mm1_mdl::DoRandomSearch(void)
 	while (true)
 	{
 		int zzz = rs.TakeStep();
-		cout << zzz << endl;
+	//	cout << zzz << endl;
+		
 		UpdateAllGraphicsViews(true);
 		if (zzz < 0) break;
 	}
 	
-	CopyCRDSet(1, 0);
+	CopyCRDSet(1, 0);		// get the best structure!!!
 	UpdateAllGraphicsViews(true);
-	cout << "RANDOM SEARCH IS READY." << endl;
+	
+	char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "lowest energy found = " << rs.GetMinEnergy() << " kJ/mol" << endl << ends;
+	PrintToLog(mbuff1);
+	
+	PrintToLog("RANDOM SEARCH IS READY.\n");
+}
+
+void mm1_mdl::DoSystematicSearch(i32s divisions, i32s optsteps)
+{
+	if (cs_vector.size() < 2)
+	{
+		PushCRDSets(1);
+		SetCRDSetVisible(1, false);
+	}
+	
+	mm1_systematic_search ss(this, 0, 0, 1, divisions, optsteps);
+	
+	while (true)
+	{
+		int zzz = ss.TakeStep();
+	//	cout << zzz << endl;
+		
+		UpdateAllGraphicsViews(true);
+		if (zzz < 0) break;
+	}
+	
+	CopyCRDSet(1, 0);		// get the best structure!!!
+	UpdateAllGraphicsViews(true);
+	
+	char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "lowest energy found = " << ss.GetMinEnergy() << " kJ/mol" << endl << ends;
+	PrintToLog(mbuff1);
+	
+	PrintToLog("SYSTEMATIC SEARCH IS READY.\n");
+}
+
+void mm1_mdl::DoMonteCarloSearch(i32s n_init_steps, i32s n_simul_steps, i32s optsteps)
+{
+	
+	if (cs_vector.size() < 2)
+	{
+		PushCRDSets(1);
+		//PushCRDSets(1 + n_simul_steps);	//DEBUG - this works...
+		SetCRDSetVisible(1, false);
+	}
+	
+	mm1_monte_carlo_search mcs(this, 0, 0, 1, n_init_steps, n_simul_steps, optsteps);
+	
+	while (true)
+	{
+		int zzz = mcs.TakeStep();
+	//	cout << zzz << endl;
+		
+		UpdateAllGraphicsViews(true);
+		if (zzz < 0) break;
+	}
+	
+	CopyCRDSet(1, 0);		// get the best structure!!!
+	UpdateAllGraphicsViews(true);
+	
+	char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "lowest energy found = " << mcs.GetMinEnergy() << " kJ/mol" << endl << ends;
+	PrintToLog(mbuff1);
+	
+	PrintToLog("MONTE CARLO SEARCH IS READY.\n");
+	
+//DEBUG
+//for (i32s n9 = 0;n9 < n_simul_steps;n9++)
+//{
+//	mm1_superimpose si(this, 1, n9 + 2);
+//	cout << "superimposing structure " << n9 << ":   ";
+//	for (i32s n2 = 0;n2 < 100;n2++) si.TakeCGStep(conjugate_gradient::Newton2An);
+//	cout << "rms = " << si.GetRMS() << endl;
+//}
 }
 
 void mm1_mdl::DoFormula()
@@ -2184,16 +2295,19 @@ void mm1_mdl::OpenTrajectory(const char * fn)
 		
 		if (natoms != (int) atom_list.size())
 		{
-			cout << "incompatible file : different number of atoms!" << endl;
+			PrintToLog("incompatible file : different number of atoms!\n");
 			CloseTrajectory(); return;
 		}
 		
 		trajfile->read((char *) & total_traj_frames, sizeof(total_traj_frames));
-		cout << "the trajectory file contains " << total_traj_frames << " frames." << endl;
+		
+		char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "the trajectory file contains " << total_traj_frames << " frames." << endl << ends;
+		PrintToLog(mbuff1);
 		
 		current_traj_frame = 0;
 	}
-	else cout << "trajectory already exists!" << endl;
+	else PrintToLog("trajectory already exists!\n");
 }
 
 void mm1_mdl::CloseTrajectory(void)
