@@ -45,6 +45,7 @@ using namespace OpenBabel;
 
 #define ENG_MM1_NORMAL		0x100		// mm1_eng_exp9
 #define ENG_MM1_PERIODIC	0x200		// mm1_eng_exp1_mim
+#define ENG_MM1_EXPERIMENTAL	0x300		// mm1_eng_prmfit
 
 /**	A "##model" class for "##all-atoms" molecular mechanics.
 	
@@ -91,7 +92,8 @@ using namespace OpenBabel;
 
 class mm1_mdl : public model_extended, public all_atoms_interface, public trajectory_interface
 {
-	protected:
+//	protected:
+	public:		// mm2-param
 	
 	i32s nmol;
 	vector<mm1_chn_info> * ref_civ;
@@ -198,13 +200,13 @@ class mm1_mdl : public model_extended, public all_atoms_interface, public trajec
 	// methods for file I/O...
 	// ^^^^^^^^^^^^^^^^^^^^^^^
 	
-	bool ReadStream_OLD(istream &);		// this is the very old version...
-//	bool ReadStream_v100(istream &);	// this will be the version 1.00, etc...
+	bool ReadStream_OLD(istream &, bool, bool = false);		// this is the very old version...
+//	bool ReadStream_v100(istream &, bool, bool = false);		// this will be the version 1.00, etc...
 
-	bool ReadStream(istream &, bool = false);	// virtual
+	bool ReadStream(istream &, bool, bool = false);		// virtual
 	
-	/// This is v1.00 output function for the mm1gp file format. The "!Section" idea is used here.
-	void WriteStream(ostream &);			// virtual
+	/// This is v1.00 output function for the .gpr (same as old .mm1gp) file format. The "!Section" idea is used here.
+	void WriteStream(ostream &);				// virtual
 
 	bool ImportFile(const char *filename, int index = 0);	// virtual
 
@@ -335,7 +337,9 @@ class mm1_mdl : public model_extended, public all_atoms_interface, public trajec
 	virtual void MolDynGetParam(mm1_moldyn_param &);
 	
 /// This will perform a random search using torsions as variables. AN INITIAL VERSION...
-	void DoRandomSearch(void);
+	void DoRandomSearch(i32s, i32s);
+	void DoSystematicSearch(i32s, i32s);
+	void DoMonteCarloSearch(i32s, i32s, i32s);
 	
 /// This will print the molecular formula and weight.
 	void DoFormula(void);
