@@ -33,7 +33,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/atomtypes.txt";
+	fn = "parameters/mm1param/stable/atomtypes.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -66,7 +66,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/parameters1.txt";
+	fn = "parameters/mm1param/stable/parameters1.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -93,7 +93,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/parameters2.txt";
+	fn = "parameters/mm1param/stable/parameters2.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -120,7 +120,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/parameters3.txt";
+	fn = "parameters/mm1param/stable/parameters3.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -147,7 +147,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/parameters4.txt";
+	fn = "parameters/mm1param/stable/parameters4.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -173,7 +173,7 @@ exp1_tables::exp1_tables(void)
 /*##############################################*/
 /*##############################################*/
 	
-	fn = "mm1param/stable/parameters5.txt";
+	fn = "parameters/mm1param/stable/parameters5.txt";
 	model_simple::OpenParameterFile(file, false, fn);
 	
 	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
@@ -527,7 +527,11 @@ i32s exp1_tables::UpdateTypes(mm1_mdl * mdl)
 		}
 		else
 		{
-			if (mdl->ostr != NULL) (* mdl->ostr) << "problem: unknown atomtype!!!" << endl;
+			char mbuff1[256];
+			ostrstream str1(mbuff1, sizeof(mbuff1));
+			str1 << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << endl << ends;
+			mdl->PrintToLog(mbuff1);
+		
 			(* it1).atmtp = NOT_DEFINED; (* it1).selected = true; errors++;
 		}
 	}
