@@ -146,14 +146,14 @@ mm1_chn_info::mm1_chn_info(void)
 {
 	description = NULL;
 	
-	type = NOT_DEFINED;
+	type = not_defined;
 	length = NOT_DEFINED;
 	
 	sequence = NULL;
 	state = NULL;
 }
 
-mm1_chn_info::mm1_chn_info(i32s p1, i32s p2, bool p3)
+mm1_chn_info::mm1_chn_info(chn_type p1, i32s p2, bool p3)
 {
 	description = NULL;
 	
@@ -636,8 +636,10 @@ istream & operator>>(istream & p1, mm1_sb_res & p2)
 
 // here we make some direct output to cout -> not good in TARGET3 ?!?!?!
 
-mm1_sequencebuilder::mm1_sequencebuilder(char * filename)
+mm1_sequencebuilder::mm1_sequencebuilder(mm1_chn_info::chn_type ct, char * filename)
 {
+	type = ct;
+	
 	ifstream file;
 	file.unsetf(ios::dec | ios::oct | ios::hex);
 	
@@ -870,7 +872,7 @@ void mm1_sequencebuilder::Identify(mm1_mdl * mdl)
 			i32s rcount = 0;
 			for (i32s n3 = 0;n3 < (i32s) path_vector[n2].size();n3 += main_vector.size())
 			{
-				i32s residue = n3 / main_vector.size();
+				i32s residue = n3 / main_vector.size();		// redundant with rcount?!?!?!
 				bool is_first = !head_blocked && !residue;
 				bool is_last = !tail_blocked && (residue == last_residue);
 				
@@ -902,27 +904,43 @@ void mm1_sequencebuilder::Identify(mm1_mdl * mdl)
 					}
 				}
 				
-				if (tmp1[0] == NOT_DEFINED) sequence.push_back('?');
-				else sequence.push_back(residue_vector[tmp1[0]].symbol);
-				
-				// rebuild the best matching template and use that to
-				// set up the atom ID numbers...
-				
-				vector<mm1_sb_tdata> tdata;
-				BuildTemplate(tdata, tmp1[0], is_first, is_last);
-				
-				for (i32s n4 = 0;n4 < (i32s) main_vector.size();n4++)
+				if (tmp1[0] == NOT_DEFINED)
 				{
-					tdata[n4].ref = path_vector[n2][n3 + n4];
-				}
-				
-				CheckTemplate(tdata, 0);
-				for (i32s n4 = 0;n4 < (i32s) tdata.size();n4++)
-				{
-					tdata[n4].ref->res_id = (residue_vector[tmp1[0]].id << 8) + tdata[n4].id[0];
+					cout << "WARNING : residue " << rcount << " was of unknown type!!!" << endl;
 					
-					tdata[n4].ref->id[1] = mdl->ref_civ->size();
-					tdata[n4].ref->id[2] = rcount;
+					sequence.push_back('?');				// handle the sequence...
+					
+					for (i32u n4 = 0;n4 < main_vector.size();n4++)		// handle the numbers...
+					{
+						path_vector[n2][n3 + n4]->res_id = NOT_DEFINED;		// WHAT HERE???
+						
+						path_vector[n2][n3 + n4]->id[1] = mdl->ref_civ->size();
+						path_vector[n2][n3 + n4]->id[2] = rcount;
+					}
+				}
+				else
+				{
+					sequence.push_back(residue_vector[tmp1[0]].symbol);
+					
+					// rebuild the best matching template and use that to
+					// set up the atom ID numbers...
+					
+					vector<mm1_sb_tdata> tdata;
+					BuildTemplate(tdata, tmp1[0], is_first, is_last);
+					
+					for (i32s n4 = 0;n4 < (i32s) main_vector.size();n4++)
+					{
+						tdata[n4].ref = path_vector[n2][n3 + n4];
+					}
+					
+					CheckTemplate(tdata, 0);
+					for (i32s n4 = 0;n4 < (i32s) tdata.size();n4++)
+					{
+						tdata[n4].ref->res_id = (residue_vector[tmp1[0]].id << 8) + tdata[n4].id[0];
+						
+						tdata[n4].ref->id[1] = mdl->ref_civ->size();
+						tdata[n4].ref->id[2] = rcount;
+					}
 				}
 				
 				rcount++;
@@ -930,7 +948,7 @@ void mm1_sequencebuilder::Identify(mm1_mdl * mdl)
 			
 			// next we will write down the sequence...
 			
-			mm1_chn_info newinfo(-1, sequence.size(), false);
+			mm1_chn_info newinfo(type, sequence.size(), false);
 			for (i32u n3 = 0;n3 < sequence.size();n3++) newinfo.sequence[n3] = sequence[n3];
 			mdl->ref_civ->push_back(newinfo);
 			
