@@ -87,7 +87,7 @@ class mm2_moldyn
 	f64 pot;
 	
 	friend class mm2_mdl;
-//	friend class glut_mm2_docv;
+	friend class glut_mm2_docv;	// temporary...
 	
 	public:
 	
