@@ -1,4 +1,4 @@
-// MM2ENG.H : calculations for reduced protein models (molecular mechanics).
+// MM2ENG.H : calculations for simplified protein models (molecular mechanics).
 
 // Copyright (C) 1998 Tommi Hassinen.
 
@@ -12,6 +12,8 @@
 #ifndef MM2ENG_H
 #define MM2ENG_H
 
+struct mm2_nonbonded_lookup;
+
 struct mm2_bt1;
 struct mm2_bt2;
 struct mm2_bt3;
@@ -30,7 +32,11 @@ struct mm2_nbt3_coi;
 struct mm2_nbt3_ips;
 struct mm2_nbt3_arc;
 
-#define SIZE_NL		100
+#define LAYERS		3
+#define LAYER_LIMIT	0.10
+
+static const int size_nl[LAYERS] = { 100, 200, 400 };
+
 #define SIZE_NT		100
 #define SIZE_COI	100
 #define SIZE_IPD	50
@@ -55,13 +61,21 @@ using namespace std;
 
 /*################################################################################################*/
 
-#define TYPE_LOOP	0x00
-#define TYPE_HELIX	0x01
-#define TYPE_STRAND	0x02
+struct mm2_nonbonded_lookup
+{
+	char s1; i32u a1;
+	char s2; i32u a2;
+	
+	f64 opte;
+};
 
-#define TYPE_SIDE	0x10
-#define TYPE_BRIDGE	0x11
-#define TYPE_TERM	0x12
+#define TTYPE_LOOP	0x00
+#define TTYPE_HELIX	0x01
+#define TTYPE_STRAND	0x02
+
+#define TTYPE_SIDE	0x10
+#define TTYPE_BRIDGE	0x11
+#define TTYPE_TERM	0x12
 
 struct mm2_bt1		// bond stretching
 {
@@ -82,7 +96,7 @@ struct mm2_bt2		// angle bending
 	i32s index1[2];
 	bool dir1[2];
 	
-	i32s type;
+	i32s ttype;
 	
 	f64 opt; f64 fc[2];
 };
@@ -95,7 +109,7 @@ struct mm2_bt3		// main-chain torsion interactions + peptide bond dipoles
 	i32s index1[4];
 	bool dir1[4];
 	
-	i32s tor_type;
+	i32s tor_ttype;
 	
 	f64 torc[3];
 	f64 tors[3];
@@ -112,7 +126,7 @@ struct mm2_bt3		// main-chain torsion interactions + peptide bond dipoles
 	f64 dv[3];
 	f64 ddv[4][3][3];
 	
-	i32s dip_type;
+	i32s dip_ttype;
 	
 	bool skip;
 	
@@ -149,6 +163,7 @@ struct mm2_nbt1			// non-bonded interactions
 	i32s atmi[2];
 	
 	f64 data[2];
+	f64 optr;		// "buffered" functions need this???
 	
 	bool operator==(const mm2_nbt1 & p1) const
 	{
@@ -188,7 +203,8 @@ struct mm2_nbt3_ipd		// SASA intersection point data
 struct mm2_nbt3_nl		// SASA neighbor list
 {
 	i32s index_count;
-	i32s index[SIZE_NL];
+//	i32s index[SIZE_NL];
+	i32s * index;
 };
 
 struct mm2_nbt3_coi		// SASA circle of intersection
@@ -319,33 +335,44 @@ class mm2_eng : public engine
 	
 	vector<mm2_nbt1> nbt1_vector;
 	
-	mm2_nbt3_nl * nbt3_nl;
-	i32s * dist1; f64 * dist2;
-	
-	i32s * nbtp;
-	f64 * sasa;
-	
 	f64 * mass;
-	
+	f64 * charge;
 	f64 * vdwr;
-	f64 * vdwr1;
-	f64 * vdwr2;
+	f64 * pnlt;	// PENALTYDEBUG!!!
+	
+	i32s * dist1; f64 * dist2;
+	mm2_nbt3_nl * nbt3_nl[LAYERS];
+	
+	f64 * vdwr1[LAYERS];
+	f64 * vdwr2[LAYERS];
+	f64 * sasa[LAYERS];
+	
+	fGL * stored_sasa_values;
 	
 	public:
 	
 	f64 energy;
 	f64 energy_bt1;
 	f64 energy_bt2;
-	f64 energy_bt3;
-	f64 energy_bt4;
+	f64 energy_bt3a;
+	f64 energy_bt3b;
+	f64 energy_bt4a;
+	f64 energy_bt4b;
 	f64 energy_nbt1a;
 	f64 energy_nbt1b;
+	f64 energy_nbt1c;
 	f64 energy_nbt2a;
 	f64 energy_nbt2b;
-	f64 energy_nbt3;
+	f64 energy_nbt2c;
+	f64 energy_nbt3a;
+	f64 energy_nbt3b;
+	f64 energy_pnlt;
+	
+	f64_a3 * penalty_data;
+	f64 penalty_weight;
+	f64 penalty_sum2;
 	
 	f64 constraints;
-	f64 constraints_OLD_FIXME;
 	
 	public:
 	
@@ -357,6 +384,8 @@ class mm2_eng : public engine
 	
 	mm2_mdl * GetModel(void) { return & mdl; }
 	
+	void SetPenaltyData(void);
+	
 	void Compute(i32s);	// virtual
 	void Check(i32s);	// virtual
 
@@ -364,7 +393,7 @@ class mm2_eng : public engine
 	
 	protected:
 	
-	bool InitNBT1(mm2_nbt1 *);
+	bool InitNBT1(mm2_nbt1 *, vector<mm2_nonbonded_lookup> &);
 	void InitLenJon(mm2_nbt1 *, f64, f64);
 	
 	i32s GetIndex(vector<mm2_atm> &, mm2_atm);
