@@ -18,26 +18,80 @@ using namespace std;
 mm2_mdl::mm2_mdl(ostream * p1, class_factory & p2, mm2_eng_param * p3) :
 	model_extended(p1, p2), model_simple(p1, p2), trajectory_interface()
 {
-	prm.wtmp1 = 0.875;			// 0.875	// bt2 fc weight (in strands and loops)
-	prm.wtmp2 = 1.000;			// 1.000	// bt4 fc weight
-	prm.wtmp3 = 0.00025;			// 0.00025	// angle rep fc
+	prm.vdwrad = 1.325;				// 1.325	// average main-chain radius 0.218 nm -> 0.275 nm???
+	prm.lenjon = 1.225;				// 1.225
+	prm.wescc = 0.350;				// 0.350 r^-1
+	prm.wescd = 0.450;				// 0.450 r^-2
+	prm.dipole1 = 1.0;				// 1.0
+	prm.dipole2 = 25.0;				// 16.0 / prm.dipole1 * prm.dipole1
+	prm.screen1 = -20.929;				// -20.929
+	prm.screen2 = 78.4 - prm.screen1;		// 78.4 - screen1
+	prm.screen3 = 3.4781;				// 3.4781
+	prm.screen4 = -0.01787;				// -0.01787
+	prm.screen5 = 0.001;				// 0.001	// about 4 ...
+	prm.sasa1[0] = -1.80;				// -1.80
+	prm.sasa1[1] = +3.00;				// +3.00 ???
+	prm.sasa2[0] = 0.60;				// 0.60		// about ???% of previous
+	prm.sasa2[1] = 0.60 * 0.60;			// 0.60 * 0.60	// about ???% of previous
+	prm.solvrad = 0.15;				// 0.15		// H2O -> 0.15
+	prm.wtor1 = 1.00;				// 1.00
+	prm.wtor2 = 3.50;				// 3.50
+	prm.wrep = 0.00175;				// 0.00175	// angle rep fc
 	
-	prm.wtor1 = 2.75;			// 2.75
-	prm.wtor2 = 3.25;			// 3.25
-	prm.lenjon = 1.2550;			// 1.2550	// DOMINATING LJ-TERM -> LOST SENSITIVITY ?!?!?!?
-	prm.vdwrad = 1.35;			// 1.35		// will give an average radius of 0.275 nm...
-	prm.screen1 = -20.929;			// -20.929
-	prm.screen2 = 78.4 - prm.screen1;	// 78.4 minus above...
-	prm.screen3 = 3.4781;			// 3.4781
-	prm.screen4 = -0.01787;			// -0.01787
-	prm.screen5 = 0.001;			// 0.001	// about 4 ...
-	prm.dipole1 = 3.0;			// 3.0		// will give 27 kJ/mol for HB's in SSE's. (27+7)/2 = 17
-	prm.dipole2 = 4.0;			// 4.0		// EASY TO CHECK USING A SINGLE ALPHA-HELIX, LIKE ALA12!!!
-	prm.pbility1 = 0.0025;			// 0.0025	// should be about 0.0015 : CRC handbook of Chem & Phys
-	prm.pbility2 = 0.0005;			// 0.0005	// 1st Student Edition, 1988, CRC Press Inc. (page E-62)
-	prm.sasa1 = -0.5100;			// -0.5100	// near -0.5 -> in the middle, both effects?!?!?!
-	prm.sasa2 = -52.00;			// -52.00	// somewhere near -50.0 !??!?!?!?!
-	prm.solvrad = 0.15;			// 0.15		// H2O -> 0.15
+	prm.sasa9[0] = +0.00;							// a +0.0
+	prm.sasa9[1] = +0.00; prm.sasa9[2] = -1.00; prm.sasa9[3] = -1.00;	// r +0.0 -1.0 +2.0
+	prm.sasa9[4] = +0.00; prm.sasa9[5] = +1.00;				// n +0.0 +1.0
+	prm.sasa9[6] = +0.00; prm.sasa9[7] = -1.00;				// d +0.0 +2.0
+	prm.sasa9[8] = +0.00; prm.sasa9[9] = +0.00;				// c +0.0 +1.0
+	prm.sasa9[10] = +0.00; prm.sasa9[11] = +1.00;				// q +0.0 +1.0
+	prm.sasa9[12] = +0.00; prm.sasa9[13] = -1.00;				// e +0.0 +2.0
+	prm.sasa9[14] = +0.00;							// g +0.0
+	prm.sasa9[15] = +0.00; prm.sasa9[16] = +0.00;				// h +0.0 +1.0
+	prm.sasa9[17] = +0.00; prm.sasa9[18] = -1.00;				// i +0.0 -1.0
+	prm.sasa9[19] = +0.00; prm.sasa9[20] = -1.00;				// l +0.0 -1.0
+	prm.sasa9[21] = +0.00; prm.sasa9[22] = -1.00; prm.sasa9[23] = -1.00;	// k +0.0 -1.0 +2.0
+	prm.sasa9[24] = +0.00; prm.sasa9[25] = -1.00;				// m +0.0 -1.0
+	prm.sasa9[26] = +0.00; prm.sasa9[27] = -1.00;				// f +0.0 -1.0
+	prm.sasa9[28] = -1.00;							// p -1.0
+	prm.sasa9[29] = +1.00;							// s +1.0
+	prm.sasa9[30] = +1.00;							// t +1.0
+	prm.sasa9[31] = +0.00; prm.sasa9[32] = +1.00; prm.sasa9[33] = -1.00;	// w +0.0 +1.0 -1.0
+	prm.sasa9[34] = +0.00; prm.sasa9[35] = +0.00;				// y +0.0 +1.0
+	prm.sasa9[36] = -1.00;							// v -1.0
+	
+	prm.charge_wes = 0.257;			// why so small???
+	prm.charge_sasa2 = 1.090;
+	prm.charge_sasa3 = 0.410;
+	
+	prm.charge_pKa[0] = 7.402;	// n-terminal		// 7.8 down
+	prm.charge_pKa[1] = 4.753;	// c-terminal		// 3.6 up
+	prm.charge_pKa[2] = 11.2;	// R2 (no data)		// 12.0 down? 0.8
+	prm.charge_pKa[3] = 4.280;	// D1			// 4.0 up
+	prm.charge_pKa[4] = 9.9;	// C1 (no data)		// 9.1 up? 0.8
+	prm.charge_pKa[5] = 5.813;	// E1			// 4.5 up
+	prm.charge_pKa[6] = 6.068;	// H1			// 6.4 down
+	prm.charge_pKa[7] = 9.393;	// K2			// 10.5 down
+	prm.charge_pKa[8] = 10.614;	// Y1			// 9.7 up
+	
+	prm.charge_sasa1[0] = 0.362;	// 1.5
+	prm.charge_sasa1[1] = 0.464;	// 1.5
+	prm.charge_sasa1[2] = 1.1;	// 4.4 no data? 25%
+	prm.charge_sasa1[3] = 0.866;	// 3.2
+	prm.charge_sasa1[4] = 0.2;	// 0.8 no data? 25%
+	prm.charge_sasa1[5] = 0.747;	// 3.9
+	prm.charge_sasa1[6] = 0.696;	// 3.1
+	prm.charge_sasa1[7] = 1.326;	// 5.0
+	prm.charge_sasa1[8] = 0.542;	// 2.0
+	
+	prm.charge_acid[0] = false;
+	prm.charge_acid[1] = true;
+	prm.charge_acid[2] = false;
+	prm.charge_acid[3] = true;
+	prm.charge_acid[4] = true;
+	prm.charge_acid[5] = true;
+	prm.charge_acid[6] = false;
+	prm.charge_acid[7] = false;
+	prm.charge_acid[8] = true;
 	
 //////////////////////////////////////////////////
 //////////////////////////////////////////////////
@@ -45,27 +99,46 @@ mm2_mdl::mm2_mdl(ostream * p1, class_factory & p2, mm2_eng_param * p3) :
 
 	if (p3 != NULL)
 	{
-		prm.wtmp1 = p3->wtmp1;
-		prm.wtmp2 = p3->wtmp2;
-		prm.wtmp3 = p3->wtmp3;
-		
-		prm.wtor1 = p3->wtor1;
-		prm.wtor2 = p3->wtor2;
-		prm.lenjon = p3->lenjon;
 		prm.vdwrad = p3->vdwrad;
+		prm.lenjon = p3->lenjon;
+		prm.wescc = p3->wescc;
+		prm.wescd = p3->wescd;
+		prm.dipole1 = p3->dipole1;
+		prm.dipole2 = p3->dipole2;
 		prm.screen1 = p3->screen1;
 		prm.screen2 = p3->screen2;
 		prm.screen3 = p3->screen3;
 		prm.screen4 = p3->screen4;
 		prm.screen5 = p3->screen5;
-		prm.dipole1 = p3->dipole1;
-		prm.dipole2 = p3->dipole2;
-		prm.pbility1 = p3->pbility1;
-		prm.pbility2 = p3->pbility2;
-		prm.sasa1 = p3->sasa1;
-		prm.sasa2 = p3->sasa2;
+		prm.sasa1[0] = p3->sasa1[0];
+		prm.sasa1[1] = p3->sasa1[1];
+		prm.sasa2[0] = p3->sasa2[0];
+		prm.sasa2[1] = p3->sasa2[1];
 		prm.solvrad = p3->solvrad;
+		prm.wtor1 = p3->wtor1;
+		prm.wtor2 = p3->wtor2;
+		prm.wrep = p3->wrep;
+
+		for (i32s n1 = 0;n1 < 37;n1++)
+		{
+			prm.sasa9[n1] = p3->sasa9[n1];
+		}
+		
+		prm.charge_wes = p3->charge_wes;
+		prm.charge_sasa2 = p3->charge_sasa2;
+		prm.charge_sasa3 = p3->charge_sasa3;
+		
+		for (i32s n1 = 0;n1 < 9;n1++)
+		{
+			prm.charge_pKa[n1] = p3->charge_pKa[n1];
+			prm.charge_sasa1[n1] = p3->charge_sasa1[n1];
+			prm.charge_acid[n1] = p3->charge_acid[n1];
+		}
 	}
+	
+	pH = 7.0;
+	
+	ms = this;	// trajectory_interface
 }
 
 mm2_mdl::~mm2_mdl(void)
@@ -83,7 +156,7 @@ const char * mm2_mdl::GetProjectFileNameExtension(void)
 
 void mm2_mdl::PushCRDSets(i32u p1)
 {
-	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());
+	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(new crd_set());
 	
 	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
 	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
@@ -166,7 +239,7 @@ i32s mm2_mdl::GetVirtAtomCount(void)
 	return virt_atom_count;
 }
 
-void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata *)
+void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata * mdata)
 {
 	mdl->GatherGroups();
 	mm1_mdl::amino_builder.Identify(mdl);
@@ -179,7 +252,7 @@ void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata *)
 	
 	PushCRDSets(new_csets);
 	
-	for (i32u n1 = 0;n1 < cs_vector.size();n1++) cs_vector[n1].visible = true;
+	for (i32u n1 = 0;n1 < cs_vector.size();n1++) SetCRDSetVisible(n1, true);
 	
 	// ok, start working...
 	
@@ -205,10 +278,88 @@ void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata *)
 	// here we should check that we don't have any nucleic acid chains...
 	// here we should check that we don't have any nucleic acid chains...
 	
+	// now, we must relate mdata and ci_vector!!! for each ci, find exactly matching mdata...
+	
+	i32s * ci_to_mdata_table = new i32s[ci_vector.size()];
+	
 	for (i32u n1 = 0;n1 < ci_vector.size();n1++)
 	{
+		vector<mm1_atom *> calpha_vector;
+		
 		iter_mm1al range1[2]; mdl->GetRange(1, n1, range1);
-		mm2_chn newchn; chn_vector.push_back(newchn);
+		iter_mm1al range2[2]; i32s res = 0;
+		
+		while (true)
+		{
+			mdl->GetRange(2, range1, res, range2);
+			if (range2[0] == range2[1]) break;
+			
+			for (iter_mm1al it1 = range2[0];it1 != range2[1];it1++)
+			{
+				if (((* it1).res_id & 0xFF) != 0x01) continue;
+				else calpha_vector.push_back(& (* it1));
+			}
+			
+			res++;		// move to next residue...
+		}
+		
+		ci_to_mdata_table[n1] = NOT_DEFINED;
+		
+		if (mdata != NULL)
+		{
+			for (i32u n2 = 0;n2 < mdata->chn_vector.size();n2++)
+			{
+				if (calpha_vector.size() != mdata->chn_vector[n2]->alpha_carbons.size()) continue;
+				
+				bool flag = true;
+				for (i32u n3 = 0;n3 < calpha_vector.size();n3++)
+				{
+					mm1_atom * ref1 = calpha_vector[n3];
+					mm1_atom * ref2 = mdata->chn_vector[n2]->alpha_carbons[n3];
+					if (ref1 != ref2) flag = false;
+				}
+				
+				if (flag)
+				{
+					ci_to_mdata_table[n1] = (i32s) n2;
+					break;
+				}
+			}
+			
+			if (ci_to_mdata_table[n1] == NOT_DEFINED)
+			{
+				cout << "failed to map ci " << n1 << " to any mdata record!!!" << endl;
+				exit(EXIT_FAILURE);
+			}
+		}
+	}
+	
+	for (i32u n1 = 0;n1 < ci_vector.size();n1++)
+	{
+		iter_mm1al range1[2];
+		mdl->GetRange(1, n1, range1);
+		
+		mm2_chn newchn;
+		chn_vector.push_back(newchn);
+		
+		if (ci_to_mdata_table[n1] != NOT_DEFINED)	// copy mdata!!!
+		{
+			i32s index = ci_to_mdata_table[n1];
+			
+			chn_vector.back().pdb_chn_id = mdata->chn_vector[index]->chn_id;
+			
+			if (mdata->chn_vector[index]->seqres != NULL)
+			{
+				chn_vector.back().pdb_seqres = new char[strlen(mdata->chn_vector[index]->seqres) + 1];
+				strcpy(chn_vector.back().pdb_seqres, mdata->chn_vector[index]->seqres);
+			}
+			else
+			{
+				chn_vector.back().pdb_seqres = NULL;
+			}
+			
+			chn_vector.back().pdb_missing_residues = mdata->chn_vector[index]->missing_residues;
+		}
 		
 		iter_mm1al range2[2]; i32s res = 0;
 		
@@ -325,7 +476,7 @@ void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata *)
 			}
 			
 			chn_vector.back().res_vector.push_back(newres);
-			res++;
+			res++;		// move to next residue...
 		}
 		
 		// peptide units...
@@ -354,6 +505,46 @@ void mm2_mdl::Reduce(mm1_mdl * mdl, mm1_readpdb_mdata *)
 		}
 	}
 	
+	delete[] ci_to_mdata_table;
+	
+	// do some output at this stage (the output should be in FASTA format)...
+	
+	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+	{
+		cout << "imported chain " << n1 << ":" << endl;
+		
+		if (chn_vector[n1].pdb_seqres != NULL)
+		{
+			i32s pdb_chn_length = strlen(chn_vector[n1].pdb_seqres);
+			
+			cout << "> ORIGINAL sequence, length " << pdb_chn_length;
+			cout << ", id '" << chn_vector[n1].pdb_chn_id << "':" << endl;
+			
+			for (i32s n2 = 0;n2 < pdb_chn_length;n2++)
+			{
+				cout << chn_vector[n1].pdb_seqres[n2];
+				
+				bool is_break = !((n2 + 1) % 50);
+				bool is_end = ((n2 + 1) == pdb_chn_length);
+				
+				if (is_break || is_end) cout << endl;
+			}
+		}
+		
+		cout << "> IMPORTED sequence, length ";
+		cout << chn_vector[n1].res_vector.size() << ":" << endl;
+		
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			cout << chn_vector[n1].res_vector[n2].symbol;
+			
+			bool is_break = !((n2 + 1) % 50);
+			bool is_end = ((n2 + 1) == chn_vector[n1].res_vector.size());
+			
+			if (is_break || is_end) cout << endl;
+		}
+	}
+	
 	// disulphide bridges...
 	
 	for (i32u n1 = 0;n1 < ci_vector.size();n1++)
@@ -523,19 +714,340 @@ void mm2_mdl::Reduce_sub1(iter_mm1al it1, vector<i32s> & idv, fGL * crd, i32u cs
 
 // here we only set charges for terminal residues!!!!!!!
 
+// HOW TO PROPERLY DETERMINE THE STATES???? DEPENDENT ON pH AND/OR GEOMETRY?!?!?!?
+// HOW TO PROPERLY DETERMINE THE STATES???? DEPENDENT ON pH AND/OR GEOMETRY?!?!?!?
+// HOW TO PROPERLY DETERMINE THE STATES???? DEPENDENT ON pH AND/OR GEOMETRY?!?!?!?
+
 void mm2_mdl::UpdateConstraints(void)
 {
+	// first, we calculate energy just to get the sasa values...
+	
+	mm2_eng * eng = new mm2_eng(* this);
+	CopyCRD(this, eng, 0); eng->Compute(0);
+	
+	i32u counter = 0;
 	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
 	{
-		chn_vector[n1].res_vector.front().type[0] = TYPE_CPOS;
-		chn_vector[n1].res_vector.back().type[0] = TYPE_CNEG;
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+				f64 sasa = eng->stored_sasa_values[counter++];
+				chn_vector[n1].res_vector[n2].tmp_sasa[n3] = sasa;
+			}
+		}
+	}
+	
+	delete eng;
+	
+	// initialize the charges...
+	
+	const i32s smoothing = 10;
+	
+	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+	{
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			for (i32u n3 = 0;n3 < 3;n3++) chn_vector[n1].res_vector[n2].tmp_smooth[n3] = NULL;
+			
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+				chn_vector[n1].res_vector[n2].charge[n3] = 0.0;
+				
+				bool is_acid = false; bool is_base = false;
+				f64 pKa = NOT_DEFINED; f64 sasa1 = NOT_DEFINED;
+				GetDefaultDConst(n1, n2, n3, is_acid, is_base, pKa, sasa1);
+				if (!is_acid && !is_base) continue;
+				
+				// pKa = pH + log([HX]/[X-])
+				// log([X-]/[HX]) = pH - pKa = log(x/(1-x))
+				// x = 10^(pH - pKa) / (1 + 10^(pH - pKa))
+				
+				f64 tmp1 = pow(10.0, pH - pKa);
+				f64 tmp2 = tmp1 / (1.0 + tmp1);
+				
+				if (is_acid) chn_vector[n1].res_vector[n2].charge[n3] = -tmp2;
+				if (is_base) chn_vector[n1].res_vector[n2].charge[n3] = 1.0 - tmp2;
+				
+				f64 charge = chn_vector[n1].res_vector[n2].charge[n3];
+				chn_vector[n1].res_vector[n2].tmp_smooth[n3] = new f64[smoothing];
+				for (i32s n4 = 0;n4 < smoothing;n4++)
+				{
+					chn_vector[n1].res_vector[n2].tmp_smooth[n3][n4] = charge;
+				}
+			}
+		}
+	}
+	
+	// the loop...
+	
+//ofstream logf("/tmp/pKa.log", ios::out);
+	for (i32s loop = 0;loop < 250;loop++)
+	{
+		for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+		{
+			for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+			{
+				for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+				{
+					bool is_acid = false; bool is_base = false;
+					f64 pKa = NOT_DEFINED; f64 sasa1 = NOT_DEFINED;
+					GetDefaultDConst(n1, n2, n3, is_acid, is_base, pKa, sasa1);
+					if (!is_acid && !is_base) continue;
+					
+					f64 k1 = pow(10.0, -pKa);
+					f64 dg = -8.31451 * 300.0 * log(k1);
+					
+					f64 delta = 0.0;
+					f64 charge1 = chn_vector[n1].res_vector[n2].charge[n3];
+					fGL * crd1 = chn_vector[n1].res_vector[n2].crd_vector[n3][0].data;
+					for (i32u n4 = 0;n4 < chn_vector.size();n4++)
+					{
+						for (i32u n5 = 0;n5 < chn_vector[n4].res_vector.size();n5++)
+						{
+							for (i32u n6 = 0;n6 < chn_vector[n4].res_vector[n5].natm;n6++)
+							{
+								if (n1 == n4 && n2 == n5 && n3 == n6) continue;
+								if (chn_vector[n4].res_vector[n5].charge[n6] == 0.0) continue;
+								
+								fGL * crd2 = chn_vector[n4].res_vector[n5].crd_vector[n6][0].data;
+								
+								f64 r2 = 0.0;
+								for (i32s n9 = 0;n9 < 3;n9++)
+								{
+									fGL tmp1 = crd1[n9] - crd2[n9];
+									r2 += tmp1 * tmp1;
+								}
+								
+								if (r2 < 0.01) continue;	// always skip if r < 1  !!!
+								f64 r1 = sqrt(r2);
+								
+					if (r1 > 1.5) continue;		// also skip large distances; charge neutralization???
+								
+								// e = A + B/(1+k*exp(L*B*r))
+								// de/dr = -(exp(L*B*r)*B^2*k*L)/(1+k*exp(L*B*r))^2
+								
+							f64 t7a = exp(prm.screen2 * prm.screen4 * r1); 	f64 t7b = 1.0 + prm.screen3 * t7a;
+							f64 t7d = prm.screen1 + prm.screen2 / t7b;
+								
+								// f = Q/(e*r)
+								// df/dr = -Q * ((1/e*r^2) + (de/dr)/(e^2*r))
+								
+								f64 qq = 138.9354518 * charge1 * chn_vector[n4].res_vector[n5].charge[n6];
+								delta += qq / (t7d * r1);
+							}
+						}
+					}
+					
+					// here "cc" is a coupling constant used to stabilize the fit...
+					// here "cc" is a coupling constant used to stabilize the fit...
+					// here "cc" is a coupling constant used to stabilize the fit...
+					
+					f64 cc = loop / 100.0;		// this is for 250 steps!!!
+					const f64 limit = 1.00; if (cc > limit) cc = limit;
+					
+					if (is_acid) dg += cc * prm.charge_wes * delta * 1000.0;	// plus!!!
+					if (is_base) dg -= cc * prm.charge_wes * delta * 1000.0;	// minus!!!
+					
+					f64 tmpA = chn_vector[n1].res_vector[n2].tmp_sasa[n3] - sasa1;
+		//	if (tmpA < 0.0) tmpA = 0.0; else tmpA = tmpA * tmpA;		// square?
+					if (tmpA < 0.0) tmpA /= 2.0;	// enhance the "burial" effect???
+					f64 tmpB = tmpA * (prm.charge_sasa2 + prm.charge_sasa3 * charge1 * charge1);
+					if (is_acid) dg -= cc * tmpB * 1000.0;		// minus?
+					if (is_base) dg += cc * tmpB * 1000.0;		// plus?
+					
+					f64 k2 = exp(-dg / (8.31451 * 300.0));
+					pKa = -log10(k2);
+					
+	chn_vector[n1].res_vector[n2].tmp_z1[n3] = delta;
+	chn_vector[n1].res_vector[n2].tmp_z2[n3] = chn_vector[n1].res_vector[n2].tmp_sasa[n3];
+					
+					f64 tmp1 = pow(10.0, pH - pKa);
+					f64 tmp2 = tmp1 / (1.0 + tmp1);
+					
+					if (is_acid) chn_vector[n1].res_vector[n2].tmp_charge[n3] = -tmp2;
+					if (is_base) chn_vector[n1].res_vector[n2].tmp_charge[n3] = 1.0 - tmp2;
+					
+					chn_vector[n1].res_vector[n2].tmp_newpKa[n3] = pKa;	// do these oscillate as well???
+				}
+			}
+		}
+		
+		f64 delta = 0.0;
+		
+		for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+		{
+			for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+			{
+				for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+				{
+					bool is_acid = false; bool is_base = false;
+					f64 pKa = NOT_DEFINED; f64 sasa1 = NOT_DEFINED;
+					GetDefaultDConst(n1, n2, n3, is_acid, is_base, pKa, sasa1);
+					if (!is_acid && !is_base) continue;	// obs!!!
+					
+					f64 ch1 = chn_vector[n1].res_vector[n2].charge[n3];
+					f64 ch2 = chn_vector[n1].res_vector[n2].tmp_charge[n3];
+					
+					for (i32u n4 = 0;n4 < smoothing - 1;n4++)
+					{
+						f64 tmp1 = chn_vector[n1].res_vector[n2].tmp_smooth[n3][n4 + 1];
+						chn_vector[n1].res_vector[n2].tmp_smooth[n3][n4 + 0] = tmp1;
+					}
+					
+					chn_vector[n1].res_vector[n2].tmp_smooth[n3][smoothing - 1] = ch2;
+					
+					ch2 = 0.0;	// now do the smoothing by averaging the array...
+					for (i32s n4 = 0;n4 < smoothing;n4++) ch2 += chn_vector[n1].res_vector[n2].tmp_smooth[n3][n4];
+					ch2 /= (f64) smoothing;
+					
+					f64 dch = ch2 - ch1;
+					delta += dch * dch;
+					
+					chn_vector[n1].res_vector[n2].charge[n3] = ch2;
+//logf << ch2 << " ";
+				}
+			}
+		}
+//logf << endl;
+		
+		cout << "cycle = " << loop << " delta = " << delta << endl;
+//		if ((loop % 50) == 49) { int qqq; cin >> qqq; }
+	}
+//logf.close();
+	
+	// report the results briefly...
+	
+	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+	{
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+				chn_vector[n1].res_vector[n2].label[n3] = 0.0;		// LABELS!!!
+				
+				bool is_acid = false; bool is_base = false;
+				f64 pKa = NOT_DEFINED; f64 sasa1 = NOT_DEFINED;
+				GetDefaultDConst(n1, n2, n3, is_acid, is_base, pKa, sasa1);
+				if (!is_acid && !is_base) continue;
+				
+				f64 dpKa = chn_vector[n1].res_vector[n2].tmp_newpKa[n3] - pKa;
+				f64 charge = chn_vector[n1].res_vector[n2].charge[n3];
+
+				chn_vector[n1].res_vector[n2].label[n3] = charge;	// LABELS!!!
+				
+				cout << "chn " << n1 << " res " << n2 << " " << chn_vector[n1].res_vector[n2].symbol << " ";
+				cout << "pKa = " << chn_vector[n1].res_vector[n2].tmp_newpKa[n3] << " (shift = " << dpKa << ") ";
+				cout << "charge = " << charge << endl;
+				
+			}
+			
+			for (i32u n3 = 0;n3 < 3;n3++)
+			{
+				if (!chn_vector[n1].res_vector[n2].tmp_smooth[n3]) continue;
+				delete[] chn_vector[n1].res_vector[n2].tmp_smooth[n3];
+			}
+			
+			// finally, update the sasa values as well since they are charge-related!!!
+			// finally, update the sasa values as well since they are charge-related!!!
+			// finally, update the sasa values as well since they are charge-related!!!
+			
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+				const char symbols[20 + 1] = "ARNDCQEGHILKMFPSTWYV";
+				
+				const i32s ntypes = 37;
+				const i32s types[ntypes] =
+				{
+					0x0100,
+					0x0200, 0x0201, 0x0202,
+					0x0300, 0x0301,
+					0x0400, 0x0401,
+					0x0500, 0x0501,
+					0x0600, 0x0601,
+					0x0700, 0x0701,
+					0x0800,
+					0x0900, 0x0901,
+					0x0A00, 0x0A01,
+					0x0B00, 0x0B01,
+					0x0C00, 0x0C01, 0x0C02,
+					0x0D00, 0x0D01,
+					0x0E00, 0x0E01,
+					0x0F00,
+					0x1000,
+					0x1100,
+					0x1200, 0x1201, 0x1202,
+					0x1300, 0x1301,
+					0x1400
+				};
+				
+				char tmp1a = chn_vector[n1].res_vector[n2].symbol;
+				i32s tmp1b = 0; while (tmp1b < 20 && symbols[tmp1b] != tmp1a) tmp1b++;
+				if (tmp1b == 20) { cout << "symbol error #1"; exit(EXIT_FAILURE); }
+				
+				tmp1b++; tmp1b = tmp1b << 8; tmp1b += n3;
+				i32s tmp1c = 0; while (tmp1c < ntypes && types[tmp1c] != tmp1b) tmp1c++;
+				if (tmp1c == ntypes) { cout << "type error #1"; exit(EXIT_FAILURE); }
+				
+				f64 charge = chn_vector[n1].res_vector[n2].charge[n3];
+				f64 sasa = prm.sasa1[0] * (prm.sasa9[tmp1c] + charge * charge * prm.sasa1[1]);
+				
+				chn_vector[n1].res_vector[n2].sasa[n3] = sasa;
+			}
+		}
+	}
+}
+
+void mm2_mdl::GetDefaultDConst(i32u n1, i32u n2, i32u n3, bool & is_acid, bool & is_base, f64 & pKa, f64 & sasa1)
+{
+	i32s index = NOT_DEFINED;
+	
+	if (!n2 && n3 == 0) index = 0;							// n-terminal
+	if ((n2 + 1) == chn_vector[n1].res_vector.size() && n3 == 0) index = 1;		// c-terminal
+	
+	if (chn_vector[n1].res_vector[n2].symbol == 'R' && n3 == 2) index = 2;
+	if (chn_vector[n1].res_vector[n2].symbol == 'D' && n3 == 1) index = 3;
+	if (chn_vector[n1].res_vector[n2].symbol == 'C' && n3 == 1) index = 4;		// cysteine!!!
+	if (chn_vector[n1].res_vector[n2].symbol == 'E' && n3 == 1) index = 5;
+	if (chn_vector[n1].res_vector[n2].symbol == 'H' && n3 == 1) index = 6;
+	if (chn_vector[n1].res_vector[n2].symbol == 'K' && n3 == 2) index = 7;
+	if (chn_vector[n1].res_vector[n2].symbol == 'Y' && n3 == 1) index = 8;
+	
+	// check the cases where cysteine is disulphide-bridged.
+	// check the cases where cysteine is disulphide-bridged.
+	// check the cases where cysteine is disulphide-bridged.
+	
+	if (index == 4)		// test for cysteine, see above...
+	{
+		for (i32u n8 = 0;n8 < dsb_vector.size();n8++)
+		{
+			for (i32u n9 = 0;n9 < 2;n9++)
+			{
+				bool test1 = (dsb_vector[n8].chn[n9] == (i32s) n1);
+				bool test2 = (dsb_vector[n8].res[n9] == (i32s) n2);
+				if (test1 && test2) return;
+			}
+		}
+	}
+	
+	if (index != NOT_DEFINED)
+	{
+		is_acid = prm.charge_acid[index];
+		is_base = !prm.charge_acid[index];
+		
+		pKa = prm.charge_pKa[index];
+		sasa1 = prm.charge_sasa1[index];
 	}
 }
 
 /*##############################################*/
 /*##############################################*/
 
-bool mm2_mdl::ReadStream(istream & istr, bool selected)
+// THIS IS NOT YET READY FOR MULTIPLE CRD-SETS!!!!!!!!!
+// THIS IS NOT YET READY FOR MULTIPLE CRD-SETS!!!!!!!!!
+// THIS IS NOT YET READY FOR MULTIPLE CRD-SETS!!!!!!!!!
+
+bool mm2_mdl::ReadStream(istream & istr, bool selected, bool)
 {
 	i32s ncs; istr >> buffer >> ncs;
 	PushCRDSets(ncs - cs_vector.size());
@@ -545,13 +1057,48 @@ bool mm2_mdl::ReadStream(istream & istr, bool selected)
 	
 	while (istr.peek() != '#')
 	{
-		if (istr.peek() == 'c')		// chn
+		char peek1 = istr.peek();
+		
+		if (peek1 == 'c')		// chn
 		{
 			mm2_chn newchn;
 			chn_vector.push_back(newchn);
 		}
 		
-		if (istr.peek() == 'r')		// res
+		if (peek1 == 'p')		// pdb_???
+		{
+			for (n1 = 0;n1 < 4;n1++) istr.get();
+			char peek2 = istr.peek();
+			
+			if (peek2 == 'c')		// pdb_chain
+			{
+				istr.getline(buffer, sizeof(buffer), '\'');
+				chn_vector.back().pdb_chn_id = istr.get();
+			}
+			
+			if (peek2 == 's')		// pdb_seqres
+			{
+				char seqres[65536];
+				istr >> buffer >> seqres;
+				
+				chn_vector.back().pdb_seqres = new char[strlen(seqres) + 1];
+				strcpy(chn_vector.back().pdb_seqres, seqres);
+			}
+		
+			if (peek2 == 'm')		// pdb_missing
+			{
+				istr >> buffer >> n1;
+				istr.getline(buffer, sizeof(buffer), ':');
+				
+				for (n2 = 0;n2 < n1;n2++)
+				{
+					istr >> n3;
+					chn_vector.back().pdb_missing_residues.push_back(n3);
+				}
+			}
+		}
+		
+		if (peek1 == 'r')		// res
 		{
 			istr >> buffer >> n1 >> tmp1 >> tmp2 >> tmp3;
 			istr.getline(buffer, sizeof(buffer));
@@ -583,7 +1130,7 @@ bool mm2_mdl::ReadStream(istream & istr, bool selected)
 			chn_vector.back().res_vector.push_back(newres);
 		}
 		
-		if (istr.peek() == 'B')		// bridge
+		if (peek1 == 'B')		// bridge
 		{
 			mm2_dsb newdsb; istr.get();
 			istr >> n1 >> n2 >> n3 >> n4;
@@ -610,6 +1157,18 @@ void mm2_mdl::WriteStream(ostream & ostr)
 	{
 		ostr << "chn " << n1 << endl;
 		
+		if (chn_vector[n1].pdb_seqres != NULL)
+		{
+			ostr << "pdb_chain '" << chn_vector[n1].pdb_chn_id << "' " << endl;
+			ostr << "pdb_seqres " << chn_vector[n1].pdb_seqres << " " << endl;
+			
+			ostr << "pdb_missing " << chn_vector[n1].pdb_missing_residues.size() << " : ";
+			for (i32u n2 = 0;n2 < chn_vector[n1].pdb_missing_residues.size();n2++)
+			{
+				ostr << chn_vector[n1].pdb_missing_residues[n2] << " ";
+			}	ostr << endl;
+		}
+		
 		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
 		{
 			ostr << "res " << n2 << " ";
@@ -670,6 +1229,15 @@ void mm2_mdl::ImportPDB(const char * fn, i32s chain_index)
 void mm2_mdl::DoEnergy(void)
 {
 	mm2_eng * eng = new mm2_eng(* this);
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Calculating Energy ";
+	str1 << "(simplified forcefield";
+	str1 << ", engine = " << "the default engine";
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, eng, 0); eng->Compute(0);
 	
 	char buffer[1024];
@@ -678,38 +1246,72 @@ void mm2_mdl::DoEnergy(void)
 // this will print also the components...
 // this will print also the components...
 // this will print also the components...
-//str << eng->energy_bt1 << " ";
-//str << eng->energy_bt2 << " ";
-//str << eng->energy_bt3 << " ";
-//str << eng->energy_bt4 << " ";
-//str << eng->energy_nbt1a << " ";
-//str << eng->energy_nbt1b << " ";
-//str << eng->energy_nbt2a << " ";
-//str << eng->energy_nbt2b << " ";
-//str << eng->energy_nbt3 << endl;
+cout << "ENERGY COMPONENTS:" << endl;
+cout << eng->energy_bt1 << " ";
+cout << eng->energy_bt2 << " ";
+cout << eng->energy_bt3a << " " << eng->energy_bt3b << " ";
+cout << eng->energy_bt4a << " " << eng->energy_bt4b << " ";
+cout << eng->energy_nbt1a << " " << eng->energy_nbt1b << " " << eng->energy_nbt1c << " ";
+cout << eng->energy_nbt2a << " " << eng->energy_nbt2b << " " << eng->energy_nbt2c << " ";
+cout << eng->energy_nbt3a << " " << eng->energy_nbt3b << " ";
+cout << eng->energy_pnlt << endl;
 
 	str << "Energy = " << eng->energy << " kJ/mol + " << eng->constraints << " kJ/mol = ";
-	str << (eng->energy + eng->constraints) << " kJ/mol" << ends;
+	str << (eng->energy + eng->constraints) << " kJ/mol" << endl << ends;
 	
-	err->Message(buffer);
+	PrintToLog(buffer);
 	delete eng;
 }
 
 void mm2_mdl::DoGeomOpt(mm2_geomopt_param & param)
 {
-  char buffer[1024];
 	GeomOptGetParam(param);
 	if (!param.confirm) return;
 	
-	mm2_eng * eng = new mm2_eng(* this); CopyCRD(this, eng, 0);
+	mm2_eng * eng = new mm2_eng(* this);
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Starting Geometry Optimization ";
+	str1 << "(simplified forcefield";
+	str1 << ", engine = " << "the default engine";
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
+	CopyCRD(this, eng, 0);
 	mm2_geomopt * opt = new mm2_geomopt(eng, 50, 0.025);	// optimal settings?!?!?
 	
+//eng->SetPenaltyData();	// save the template...
+//eng->penalty_weight = ???;	// use the penalty function...
+
+	char buffer[1024];
+	f64  last_energy = 0.0;		// this is for output and delta_e test...
+
+	PrintToLog("Cycle    Energy    Gradient       Step        Delta E\n");
+
 	i32s n1 = 0;	// n1 counts the number of steps...
 	while (true)
 	{
 		opt->TakeCGStep(conjugate_gradient::Newton2An);
-		sprintf(buffer, "%d %10.3f  %4.6e", n1, opt->optval, opt->optstp);
-		cout << buffer << endl;
+		
+// problem: the gradient information is in fact not precise in this stage. the current gradient
+// is the one that was last calculated in the search, and it is not necessarily the best one.
+// to update the gradient, we need to Compute(1) here...
+	eng->Compute(1);	// this is not vital, but will update the gradient vector length...
+	
+		if (n1 != 0)
+		  {
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e \n", n1,
+			    opt->optval, eng->GetGradientVectorLength(),
+			    opt->optstp, last_energy - opt->optval);
+		  }
+		else
+		  {
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e ********** \n", n1,
+			    opt->optval, eng->GetGradientVectorLength(),
+			    opt->optstp);
+		  }
+		PrintToLog(buffer);
 		
 		bool terminate = false;
 		
@@ -718,7 +1320,7 @@ void mm2_mdl::DoGeomOpt(mm2_geomopt_param & param)
 			if (n1 >= param.treshold_nsteps)
 			{
 				terminate = true;
-				cout << "the nsteps termination test was passed." << endl;
+				PrintToLog("the nsteps termination test was passed.\n");
 			}
 		}
 		
@@ -746,23 +1348,40 @@ void mm2_mdl::GeomOptGetParam(mm2_geomopt_param & param)
 	param.confirm = true;
 }
 
-#define CFRQ 1
-#define CSIZE 100
+#define START 15000
+#define S_TC_NSTEPS 100
+#define S_TC_COUPLING 1.0
 
-#define START 10000
+#define EQUIL 5000
+#define E_TC_NSTEPS 250
+#define E_TC_COUPLING 0.25
 
 void mm2_mdl::DoMolDyn(mm2_moldyn_param & param)
 {
 	MolDynGetParam(param);
 	if (!param.confirm) return;
 	
-	mm2_eng * eng = new mm2_eng(* this); CopyCRD(this, eng, 0);
+	mm2_eng * eng = new mm2_eng(* this);
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Starting Molecular Dynamics ";
+	str1 << "(simplified forcefield";
+	str1 << ", engine = " << "the default engine";
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
+	CopyCRD(this, eng, 0);
+	
 	mm2_moldyn * dyn = new mm2_moldyn(eng, param.temperature, param.timestep);
 	
-// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
-// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
-// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
+//eng->SetPenaltyData();		// save the template...
+//eng->penalty_weight = 0.0;	// use the penalty function...
 
+	// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
+	// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
+	// WHAT IS THE LARGEST STABLE INTEGRATION TIME STEP !?!?!??!
+	
 	ofstream ofile;
 	ofile.open(param.filename, ios::out | ios::binary);
 	
@@ -774,38 +1393,66 @@ void mm2_mdl::DoMolDyn(mm2_moldyn_param & param)
 	ofile.write((char *) & number_of_atoms, sizeof(number_of_atoms));	// number of atoms, int.
 	ofile.write((char *) & total_frames, sizeof(total_frames));		// total number of frames, int.
 	
-	i32s ccnt = 0; f64 control[CSIZE];
+	i32s ccount = 0; f64 control = 0.0;
 	
-	for (i32s n1 = 0;n1 < START + param.nsteps;n1++)
+	for (i32s n1 = 0;n1 < START + EQUIL + param.nsteps;n1++)
 	{
 		dyn->TakeMDStep();
 		if (!(n1 % 50))		// 250
 		{
-			cout << n1 << " T = " << dyn->ConvEKinTemp(dyn->kin);
-			cout << " Ep = " << dyn->pot << " Et = " << (dyn->kin + dyn->pot) << endl;
+			ostrstream str2(mbuff1, sizeof(mbuff1));
+			str2 << "step " << n1 << " T = " << dyn->ConvEKinTemp(dyn->kin);
+			str2 << " Epot = " << dyn->pot << " Etot = " << (dyn->kin + dyn->pot) << endl << ends;
+			PrintToLog(mbuff1);
 		}
 		
-		if (n1 < START)
+	//	if (n1 < START + EQUIL)		// remove this to enable TC in simulation...
 		{
-			if (!(n1 % CFRQ))
-			{
-				control[ccnt++] = dyn->kin;
-			}
+			ccount++; control += dyn->kin;
 			
-			if (ccnt == CSIZE)
+			bool flag = false;
+			if (n1 < START && ccount == S_TC_NSTEPS) flag = true;
+			else if (ccount == E_TC_NSTEPS) flag = true;
+			
+			if (flag)
 			{
-				f64 sum = 0.0;
-				for (i32s n9 = 0;n9 < ccnt;n9++)
+				f64 ekin = control / (f64) ccount;
+				
+				ostrstream str2(mbuff1, sizeof(mbuff1));
+				str2 << "average Ekin = " << ekin << " kJ/mol = " << dyn->ConvEKinTemp(ekin) << " K." << endl << ends;
+				PrintToLog(mbuff1);
+				
+				f64 coupling = E_TC_COUPLING; f64 rtemp = 1.0;
+				if (n1 < START)
 				{
-					sum += control[n9];
+					coupling = S_TC_COUPLING;
+					rtemp = (f64) (n1 + 1) / (f64) START;
 				}
 				
-				sum /= (f64) ccnt; ccnt = 0; f64 rtemp = (f64) (n1 + CFRQ) / (f64) START;
-				cout << "average = " << sum << " kJ/mol = " << dyn->ConvEKinTemp(sum) << " K." << endl;
-				dyn->SetEKin(dyn->kin + (dyn->ConvTempEKin(dyn->temp * rtemp) - sum));
+				dyn->SetEKin(dyn->kin + coupling * (dyn->ConvTempEKin(dyn->temp * rtemp) - ekin));
+				
+				ccount = 0; control = 0.0;
 			}
 		}
-		else if (!(n1 % 100))
+/*
+if (!(n1 % 10))		// penalty function adjustment...
+{
+	f64 rms = sqrt(eng->penalty_sum2 / number_of_atoms);
+	cout << "RMS = " << rms << "   ";
+	
+	f64 e1 = eng->penalty_weight * eng->penalty_sum2;
+	
+	eng->penalty_weight += 2.5 * (rms - 0.2);
+	if (eng->penalty_weight < 0.0) eng->penalty_weight = 0.0;
+	cout << "penalty_weight = " << eng->penalty_weight << endl;
+	
+	f64 e2 = eng->penalty_weight * eng->penalty_sum2;
+	
+	f64 new_ekin = dyn->kin - (e2 - e1);
+	dyn->SetEKin(new_ekin < 0.0 ? 0.0 : new_ekin);
+}
+*/
+		if (!(n1 < START + EQUIL) && !(n1 % 100))
 		{
 			CopyCRD(eng, this, 0);
 			
@@ -826,6 +1473,9 @@ void mm2_mdl::DoMolDyn(mm2_moldyn_param & param)
 							float t1a = chn_vector[t1].res_vector[t2].crd_vector[t3][0][t4];
 							ofile.write((char *) & t1a, sizeof(t1a));
 						}
+						
+						float t1a = chn_vector[t1].res_vector[t2].label[t3];	// PENALTYDEBUG!!!
+						ofile.write((char *) & t1a, sizeof(t1a));
 					}
 				}
 			}
@@ -865,16 +1515,19 @@ void mm2_mdl::OpenTrajectory(const char * fn)
 		
 		if (natoms != GetVirtAtomCount())
 		{
-			cout << "incompatible file : different number of atoms!" << endl;
+			PrintToLog("incompatible file : different number of atoms!\n");
 			CloseTrajectory(); return;
 		}
 		
 		trajfile->read((char *) & total_traj_frames, sizeof(total_traj_frames));
-		cout << "the trajectory file contains " << total_traj_frames << " frames." << endl;
+
+		char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "the trajectory file contains " << total_traj_frames << " frames." << endl << ends;
+		PrintToLog(mbuff1);
 		
 		current_traj_frame = 0;
 	}
-	else cout << "trajectory already exists!" << endl;
+	else PrintToLog("trajectory already exists!\n");
 }
 
 void mm2_mdl::CloseTrajectory(void)
@@ -893,7 +1546,8 @@ void mm2_mdl::ReadFrame(void)
 	i32s atoms = GetVirtAtomCount();
 	
 	i32s place = 8 + 2 * sizeof(int);					// skip the header...
-	place += (2 + 3 * atoms) * sizeof(float) * current_traj_frame;		// get the correct frame...
+//	place += (2 + 3 * atoms) * sizeof(float) * current_traj_frame;		// get the correct frame... ORIGINAL!!!
+	place += (2 + 4 * atoms) * sizeof(float) * current_traj_frame;		// get the correct frame... PENALTYDEBUG!!!
 	place += 2 * sizeof(float);						// skip epot and ekin...
 	
 	trajfile->seekg(place, ios::beg);
@@ -909,6 +1563,9 @@ void mm2_mdl::ReadFrame(void)
 					float t1a; trajfile->read((char *) & t1a, sizeof(t1a));
 					chn_vector[t1].res_vector[t2].crd_vector[t3][0][t4] = t1a;
 				}
+				
+				float t1a; trajfile->read((char *) & t1a, sizeof(t1a));		// PENALTYDEBUG!!!
+				chn_vector[t1].res_vector[t2].label[t3] = t1a;
 			}
 		}
 	}
@@ -938,10 +1595,8 @@ fGL mm2_GetESPValue(fGL * crd, mm2_mdl * ref, fGL * grad)
 		{
 			for (i32u atm = 0;atm < ref->chn_vector[chn].res_vector[res].natm;atm++)
 			{
-				fGL charge;
-				if (ref->chn_vector[chn].res_vector[res].type[atm] == TYPE_CPOS) charge = +1.0;
-				else if (ref->chn_vector[chn].res_vector[res].type[atm] == TYPE_CNEG) charge = -1.0;
-				else continue;
+				fGL charge = ref->chn_vector[chn].res_vector[res].charge[atm];
+				if (charge == 0.0) continue;
 				
 				fGL tmp1[3]; fGL r2 = 0.0;
 				for (i32s n1 = 0;n1 < 3;n1++)
@@ -959,11 +1614,10 @@ fGL mm2_GetESPValue(fGL * crd, mm2_mdl * ref, fGL * grad)
 	// e = A + B/(1+k*exp(L*B*r))
 	// de/dr = -(exp(L*B*r)*B^2*k*L)/(1+k*exp(L*B*r))^2
 	
-	f64 t7a = exp(ref->prm.screen2 * ref->prm.screen4 * r1);
-	f64 t7b = 1.0 + ref->prm.screen3 * t7a; f64 t7c = t7b * t7b;
+	f64 t7a = exp(ref->prm.screen2 * ref->prm.screen4 * r1); f64 t7b = 1.0 + ref->prm.screen3 * t7a;
 	f64 t7d = ref->prm.screen1 + ref->prm.screen2 / t7b;
 	
-	f64 t7e = t7d * t7d;
+	f64 t7c = t7b * t7b; f64 t7e = t7d * t7d;
 	
 	f64 t7f = t7a * ref->prm.screen2 * ref->prm.screen2 * ref->prm.screen3 * ref->prm.screen4;
 	
@@ -1030,11 +1684,10 @@ fGL mm2_GetESPValue(fGL * crd, mm2_mdl * ref, fGL * grad)
 	// e = A + B/(1+k*exp(L*B*r)) + Z/r^9
 	// de/dr = -(exp(L*B*r)*B^2*k*L)/(1+k*exp(L*B*r))^2 - 9Z/r^10
 	
-	f64 t7a = exp(ref->prm.screen2 * ref->prm.screen4 * r1);
-	f64 t7b = 1.0 + ref->prm.screen3 * t7a; f64 t7c = t7b * t7b;
+	f64 t7a = exp(ref->prm.screen2 * ref->prm.screen4 * r1); f64 t7b = 1.0 + ref->prm.screen3 * t7a;
 	f64 t7d = ref->prm.screen1 + ref->prm.screen2 / t7b + ref->prm.screen5 / (t9y * r1);
 	
-	f64 t7e = t7d * t7d;
+	f64 t7c = t7b * t7b; f64 t7e = t7d * t7d;
 	
 	f64 t7f = t7a * ref->prm.screen2 * ref->prm.screen2 * ref->prm.screen3 * ref->prm.screen4;
 	f64 t7g = 9.0 * ref->prm.screen5 / (t9y * r2);
