@@ -1,4 +1,4 @@
-// MM2MDL.H : data storage for reduced protein models (molecular mechanics).
+// MM2MDL.H : data storage for simplified protein models (molecular mechanics).
 
 // Copyright (C) 1998 Tommi Hassinen.
 
@@ -35,28 +35,35 @@ using namespace std;
 
 struct mm2_eng_param
 {
-	f64 wtmp1;
-	f64 wtmp2;
-	f64 wtmp3;
-	
-	f64 wtor1;
-	f64 wtor2;
-	f64 lenjon;
 	f64 vdwrad;
+	f64 lenjon;
+	f64 wescc;
+	f64 wescd;
+	f64 dipole1;
+	f64 dipole2;
 	f64 screen1;
 	f64 screen2;
 	f64 screen3;
 	f64 screen4;
 	f64 screen5;
-	f64 dipole1;
-	f64 dipole2;
-	f64 pbility1;
-	f64 pbility2;
-	f64 sasa1;
-	f64 sasa2;
+	f64 sasa1[2];
+	f64 sasa2[2];
 	f64 solvrad;
+	f64 wtor1;
+	f64 wtor2;
+	f64 wrep;
+
+	f64 sasa9[37];
 	
-	f64 rms;
+	f64 charge_wes;
+	f64 charge_sasa2;
+	f64 charge_sasa3;
+	
+	f64 charge_pKa[9];
+	f64 charge_sasa1[9];
+	bool charge_acid[9];
+	
+	f64 rms;	// for random search only... ERROR???
 	
 	bool operator<(const mm2_eng_param & p1) const
 	{
@@ -80,6 +87,8 @@ class mm2_mdl : public model_extended, public trajectory_interface
 
 	mm2_eng_param prm;
 	
+	fGL pH;
+	
 	friend class mm2_eng;
 	friend class mm2_superimpose;
 	friend class mm2_ribbon;
@@ -112,12 +121,13 @@ class mm2_mdl : public model_extended, public trajectory_interface
 	void Reduce_sub1(iter_mm1al, vector<i32s> &, fGL *, i32u);
 	
 	void UpdateConstraints(void);
-	
+	void GetDefaultDConst(i32u, i32u, i32u, bool &, bool &, f64 &, f64 &);
+
 	// methods for file I/O...
 	// ^^^^^^^^^^^^^^^^^^^^^^^
 	
-	bool ReadStream(istream &, bool = false);	// virtual
-	void WriteStream(ostream &);			// virtual
+	bool ReadStream(istream &, bool, bool = false);		// virtual
+	void WriteStream(ostream &);				// virtual
 	
 /// someday we might move ImportFile() to all_atoms_interface, but at the moment we just have a dummy function here...
 	bool ImportFile(const char*, int) { return false; }	// virtual
