@@ -37,16 +37,14 @@ model_simple::~model_simple(void)
 const char * model_simple::GetFullPrimaryFN(const char * fn)
 {
 	ostrstream str(local_fn_buffer, sizeof(local_fn_buffer));
-	str << PRIMARY_PATH << VERSION << DIR_SEPARATOR 
-	    << "parameters" << DIR_SEPARATOR << fn << ends;
+	str << PRIMARY_PATH << VERSION << DIR_SEPARATOR << fn << ends;
 	return local_fn_buffer;
 }
 
-const char * model_simple::GetFullSecondaryFN(const char * fn)
+const char * model_simple::GetFullSecondaryFN(const char * srcdir, const char * fn)
 {
 	ostrstream str(local_fn_buffer, sizeof(local_fn_buffer));
-	str << SECONDARY_PATH << "parameters" 
-	    << DIR_SEPARATOR << fn << ends;
+	str << SECONDARY_PATH << srcdir << fn << ends;
 	return local_fn_buffer;
 }
 
@@ -59,8 +57,8 @@ void model_simple::OpenParameterFile(ifstream & file, bool is_binary_file, const
 	file.close();
 	cout << "warning: primary path failed for " << model_simple::GetFullPrimaryFN(fn) << endl;
 	
-	if (!is_binary_file) file.open(model_simple::GetFullSecondaryFN(fn), ios::in);
-	else file.open(model_simple::GetFullSecondaryFN(fn), ios::in | ios::binary);
+	if (!is_binary_file) file.open(model_simple::GetFullSecondaryFN("", fn), ios::in);
+	else file.open(model_simple::GetFullSecondaryFN("", fn), ios::in | ios::binary);
 	if (file.good()) return;
 	
 	file.close();
@@ -73,13 +71,16 @@ void model_simple::OpenParameterFile(ifstream & file, bool is_binary_file, const
 
 model_extended::model_extended(ostream * p1, class_factory & p2) : model_simple(p1, p2)
 {
-//	cs_vector.push_back();			// this should work...
-	cs_vector.push_back(crd_set());		// ...but this is safer?!?!?!
-	cs_vector[0].visible = true;
+	cs_vector.push_back(new crd_set());
+	SetCRDSetVisible(0, true);
 }
 
 model_extended::~model_extended(void)
 {
+	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
+	{
+		delete cs_vector[n1];
+	}
 }
 
 void model_extended::UpdateAccumValues(void)		// or NormalizeAccumValues???
@@ -87,18 +88,18 @@ void model_extended::UpdateAccumValues(void)		// or NormalizeAccumValues???
 	fGL sum = 0.0;
 	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
 	{
-		if (cs_vector[n1].visible)
+		if (GetCRDSetVisible(n1))
 		{
-			sum += cs_vector[n1].accum_weight;
+			sum += cs_vector[n1]->accum_weight;
 		}
 	}
 	
 	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
 	{
-		if (cs_vector[n1].visible)
+		if (GetCRDSetVisible(n1))
 		{
-			fGL tmp1 = cs_vector[n1].accum_weight / sum;
-			cs_vector[n1].accum_value = tmp1;
+			fGL tmp1 = cs_vector[n1]->accum_weight / sum;
+			cs_vector[n1]->accum_value = tmp1;
 		}
 	}
 }
@@ -108,21 +109,37 @@ i32u model_extended::GetCRDSetCount(void)
 	return cs_vector.size();
 }
 
-bool model_extended::SetCRDSetVisible(i32u index, bool visible)
+bool model_extended::GetCRDSetVisible(i32u index)
 {
 	if (index < cs_vector.size())
 	{
-		cs_vector[index].visible = visible;
-		return true;
+		return cs_vector[index]->visible;
+	}
+	else
+	{
+		cout << "invalid call to model_extended::GetCRDSetVisible()" << endl;
+		exit(EXIT_FAILURE);
+	}
+}
+
+void model_extended::SetCRDSetVisible(i32u index, bool visible)
+{
+	if (index < cs_vector.size())
+	{
+		cs_vector[index]->visible = visible;
+	}
+	else
+	{
+		cout << "invalid call to model_extended::SetCRDSetVisible()" << endl;
+		exit(EXIT_FAILURE);
 	}
-	else return false;
 }
 
 /*################################################################################################*/
 
 crd_set::crd_set(void)
 {
-	SetDescription(NULL);
+	description = NULL;
 	
 	accum_weight = 1.0;		// accum_value is not updated here?!?!?!?!?!
 	visible = false;
@@ -130,6 +147,7 @@ crd_set::crd_set(void)
 
 crd_set::crd_set(const crd_set & p1)
 {
+	description = NULL;
 	SetDescription(p1.description);
 	
 	accum_weight = p1.accum_weight;
@@ -144,6 +162,8 @@ crd_set::~crd_set(void)
 
 void crd_set::SetDescription(const char * p1)
 {
+	if (description != NULL) delete[] description;
+	
 	if (p1 != NULL)
 	{
 		description = new char[strlen(p1) + 1];
