@@ -27,6 +27,7 @@ class superimpose;
 
 #include <vector>
 #include <fstream>
+#include <iostream>
 using namespace std;
 
 /*################################################################################################*/
@@ -64,23 +65,20 @@ class model_simple
 {
 	public:
 	
-	/**	A general-purpose output stream; 
-		if this is NULL then no output is allowed/needed.
-	*/
-	
+/**	A general-purpose output stream; 
+	if this is NULL then no output is allowed/needed.
+*/
 	ostream * ostr;
 	
-	/// A general-purpose text buffer.
-	
+/// A general-purpose text buffer.
 	char buffer[1024];
 	
-	/**	A newer system, which should eventually replace "##ostr" in user interaction. 
-		But what about logfiles? Do we need them???
-		(GRH: The err_util class could optionally direct output to a logfile as well)
-	*/
-	
+/**	A newer system, which should eventually replace "##ostr" in user interaction. 
+	But what about logfiles? Do we need them???
+	(GRH: The err_util class could optionally direct output to a logfile as well)
+*/
 	err_util * err;
-	prefs	 * model_prefs;
+	prefs * model_prefs;
 	
 	protected:
 	
@@ -96,29 +94,26 @@ class model_simple
 	virtual ~model_simple(void);
 	
 	static const char * GetFullPrimaryFN(const char *);
-	static const char * GetFullSecondaryFN(const char *);
+	static const char * GetFullSecondaryFN(const char *, const char *);
 	
 	static void OpenParameterFile(ifstream &, bool, const char *);
 	
 	virtual const char * GetProjectFileNameExtension(void) = 0;
 	
-	/// This will read the model from a file.
-	// (input stream, flag to leave the molecule selected)
-	virtual bool ReadStream(istream &, bool = false) = 0;
-	
-	/// This will write the model into a file.
+/// This will read the model from a file; (input stream, flag to leave the molecule selected).
+	virtual bool ReadStream(istream &, bool, bool = false) = 0;
 	
+/// This will write the model into a file.
 	virtual void WriteStream(ostream &) = 0;
 	
-	/// This will import other filetypes into a file
-	// (the "index" parameter defines automatic (0)
-	//  or offsets based on the filetrans class
-	
+/// This will import other filetypes into a file; (the "index" parameter defines automatic (0) or offsets based on the filetrans class.
 	virtual bool ImportFile(const char *filename, int index = 0) = 0;
 
-	/// The "##docview"-class will override this...
-	
+/// The "##docview"-class will override this...
 	virtual void UpdateAllGraphicsViews(bool = false) { }
+	
+/// Add a message string to the logfile. This is just a default for TARGET2...
+	virtual void PrintToLog(const char * p1) { cout << "PrintToLog: " << p1 << endl; }
 };
 
 /*################################################################################################*/
@@ -131,11 +126,11 @@ class model_extended : virtual public model_simple
 {
 	protected:
 	
-	vector<crd_set> cs_vector;
+	vector<crd_set *> cs_vector;
 	
-	// this is needed for access to cs_vector...
-	// is there any easier way?!?!?! a read-access function perhaps???
-	// usually only "default" set is needed, and rest is ignored...
+// this is needed for access to cs_vector...
+// is there any easier way?!?!?! a read-access function perhaps???
+// usually only "default" set is needed, and rest is ignored...
 	friend class mm1_sequencebuilder;
 	
 	public:
@@ -145,12 +140,13 @@ class model_extended : virtual public model_simple
 	
 	void UpdateAccumValues(void);
 	
-	/**	This will return the number of coordinate sets. 
-		It is supposed that at least one coordinate set exists all the time!!!
-	*/
-	
+/**	This will return the number of coordinate sets. 
+	It is supposed that at least one coordinate set exists all the time!!!
+*/
 	i32u GetCRDSetCount(void);
-	bool SetCRDSetVisible(i32u, bool);
+	
+	bool GetCRDSetVisible(i32u);
+	void SetCRDSetVisible(i32u, bool);
 	
 	virtual void PushCRDSets(i32u) = 0;
 	virtual void PopCRDSets(i32u) = 0;
