@@ -12,6 +12,9 @@
 #ifndef NOTICE_H
 #define NOTICE_H
 
+/* added by Robert Williams for Compaq cxx, alpha 11/28/01 */
+#define __USE_STD_IOSTREAM
+
 #include <iostream>
 using namespace std;
 
