@@ -22,21 +22,24 @@ class qm1_geomopt_param;
 
 /*################################################################################################*/
 
-/// A geometry optimization class for MM models.
+/// A geometry optimization class for QM models.
 
 class qm1_geomopt : public conjugate_gradient
 {
 	protected:
 	
 	qm1_eng * eng;
+	f64 * tss_target;
 	
 	public:
 	
-	qm1_geomopt(qm1_eng *, i32s, f64);
+	qm1_geomopt(qm1_eng *, i32s, f64, f64 * = NULL);
 	~qm1_geomopt(void);
 	
 	f64 GetValue(void);		// virtual
 	f64 GetGradient(void);		// virtual
+	
+	static void AddConst(i32s, qm1_eng *, f64 *);
 };
 
 class qm1_geomopt_param
@@ -51,7 +54,14 @@ class qm1_geomopt_param
 	
 	public:
 	
-	i32s nsteps;
+	bool enable_nsteps;
+	i32s treshold_nsteps;
+	
+	bool enable_grad;
+	f64  treshold_grad;
+	
+	bool enable_delta_e;
+	f64  treshold_delta_e;
 	
 	public:
 	
@@ -61,6 +71,32 @@ class qm1_geomopt_param
 
 /*################################################################################################*/
 
+/// A transition state search class...
+
+class qm1_simple_ts_search
+{
+	protected:
+	
+	qm1_mdl * mdl;
+	
+	f64 * target[2];
+	f64 energy[2];
+	
+	public:
+	
+	qm1_simple_ts_search(qm1_mdl *);
+	~qm1_simple_ts_search(void);
+	
+	void UpdateTargets(void);
+	void Run(i32s);
+	
+	protected:
+	
+	void SetTarget(i32s, i32s);
+};
+
+/*################################################################################################*/
+
 #endif	// QM1ALG_H
 
 // eof
