@@ -107,7 +107,7 @@ qm1_mdl::~qm1_mdl(void)
 
 const char * qm1_mdl::GetProjectFileNameExtension(void)
 {
-	static const char ext[] = "qm1gp";
+	static const char ext[] = "gpr";
 	return ext;
 }
 
@@ -255,9 +255,7 @@ void qm1_mdl::SetupPlotting(void)
 
 void qm1_mdl::PushCRDSets(i32u p1)
 {
-//cs_vector.resize(cs_vector.size() + p1);				// more convenient way?!?!?!
-//for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();			// more crash-proof way?!?!?!?
-	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
+	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(new crd_set());
 	
 	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
 	for (iter_qm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
@@ -362,27 +360,20 @@ void qm1_mdl::RemoveBond(iter_qm1bl it1)
 /*##############################################*/
 /*##############################################*/
 
-// also this seems to relate all_atoms_interface???
-// also this seems to relate all_atoms_interface???
-// also this seems to relate all_atoms_interface???
-
-void qm1_mdl::ImportMM1(const char * fn)
+bool qm1_mdl::ReadStream(istream & ifile, bool selected, bool)
 {
 	mm1_mdl * mdl = new mm1_mdl(NULL, (* factory));
+	bool retval = mdl->ReadStream(ifile, selected);
 	
-	ifstream ifile(fn, ios::in);
-	mdl->ReadStream(ifile);
-	ifile.close();
-	
-	// the crd-set handling here might have problems if QM model has more sets than MM model...
-	// the crd-set handling here might have problems if QM model has more sets than MM model...
-	// the crd-set handling here might have problems if QM model has more sets than MM model...
+// the crd-set handling here might have problems if QM model has more sets than MM model...
+// the crd-set handling here might have problems if QM model has more sets than MM model...
+// the crd-set handling here might have problems if QM model has more sets than MM model...
 	
 	i32s new_crd_sets = mdl->cs_vector.size() - cs_vector.size();
 	if (new_crd_sets > 0)
 	{
 		PushCRDSets(new_crd_sets);
-		for (i32u n1 = 1;n1 < cs_vector.size();n1++) cs_vector[n1].visible = true;
+		for (i32u n1 = 1;n1 < cs_vector.size();n1++) SetCRDSetVisible(n1, true);
 	}
 	
 	vector<mm1_atom *> mmtab;	// store the pointers here...
@@ -428,6 +419,18 @@ void qm1_mdl::ImportMM1(const char * fn)
 	}
 	
 	delete mdl;
+	
+	return retval;
+}
+
+void qm1_mdl::WriteStream(ostream & ostr)
+{
+	mm1_mdl * mdl = new mm1_mdl(NULL, (* factory));
+	
+	mdl->aai_MakeCopy(this);
+	mdl->WriteStream(ostr);
+	
+	delete mdl;
 }
 
 /*##############################################*/
@@ -656,14 +659,22 @@ void qm1_mdl::DoEnergy(void)
 	if (!current_eng) current_eng = CreateDefaultEngine();
 	if (!current_eng) return;	// sanity check failed...
 	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Calculating Energy ";
+	str1 << "(quantum mechanics";
+	str1 << ", engine = " << engtab1[default_eng];
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, current_eng, 0); current_eng->Compute(0);
 	
 	char buffer[128];
 	ostrstream str(buffer, sizeof(buffer)); str.setf(ios::fixed); str.precision(8);
 	
-	str << "Energy = " << current_eng->energy << " kJ/mol" << ends;
+	str << "Energy = " << current_eng->energy << " kJ/mol" << endl << ends;
 	
-	err->Message(buffer);
+	PrintToLog(buffer);
 	
 	// we will not delete current_eng here, so that we can draw plots using it...
 	// we will not delete current_eng here, so that we can draw plots using it...
@@ -679,6 +690,15 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 	
 	if (!current_eng) current_eng = CreateDefaultEngine();
 	if (!current_eng) return;	// sanity check failed...
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Starting Geometry Optimization ";
+	str1 << "(quantum mechanics";
+	str1 << ", engine = " << engtab1[default_eng];
+	str1 << ")." << endl << ends;
+	PrintToLog(mbuff1);
+	
 	CopyCRD(this, current_eng, 0);
 	
 	qm1_geomopt * opt = new qm1_geomopt(current_eng, 20, 0.0125);	// optimal settings?!?!?
@@ -686,8 +706,7 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 	char buffer[1024];
 	f64  last_energy = 0.0;		// this is for output and delta_e test...
 	
-	cout << "Cycle     Energy   Gradient     Step      Delta E" << endl;
-	cout << "--------------------------------------------------" << endl;
+	PrintToLog("Cycle    Energy    Gradient       Step        Delta E\n");
 	
 	i32s n1 = 0;	// n1 counts the number of steps...
 	while (true)
@@ -703,17 +722,17 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 		
 		if (n1 != 0)
 		  {
-		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e", n1,
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e \n", n1,
 			    opt->optval, current_eng->GetGradientVectorLength(),
 			    opt->optstp, last_energy - opt->optval);
 		  }
 		else
 		  {
-		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e **********", n1,
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e ********** \n", n1,
 			    opt->optval, current_eng->GetGradientVectorLength(),
 			    opt->optstp);
 		  }
-		cout << buffer << endl;
+		PrintToLog(buffer);
 		
 		bool terminate = false;
 		
@@ -722,7 +741,7 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 			if (n1 >= param.treshold_nsteps)
 			{
 				terminate = true;
-				cout << "the nsteps termination test was passed." << endl;
+				PrintToLog("the nsteps termination test was passed.\n");
 			}
 		}
 		
@@ -731,7 +750,7 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 			if (current_eng->GetGradientVectorLength() < param.treshold_grad)
 			{
 				terminate = true;
-				cout << "the grad termination test was passed." << endl;
+				PrintToLog("the grad termination test was passed.\n");
 			}
 		}
 		
@@ -744,7 +763,7 @@ void qm1_mdl::DoGeomOpt(qm1_geomopt_param & param)
 			if (flag)
 			{
 				terminate = true;
-				cout << "the delta_e termination test was passed." << endl;
+				PrintToLog("the delta_e termination test was passed.\n");
 			}
 		}
 		
