@@ -165,6 +165,9 @@ class qm1_mdl : public model_extended, public all_atoms_interface
 	friend class qm1_simple_ts_search;
 	
 	friend class enlevdiag_view;
+
+	friend class qm1_file_open_dialog;
+	friend class qm1_file_save_dialog;
 	
 	friend class gnome_mm1_docv;	// temporary, for MM->QM conversion, will move into all_atoms_interface???
 	
@@ -211,15 +214,14 @@ class qm1_mdl : public model_extended, public all_atoms_interface
 	
 	iter_qm1bl GetBondsBegin(void) { return bond_list.begin(); }	//const;
 	iter_qm1bl GetBondsEnd(void) { return bond_list.end(); }	//const;
-	
+
 	// methods for file I/O...
 	// ^^^^^^^^^^^^^^^^^^^^^^^
 	
-	bool ReadStream(istream &, bool = false) { return false; }	// virtual
-	void WriteStream(ostream &) { }					// virtual
+	bool ReadStream(istream &, bool, bool = false);		// virtual
+	void WriteStream(ostream &);				// virtual
 	
-	bool ImportFile(const char*, int) { return false; }		// virtual
-	void ImportMM1(const char *);
+	bool ImportFile(const char*, int) { return false; }	// virtual
 	
 	// these are for glade callbacks etc...
 	
