@@ -231,7 +231,21 @@ const fGL * element::GetColor(prefs *user_prefs) const
 	    char keybuffer[32];
 	    ostrstream fullkey(keybuffer, sizeof(keybuffer));
 	    fullkey << "ElementColor/" << atomic_number << ends;
-	    return user_prefs->ColorRGB(keybuffer, color[atomic_number - 1]);
+	    
+	    // prefs->ColorRGB() returns a pointer that should be deleted.
+	    // however, in other cases we return a pointer that must NOT be deleted.
+	    // therefore we make here a static array that we can return, and copy
+	    // the result of prefs->ColorRGB() there and delete it's return value.
+	    
+	    float * tmpcolor = user_prefs->ColorRGB(keybuffer, color[atomic_number - 1]);
+	    
+	    static fGL user_color[3];
+	    user_color[0] = tmpcolor[0];
+	    user_color[1] = tmpcolor[1];
+	    user_color[2] = tmpcolor[2];
+	    
+	    delete[] tmpcolor;	// delete the array that we got from prefs->ColorRGB()!!!
+	    return user_color;	// return the static array!!!
 	  }
 }
 
