@@ -13,6 +13,9 @@
 #include "docview.h"
 #include "views.h"
 
+//#include "time.h"	// for frames-per-second counter...
+//#include "sys/time.h"	// for frames-per-second counter...
+
 /*################################################################################################*/
 
 camera::camera(const object_location & p1, fGL p2, docview * p3) : dummy_object(p1)
@@ -132,6 +135,30 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 {
 	if (gv->size[0] < 0) return;	// skip rendering if invalid window!!!
 	
+// A SIMPLE FRAMES-PER-SECOND COUNTER FOR GRAPHICS OPTIMIZATION
+// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+/*	const int numframes = 30;
+	static int framecounter = 0;
+	static double timecounter = 0.0;
+	static long prevtime = 0;
+	timeval tv; gettimeofday(& tv, NULL);
+	long currtime = tv.tv_usec;		// according to manpages, this should be in microseconds???
+	long difftime = currtime - prevtime;	// the CLOCKS_PER_SEC factor works a lot better than 1.0e-06
+	prevtime = currtime;
+	if (difftime > 0)	// skip the frame in timer overflow situation!!!
+	{
+		double t_secs = (double) difftime / (double) CLOCKS_PER_SEC;	// time unit???
+		framecounter++; timecounter += t_secs;
+		if (framecounter >= numframes)
+		{
+			cout << "frame rate = " << ((double) framecounter / timecounter) << " FPS   (";
+			cout << framecounter << " frames in " << timecounter << " seconds)." << endl;
+			framecounter = 0; timecounter = 0.0;
+		}
+	}	*/
+// A SIMPLE FRAMES-PER-SECOND COUNTER FOR GRAPHICS OPTIMIZATION
+// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	
@@ -158,11 +185,13 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 	if (accum) { glClear(GL_ACCUM_BUFFER_BIT); /*docv->UpdateAccumValues();*/ }
 	else glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	
-// old non-stereo code starts here... THE NEW RELAXED-EYE HACK IS HERE!!!
-// old non-stereo code starts here... THE NEW RELAXED-EYE HACK IS HERE!!!
-// old non-stereo code starts here... THE NEW RELAXED-EYE HACK IS HERE!!!
-/*
-	i32s width = gv->size[0]; if (stereo_mode) width /= 2;
+	// the stereo-modes related stuff start here...
+	// the stereo-modes related stuff start here...
+	// the stereo-modes related stuff start here...
+	
+	// the stereo modes are implemented as a loop, that is cycled either once or twice.
+	
+	i32s width = gv->size[0]; if (stereo_mode && stereo_relaxed) width /= 2;
 	i32s height = gv->size[1]; i32s shift = 0;
 	
 	fGL aspect = (fGL) width / (fGL) height;
@@ -171,11 +200,11 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 	gv->range[1] = focus * tan(M_PI * fovy / 360.0);
 	gv->range[0] = aspect * gv->range[1];
 	
-	i32s stereo_count = (stereo_relaxed ? 2 : 1);
+	i32s stereo_count = (stereo_mode ? 2 : 1);
 	for (i32s stereo_loop = 0;stereo_loop < stereo_count;stereo_loop++)
 	{
 		glViewport(shift, 0, width, height);
-		shift += width;		// this will shift our viewport right at the next round...
+		if (stereo_mode && stereo_relaxed) shift += width;	// this will shift our viewport right at the next round...
 		
 		if (!ortho) gluPerspective(fovy, aspect, 0.1 * focus, 2.0 * focus);
 		else glOrtho(-gv->range[0], +gv->range[0], -gv->range[1], +gv->range[1], 0.0, 2.0 * focus);
@@ -187,9 +216,9 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 		
 		const fGL * r1; const fGL * r2; const fGL * r3;
 		
-		if (stereo_relaxed == false)
+		if (!stereo_mode)
 		{
-			// if not relaxed-eye stereo mode, then just use normal camera settings...
+			// if not a stereo mode, then just use normal camera settings...
 			
 			r1 = loc1->crd;		// eye coordinates
 			r2 = target.data;	// target coordinates
@@ -197,7 +226,7 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 		}
 		else
 		{
-			// for relaxed-eye stereo mode, translate the camera along camera's x-axis.
+			// for all stereo modes, translate the camera along camera's x-axis.
 			// y-direction will remain constant, but other directions won't. does it matter???
 			
 			fGL displacement = rang / 20.0;
@@ -216,11 +245,19 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 			r1 = tmp_crd;		// eye coordinates
 			r2 = target.data;	// target coordinates
 			r3 = loc1->ydir.data;	// y-direction vector
+			
+			if (!stereo_relaxed)
+			{
+				// for the red-blue stereo mode, set the color masks...
+				
+				if (!stereo_loop) glColorMask(GL_TRUE,GL_FALSE,GL_FALSE,GL_TRUE);	// left eye - red
+				else glColorMask(GL_FALSE,GL_FALSE,GL_TRUE,GL_TRUE);			// right eye - blue
+			}
 		}
 		
 		gluLookAt(r1[0], r1[1], r1[2], r2[0], r2[1], r2[2], r3[0], r3[1], r3[2]);
 		
-		if (stereo_relaxed == true)
+		if (stereo_mode && stereo_relaxed)
 		{
 			// this will do the other displacement.
 			// i don't understand *how* this works, but it works!  :)  TH
@@ -277,222 +314,17 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 			
 			gv->GetDV()->Render(gv, docview::Normal);
 		}
-	}
-//*/
-// mike's original stereo code starts here...
-// mike's original stereo code starts here...
-// mike's original stereo code starts here...
-///*
-	
-// Declaration of Variables
-	
-	i32s width = gv->size[0]; if (stereo_relaxed) width /= 2;
-	i32s height = gv->size[1]; i32s shift = 0;
-	
-	fGL aspect = (fGL) width / (fGL) height;
-	fGL fovy = (aspect > 1.0 ? 45.0 / aspect : 45.0);
-	gv->range[1] = focus * tan(M_PI * fovy / 360.0);
-	gv->range[0] = aspect * gv->range[1];			
-			
-	const obj_loc_data * loc1 = GetLocData();
-	v3d<fGL> target = v3d<fGL>(loc1->crd) + (loc1->zdir * focus);
-	
-	const fGL * r1; const fGL * r2; const fGL * r3;
-	r1 = loc1->crd;		// eye coordinates
-	r2 = target.data;	// target coordinates
-	r3 = loc1->ydir.data;	// y-direction vector
-	
-// Environment Variables
-	int camfocallength = 30;
-	int camaperture = 20;
-	int eyesep = camfocallength / 20;
-	double eye = 0.05;
-	double eyetrans = 0;
-
-// Red-Green Stereo Variables
-	float ratio = width/height;	
-	if (ratio < 1.0) ratio = 1.0;
-	float radians = 0.0174532925 * camaperture / 2;
-	float near = 0.1 * focus; float far = 20 * focus;
-	float wd2 = near * tan(radians);
-	float ndfl = near / camfocallength; 
-	
-
-// Red-Green Stereo Mode		
-	if (stereo_mode && !stereo_relaxed) {
-		glViewport(0,0,width,height);
-		// Derive eye positions for glLookAt
-		//fGL * r4;
-		float r4x = (r2[1]*r3[2] - r2[2]*r3[1]);
-		float r4y = (r2[2]*r3[0] - r2[0]*r3[2]);
-		float r4z = (r2[0]*r3[1] - r2[1]*r3[0]);
-		//Normalize it
-		double length = sqrt(r4x*r4x+r4y*r4y+r4z*r4z);
-		if (length != 0) {
-			r4x /= length; r4y /= length; r4z /= length;
-		} else {
-			r4x = 0; r4y = 0; r4z = 0;
-		}
-		r4x *= eyesep / 20.0; r4y *= eyesep / 20.0; r4z *= eyesep / 20.0;
 		
-	//	cout << "r1 values: " << r1[0] << " " << r1[1] << " " << r1[2] << endl;
-	//	cout << "r2 values: " << r2[0] << " " << r2[1] << " " << r2[2] << endl;
-	//	cout << "r3 values: " << r3[0] << " " << r3[1] << " " << r3[2] << endl;
-	//	cout << "r4 values: " << r4x << " " << r4y << " " << r4z << endl;
-		
-		for (int rg_count = 0; rg_count < 2; rg_count++) {
-			if (rg_count==0) {
-				cout << "Left Eye - Red" << endl;
-				eyetrans = -eye;
-   			 glColorMask(GL_TRUE,GL_FALSE,GL_FALSE,GL_TRUE);
-   		 } else {
-   		 	cout << "Right Eye - Blue" << endl;
-   		 	eyetrans =  eye;
-			    glColorMask(GL_FALSE,GL_FALSE,GL_TRUE,GL_TRUE);
-			}
-			glMatrixMode(GL_PROJECTION); glLoadIdentity();
-			float left   = -ratio * wd2 + eyetrans * eyesep * ndfl;
-			float right  =  ratio * wd2 + eyetrans * eyesep * ndfl;
-			float top    =  wd2;
-			float bottom = -wd2;
-			glFrustum(left,right,bottom,top,near,far);
-			cout << "Left: " << left << " Right: " << right << endl;
-			cout << "Top: " << top << " Bottom: " << bottom << endl;
-			cout << "Near: " << near << " Far: " << far << endl;
-			glMatrixMode(GL_MODELVIEW); glLoadIdentity();
-			if (rg_count==0) 
-				gluLookAt(r1[0]+r4x, r1[1]+r4y, r1[2]+r4z,r2[0]+r4x,r2[1]+r4y,r2[2]+r4z,
-						  //r1[0]+r4x+r2[0],
-						  //r1[1]+r4y+r2[1],
-						  //r1[2]+r4z+r2[2],
-						  r3[0], r3[1], r3[2]);
-			else gluLookAt(r1[0]-r4x, r1[1]-r4y, r1[2]-r4z,r2[0]-r4x,r2[1]-r4y,r2[2]-r4z,
-						  //r1[0]-r4x+r2[0],
-						  //r1[1]-r4y+r2[1],
-						  //r1[2]-r4z+r2[2],
-						  r3[0], r3[1], r3[2]);
-			
-			//gluLookAt(r1[0],r1[1],r1[2],r2[0],r2[1],r2[2],r3[0],r3[1],r3[2]);
-			// now, it's time to set up the lights...
-			
-			for (i32u n1 = 0;n1 < gv->GetDV()->light_vector.size();n1++)
-			{
-				camera * owner = gv->GetDV()->light_vector[n1]->owner;
-				if (owner != NULL && owner != this) continue;
-				gv->GetDV()->light_vector[n1]->SetupLocation();
-			}
-		
-	// here, we are finally ready to actually render the view. for simple non-stereo views, just erase the background
-	// by calling glClear(), and then call docv->Render() to render the view. For relaxed-eye stereo views, two viewports
-	// are used and the view is therefore rendered twice, calling glClear only once. TODO(???): there is also a hardware
-	// stereo mode (not much supported yet, except SGI) that could be also used; that is a special graphics mode with two
-	// color buffers, and glDrawBuffer(???) is called to determine which is active.
-		
-	// to make things more complicated, there is the "accumulation" effect that relies on glClear() call. to make all this
-	// work together, we must take all glClear() and glAccum() calls out from the docv->Render, and do them here.
-			
-			glInitNames();
-			if (transformer::transform_in_progress)
-			{				
-				gv->GetDV()->Render(gv, docview::Transform1);
-				glPushMatrix();
-				gv->current_tool->ref_to_object->SetModelView();
-				gv->GetDV()->Render(gv, docview::Transform2);
-				glPopMatrix();
-			}
-			else
-			{
-				gv->GetDV()->Render(gv, docview::Normal);
-			}
-			
- 		   glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE); // Reset the Color Mask
-		}
-	} else { // Relaxed Eye Stereo Mode		
-		i32s stereo_count = (stereo_relaxed ? 2 : 1);
-		for (i32s stereo_loop = 0;stereo_loop < stereo_count;stereo_loop++)
+		if (stereo_mode && !stereo_relaxed)
 		{
-			if (!stereo_relaxed) glViewport(0,0,width,height);
-			else 
-			{
-					glViewport(shift, 0, width, height);
-					shift += width;		// this will shift our viewport right at the next round...
-			}
-			
-			if (!ortho) gluPerspective(fovy, aspect, 0.1 * focus, 2.0 * focus);
-			else glOrtho(-gv->range[0], +gv->range[0], -gv->range[1], +gv->range[1], 0.0, 2.0 * focus);
-		
-			//const obj_loc_data * loc1 = GetLocData();
-			
-			glMatrixMode(GL_MODELVIEW); glLoadIdentity();
-		               
-				if (stereo_relaxed)
-				{
-					if (stereo_loop==0)
-					{
-						gluLookAt(r1[0], r1[1], r1[2], r2[0], r2[1], r2[2], r3[0], r3[1], r3[2]);
-						glMatrixMode(GL_PROJECTION); 
-						glTranslatef(-slider/10,0.f,0.f);
-						glMatrixMode(GL_MODELVIEW);
-					}	
-					else
-					{
-						gluLookAt(r1[0], r1[1], r1[2], r2[0], r2[1], r2[2], r3[0], r3[1], r3[2]);
-						glMatrixMode(GL_PROJECTION); 
-						glTranslatef(2*slider/10,0.f,0.f);
-						glMatrixMode(GL_MODELVIEW);
-						glRotatef(rang*20,0,1,0);
-					}
-				}
-				else gluLookAt(r1[0], r1[1], r1[2], r2[0], r2[1], r2[2], r3[0], r3[1], r3[2]);
-				
-			// now, it's time to set up the lights...
-			
-			for (i32u n1 = 0;n1 < gv->GetDV()->light_vector.size();n1++)
-			{
-				camera * owner = gv->GetDV()->light_vector[n1]->owner;
-				if (owner != NULL && owner != this) continue;
-				gv->GetDV()->light_vector[n1]->SetupLocation();
-			}
-		
-	// here, we are finally ready to actually render the view. for simple non-stereo views, just erase the background
-	// by calling glClear(), and then call docv->Render() to render the view. For relaxed-eye stereo views, two viewports
-	// are used and the view is therefore rendered twice, calling glClear only once. TODO(???): there is also a hardware
-	// stereo mode (not much supported yet, except SGI) that could be also used; that is a special graphics mode with two
-	// color buffers, and glDrawBuffer(???) is called to determine which is active.
-
-	// to make things more complicated, there is the "accumulation" effect that relies on glClear() call. to make all this
-	// work together, we must take all glClear() and glAccum() calls out from the docv->Render, and do them here.
-	
-	// MIKE: this "accumulation" stuff is not yet ready (it's used only in gt2 so far), i will look at it later...
-	
-			glInitNames();
-			if (transformer::transform_in_progress)
-			{
-				// do the "transformation" effect: first round renders non-selected atoms/bonds,
-				// and the second one renders the selected ones translated/rotated as told by the mouse tool.
-				
-				// accumulation should be DISABLED here!!!
-				
-				gv->GetDV()->Render(gv, docview::Transform1);
-			
-				glPushMatrix();
-				gv->current_tool->ref_to_object->SetModelView();
-				gv->GetDV()->Render(gv, docview::Transform2);
-				glPopMatrix();
-			}
-			else
-			{
-				// do the normal rendering...
-				
-				gv->GetDV()->Render(gv, docview::Normal);
-			}
+			glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);		// reset the color mask...
 		}
 	}
-//*/
-// end of stereo-stuff...
-// end of stereo-stuff...
-// end of stereo-stuff...
-
+	
+	// end of stereo-modes related stuff...
+	// end of stereo-modes related stuff...
+	// end of stereo-modes related stuff...
+	
 #ifdef TARGET2
 
 	// in TARGET2, print out the tool name and the selected object (if any)...
