@@ -174,8 +174,7 @@ void docview::ParseProjectFileNameAndPath(const char * string)
 	if (lastext != NOT_DEFINED)
 	{
 	  // This only removes an extension if it matches *our* extension,
-	  // which makes problems for imported files
-	  // e.g. nh3.mol.mmg1p (!) instead of nh3.mol or nh3.mm1gp
+	  // which makes problems for imported files e.g. nh3.mol.mmg1p (!) instead of nh3.mol or nh3.mm1gp
 	  //		char * localstring3 = & localstring2[lastext + 1];
 	  //		bool has_extension = !strcmp(localstring3, GetProjectFileNameExtension());
 	  //		if (has_extension) localstring2[lastext] = 0;
@@ -778,6 +777,75 @@ void docview::RenderAllTPs(graphics_view * gv, rmode)
 /*##############################################*/
 /*##############################################*/
 
+bool docview::ProcessCommonCommands(graphics_view * gv, const char * command)
+{
+	istrstream istr(command);
+	char kw1[32]; istr >> kw1;
+	
+	// is it "help"??? if is, print help for common command and return FALSE!!!
+	
+	if (!strcmp("help", kw1))
+	{
+		PrintToLog("> AVAILABLE COMMANDS:\n");
+		
+		PrintToLog("> add light (local/global) (directional/spotlight) -- add a new light object.\n");
+		
+		PrintToLog("> help -- print all available commands in command strings.\n");
+		
+		return false;
+	}
+	
+	// process other common commands.
+	// process other common commands.
+	// process other common commands.
+	
+	char mbuff1[512];
+	
+	if (!strcmp("add", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// the 2nd keyword; type of the object to add.
+		
+		if (!strcmp("light", kw2))
+		{
+			char kw3[32]; istr >> kw3;
+			char kw4[32]; istr >> kw4;
+			
+			bool is_local = true;
+			if (kw3[0] == 'g') is_local = false;
+			
+			bool is_directional = true;
+			if (kw4[0] == 's') is_directional = false;
+			
+			light * new_light;
+			if (is_directional) new_light = new directional_light(ol_static());
+			else
+			{
+				new_light = new spot_light(ol_static());
+				
+				const fGL trans[3] = { 0.0, 0.0, -1.0 };
+				new_light->TranslateObject((const fGL *) trans, gv->cam->GetLocData());
+			}
+			
+			if (!is_local) AddGlobalLight(new_light);
+			else AddLocalLight(new_light, gv->cam);
+			
+			if (!is_local || graphics_view::draw_info) UpdateAllGraphicsViews();
+			else if (is_local) UpdateGraphicsViews(gv->cam);
+			
+			ostrstream strR(mbuff1, sizeof(mbuff1));
+			strR << "Added a new object : light (";
+			strR << (is_local ? "local" : "global") << " ";
+			strR << (is_directional ? "directional" : "spotlight") << ")." << endl << ends;
+			PrintToLog(mbuff1);
+			return true;
+		}
+		
+		return false;	// FALSE -> not all "add" commands are common...
+	}
+	
+	return false;	// no success, must be a specific command.
+}
+
 void docview::DoDeleteCurrentObject(void)
 {
 	if (selected_object != NULL)
