@@ -230,6 +230,9 @@ class docview : virtual public model_simple
 	
 	// some common operations are implemented here:
 	
+	virtual void ProcessCommandString(graphics_view *, const char *) = 0;
+	bool ProcessCommonCommands(graphics_view *, const char *);
+	
 	void DoDeleteCurrentObject(void);
 	
 	void DoSwitchLocalLights(camera *, bool);
