@@ -328,7 +328,7 @@ void mm1_docv::RenderScene(graphics_view * gv, rmode rm, bool accum)
 {
 	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
 	{
-		if (!cs_vector[n1].visible) continue;
+		if (!GetCRDSetVisible(n1)) continue;
 if (accum) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// FIXME!!!
 
 		if (gv->render == RENDER_WIREFRAME)
@@ -556,7 +556,7 @@ if (accum) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// FIXME!!!
 		if (accum)
 		{
 			gv->cam->RenderObjects(gv);
-			glAccum(GL_ACCUM, cs_vector[n1].accum_value);
+			glAccum(GL_ACCUM, cs_vector[n1]->accum_value);
 		}
 	}
 }
@@ -734,7 +734,10 @@ void mm1_docv::SelectEvent(graphics_view *, vector<iGLu> & names)
 
 void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 {
-        static mm1_atom *a1 = NULL;
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	
+	static mm1_atom *a1 = NULL;
 	static mm1_atom *a2 = NULL;
 	static mm1_atom *a3 = NULL;
 // this function is quite similar in mm1_docv and qm1_docv. could this be shorter as a loop???
@@ -748,7 +751,8 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 		if (a1 == NULL)
 		  {
 		    a1 = ref;
-		    cout << "charge: " << ref->charge << endl;
+		    str1 << "charge: " << ref->charge << endl << ends;
+		    PrintToLog(mbuff1);
 		  }
 		else if (a1 != NULL && a2 == NULL)
 		  {
@@ -760,7 +764,8 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 		    float * p2 = a2->crd_vector[0].data;
 		    
 		    float len = measure_len(p1, p2);
-		    cout << "distance: " << len << " nm" << endl;
+		    str1 << "distance: " << len << " nm" << endl << ends;
+		    PrintToLog(mbuff1);
 		  }
 		else if (a1 != NULL && a2 != NULL && a3 == NULL)
 		  {
@@ -774,7 +779,8 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 		    float * p3 = a3->crd_vector[0].data;
 		    
 		    float ang = measure_ang(p1, p2, p3);
-		    cout << "angle: " << ang << " deg" << endl;
+		    str1 << "angle: " << ang << " deg" << endl << ends;
+		    PrintToLog(mbuff1);
 		  }
 		else
 		  {
@@ -788,7 +794,8 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 		    float * p4 = ref->crd_vector[0].data;
 		    
 		    float tor = measure_tor(p1, p2, p3, p4);
-		    cout << "torsion: " << tor << " deg " << endl;
+		    str1 << "torsion: " << tor << " deg " << endl << ends;
+		    PrintToLog(mbuff1);
 		    
 		    a1->selected = false; a1 = NULL;
 		    a2->selected = false; a2 = NULL;
@@ -800,66 +807,548 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 	}
 }
 
-void mm1_docv::DoEnergyPlot1D(void)
+void mm1_docv::ProcessCommandString(graphics_view * gv, const char * command)
 {
-	char question; i32s osteps = 100;
-	i32s qm_eng_index = NOT_DEFINED; i32s qm_total_charge = 0;
+	char mbuff1[512];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "Processing Command : " << command << endl << ends;
+	PrintToLog(mbuff1);
+	
+	bool retval = ProcessCommonCommands(gv, command);
+	if (retval == true) return;
+	
+	istrstream istr(command);
+	char kw1[32]; istr >> kw1;
+	
+	// is it "help"??? if is, then append...
+	
+	if (!strcmp("help", kw1))
+	{
+		PrintToLog("> add plane <vf> <cf> <cscale> <dim> <res> <tp> <alpha> -- add a plane object.\n");
+		PrintToLog(">   where: <vf> = value function : esp vdws dist unity\n");
+		PrintToLog(">          <cf> = colour function : red green blue rb1 rb2\n");
+		PrintToLog(">          <cscale> = scaling value for calculating the colours\n");
+		PrintToLog(">          <dim> = dimension of the plane object (in nm units)\n");
+		PrintToLog(">          <res> = resolution of the plane object\n");
+		PrintToLog(">          <tp> = 0 or 1 telling if the object is transparent\n");
+		PrintToLog(">          <alpha> = transparency alpha value\n");
+		
+		PrintToLog("> add volrend <vf> <cf> <cscale> <dim> <res> <alpha> -- add a volume-rendering object.\n");
+		PrintToLog(">   where: <vf> = value function : esp vdws dist unity\n");
+		PrintToLog(">          <cf> = colour function : red green blue rb1 rb2\n");
+		PrintToLog(">          <cscale> = scaling value for calculating the colours\n");
+		PrintToLog(">          <dim> = dimension of the plane object (in nm units)\n");
+		PrintToLog(">          <res> = resolution of the plane object\n");
+		PrintToLog(">          <alpha> = transparency alpha value\n");
+		
+		PrintToLog("> add surf1 <vf1> <vf2> <cf> <sscale> <cscale> <dim> <res> <solid> <tp> <alpha> -- add a single surface object.\n");
+		PrintToLog(">   where: <vf1> = value function for calculating the surface : esp vdws dist unity\n");
+		PrintToLog(">          <vf2> = value function for calculating the colours : esp vdws dist unity\n");
+		PrintToLog(">          <cf> = colour function : red green blue rb1 rb2\n");
+		PrintToLog(">          <sscale> = scaling value for calculating the surface\n");
+		PrintToLog(">          <cscale> = scaling value for calculating the colours\n");
+		PrintToLog(">          <dim> = dimension of the plane object (in nm units)\n");
+		PrintToLog(">          <res> = resolution of the plane object\n");
+		PrintToLog(">          <solid> = 0 or 1 telling if the object is solid\n");
+		PrintToLog(">          <tp> = 0 or 1 telling if the object is transparent\n");
+		PrintToLog(">          <alpha> = transparency alpha value\n");
+		
+		PrintToLog("> add surf2 <vf1> <vf2> <cf1> <cf2> <sscale1> <sscale2> <cscale> <dim> <res> <solid> <tp> <alpha> -- add a pair of surface objects.\n");
+		PrintToLog(">   where: <vf1> = value function for calculating the surface : esp vdws dist unity\n");
+		PrintToLog(">          <vf2> = value function for calculating the colours : esp vdws dist unity\n");
+		PrintToLog(">          <cf1> = colour function for 1st surface : red green blue rb1 rb2\n");
+		PrintToLog(">          <cf2> = colour function for 2nd surface : red green blue rb1 rb2\n");
+		PrintToLog(">          <sscale1> = scaling value for calculating the surface for 1st surface\n");
+		PrintToLog(">          <sscale2> = scaling value for calculating the surface for 2nd surface\n");
+		PrintToLog(">          <cscale> = scaling value for calculating the colours\n");
+		PrintToLog(">          <dim> = dimension of the plane object (in nm units)\n");
+		PrintToLog(">          <res> = resolution of the plane object\n");
+		PrintToLog(">          <solid> = 0 or 1 telling if the object is solid\n");
+		PrintToLog(">          <tp> = 0 or 1 telling if the object is transparent\n");
+		PrintToLog(">          <alpha> = transparency alpha value\n");
+		
+		PrintToLog("> random_search <cycles> <optsteps> -- perform a random conformational search.\n");
+		PrintToLog("> systematic_search <divisions> <optsteps> -- perform a systematic conformational search.\n");
+		PrintToLog("> montecarlo_search <init_cycles> <simul_cycles> <optsteps> -- perform a MonteCarlo search.\n");
+		
+		PrintToLog("> make_plot1 A B C D <div> <start_ang> <end_ang> <optsteps> (qmeng qmc) -- create a 1D energy vs. torsion plot.\n");
+		PrintToLog("> make_plot2 A B C D <div> <start_ang> <end_ang> I J K L <div> <start_ang> <end_ang> <optsteps> (qmeng qmc) -- create a 2D energy vs. torsions plot.\n");
+		
+		PrintToLog("> build_amino <sequence> (helix/strand) -- amino acid sequence builder.\n");
+		PrintToLog("> build_nucleic <sequence> -- nucleic acid sequence builder.\n");
+		
+		return;
+	}
+	
+	// process other commands.
+	// process other commands.
+	// process other commands.
+	
+	if (!strcmp("add", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// the 2nd keyword; type of the object to add.
+		char object_name[128];
+		
+		if (!strcmp("plane", kw2))
+		{
+			char kw3[32]; istr >> kw3;	// vf
+			char kw4[32]; istr >> kw4;	// cf
+			char kw5[32]; istr >> kw5;	// cscale
+			char kw6[32]; istr >> kw6;	// dim
+			char kw7[32]; istr >> kw7;	// res
+			char kw8[32]; istr >> kw8;	// tp
+			char kw9[32]; istr >> kw9;	// alpha
+			char ** endptr = NULL;
+			
+			cp_param cpp;
+			cpp.docv = this; cpp.ref = (mm1_docv *) this;
+			
+			if (!strcmp(kw3, "esp")) cpp.vf = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw3, "vdws")) cpp.vf = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw3, "dist")) cpp.vf = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw3, "unity")) cpp.vf = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add plane : unknown value function.\n"); return; }
+			
+			if (!strcmp(kw4, "red")) cpp.cf = (ColorFunction *) GetRedColor;
+			else if (!strcmp(kw4, "green")) cpp.cf = (ColorFunction *) GetGreenColor;
+			else if (!strcmp(kw4, "blue")) cpp.cf = (ColorFunction *) GetBlueColor;
+			else if (!strcmp(kw4, "rb1")) cpp.cf = (ColorFunction *) GetRBRange1;
+			else if (!strcmp(kw4, "rb2")) cpp.cf = (ColorFunction *) GetRBRange2;
+			else { PrintToLog("ERROR : add plane : unknown colour function.\n"); return; }
+			
+			f64 cscale = strtod(kw5, endptr);
+			
+			f64 dim = strtod(kw6, endptr);
+			
+			i32s res = strtol(kw7, endptr, 10);
+			if (res < 2) res = 2;
+			
+			i32s tp = strtol(kw8, endptr, 10);
+			if (tp < 0) tp = 0; if (tp > 1) tp = 1;
+			
+			f64 alpha = strtod(kw9, endptr);
+			
+			cpp.value = cscale; cpp.dim = dim; cpp.np = res;
+			cpp.transparent = tp; cpp.alpha = alpha;
+			
+			ostrstream strN(object_name, sizeof(object_name));
+			strN << kw3 << "-" << ends;
+			
+			AddObject(new color_plane_object(ol_static(), cpp, object_name));
+			UpdateAllGraphicsViews();
+			
+			ostrstream strR(mbuff1, sizeof(mbuff1));
+			strR << "Added a new object : plane (" << kw3 << " " << kw4 << ")." << endl << ends;
+			PrintToLog(mbuff1);
+			return;
+		}
+		
+		if (!strcmp("volrend", kw2))
+		{
+			char kw3[32]; istr >> kw3;	// vf
+			char kw4[32]; istr >> kw4;	// cf
+			char kw5[32]; istr >> kw5;	// cscale
+			char kw6[32]; istr >> kw6;	// dim
+			char kw7[32]; istr >> kw7;	// res
+			char kw8[32]; istr >> kw8;	// alpha
+			char ** endptr = NULL;
+			
+			cp_param cpp;
+			cpp.docv = this; cpp.ref = (mm1_docv *) this;
+			
+			if (!strcmp(kw3, "esp")) cpp.vf = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw3, "vdws")) cpp.vf = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw3, "dist")) cpp.vf = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw3, "unity")) cpp.vf = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add volrend : unknown value function.\n"); return; }
+			
+			if (!strcmp(kw4, "red")) cpp.cf = (ColorFunction *) GetRedColor;
+			else if (!strcmp(kw4, "green")) cpp.cf = (ColorFunction *) GetGreenColor;
+			else if (!strcmp(kw4, "blue")) cpp.cf = (ColorFunction *) GetBlueColor;
+			else if (!strcmp(kw4, "rb1")) cpp.cf = (ColorFunction *) GetRBRange1;
+			else if (!strcmp(kw4, "rb2")) cpp.cf = (ColorFunction *) GetRBRange2;
+			else { PrintToLog("ERROR : add volrend : unknown colour function.\n"); return; }
+			
+			f64 cscale = strtod(kw5, endptr);
+			
+			f64 dim = strtod(kw6, endptr);
+			
+			i32s res = strtol(kw7, endptr, 10);
+			if (res < 4) res = 4;
+			
+			f64 alpha = strtod(kw8, endptr);
+			
+			cpp.value = cscale; cpp.dim = dim; cpp.np = res;
+			cpp.transparent = true; cpp.alpha = alpha;
+			
+			ostrstream strN(object_name, sizeof(object_name));
+			strN << kw3 << "-" << ends;
+			
+			AddObject(new volume_rendering_object(ol_static(), cpp, res / 2, dim / 2.0, (* gv->cam), object_name));
+			UpdateAllGraphicsViews();
+			
+			ostrstream strR(mbuff1, sizeof(mbuff1));
+			strR << "Added a new object : volrend (" << kw3 << " " << kw4 << ")." << endl << ends;
+			PrintToLog(mbuff1);
+			return;
+		}
+		
+		if (!strcmp("surf1", kw2))
+		{
+			char kw3[32]; istr >> kw3;	// vf1
+			char kw4[32]; istr >> kw4;	// vf2
+			char kw5[32]; istr >> kw5;	// cf
+			char kw6[32]; istr >> kw6;	// sscale
+			char kw7[32]; istr >> kw7;	// cscale
+			char kw8[32]; istr >> kw8;	// dim
+			char kw9[32]; istr >> kw9;	// res
+			char kwA[32]; istr >> kwA;	// solid
+			char kwB[32]; istr >> kwB;	// tp
+			char kwC[32]; istr >> kwC;	// alpha
+			char ** endptr = NULL;
+			
+			cs_param csp1; csp1.docv = this; csp1.ref = (mm1_docv *) this; csp1.next = NULL;
+			
+			if (!strcmp(kw3, "esp")) csp1.vf1 = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw3, "vdws")) csp1.vf1 = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw3, "dist")) csp1.vf1 = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw3, "unity")) csp1.vf1 = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add surf1 : unknown value function 1.\n"); return; }
+			
+			if (!strcmp(kw4, "esp")) csp1.vf2 = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw4, "vdws")) csp1.vf2 = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw4, "dist")) csp1.vf2 = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw4, "unity")) csp1.vf2 = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add surf1 : unknown value function 2.\n"); return; }
+			
+			if (!strcmp(kw5, "red")) csp1.cf = (ColorFunction *) GetRedColor;
+			else if (!strcmp(kw5, "green")) csp1.cf = (ColorFunction *) GetGreenColor;
+			else if (!strcmp(kw5, "blue")) csp1.cf = (ColorFunction *) GetBlueColor;
+			else if (!strcmp(kw5, "rb1")) csp1.cf = (ColorFunction *) GetRBRange1;
+			else if (!strcmp(kw5, "rb2")) csp1.cf = (ColorFunction *) GetRBRange2;
+			else { PrintToLog("ERROR : add surf1 : unknown colour function.\n"); return; }
+			
+			f64 sscale = strtod(kw6, endptr);
+			f64 cscale = strtod(kw7, endptr);
+			
+			f64 dim = strtod(kw8, endptr);
+			
+			i32s res = strtol(kw9, endptr, 10);
+			if (res < 4) res = 4;
+			
+			i32s solid = strtol(kwA, endptr, 10);
+			if (solid < 0) solid = 0; if (solid > 1) solid = 1;
+
+			i32s tp = strtol(kwB, endptr, 10);
+			if (tp < 0) tp = 0; if (tp > 1) tp = 1;
+			
+			f64 alpha = strtod(kwC, endptr);
+			
+			static fGL dim_arr[3];
+			dim_arr[0] = dim_arr[1] = dim_arr[2] = dim;
+			
+			static i32s res_arr[3];
+			res_arr[0] = res_arr[1] = res_arr[2] = res;
+			
+			csp1.svalue = sscale; csp1.cvalue = cscale;
+			csp1.dim = dim_arr; csp1.np = res_arr; csp1.wireframe = !solid;
+			csp1.transparent = tp; csp1.alpha = alpha; csp1.toler = fabs(1.0e-6 * sscale); csp1.maxc = 250;
+			
+			ostrstream strN(object_name, sizeof(object_name));
+			strN << kw3 << "-" << kw4 << "-" << ends;
+			
+			AddObject(new color_surface_object(ol_static(), csp1, object_name));
+			UpdateAllGraphicsViews();
+			
+			ostrstream strR(mbuff1, sizeof(mbuff1));
+			strR << "Added a new object : surf1 (" << kw3 << " " << kw4 << " " << kw5 << ")." << endl << ends;
+			PrintToLog(mbuff1);
+			return;
+		}
+		
+		if (!strcmp("surf2", kw2))
+		{
+			char kw3[32]; istr >> kw3;	// vf1
+			char kw4[32]; istr >> kw4;	// vf2
+			char kw5[32]; istr >> kw5;	// cf1
+			char kw6[32]; istr >> kw6;	// cf2
+			char kw7[32]; istr >> kw7;	// sscale1
+			char kw8[32]; istr >> kw8;	// sscale2
+			char kw9[32]; istr >> kw9;	// cscale
+			char kwA[32]; istr >> kwA;	// dim
+			char kwB[32]; istr >> kwB;	// res
+			char kwC[32]; istr >> kwC;	// solid
+			char kwD[32]; istr >> kwD;	// tp
+			char kwE[32]; istr >> kwE;	// alpha
+			char ** endptr = NULL;
+			
+			cs_param csp2a; cs_param csp2b;
+			
+			csp2a.docv = this; csp2a.ref = (mm1_docv *) this; csp2a.next = & csp2b;
+
+			csp2b.docv = this; csp2b.ref = (mm1_docv *) this; csp2b.next = NULL;
+			
+			if (!strcmp(kw3, "esp")) csp2a.vf1 = csp2b.vf1 = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw3, "vdws")) csp2a.vf1 = csp2b.vf1 = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw3, "dist")) csp2a.vf1 = csp2b.vf1 = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw3, "unity")) csp2a.vf1 = csp2b.vf1 = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add surf2 : unknown value function 1.\n"); return; }
+			
+			if (!strcmp(kw4, "esp")) csp2a.vf2 = csp2b.vf2 = (ValueFunction *) mm1_GetESPValue;
+			else if (!strcmp(kw4, "vdws")) csp2a.vf2 = csp2b.vf2 = (ValueFunction *) mm1_GetVDWSValue;
+			else if (!strcmp(kw4, "dist")) csp2a.vf2 = csp2b.vf2 = (ValueFunction *) GetDistance;
+			else if (!strcmp(kw4, "unity")) csp2a.vf2 = csp2b.vf2 = (ValueFunction *) GetUnity;
+			else { PrintToLog("ERROR : add surf2 : unknown value function 2.\n"); return; }
+			
+			if (!strcmp(kw5, "red")) csp2a.cf = (ColorFunction *) GetRedColor;
+			else if (!strcmp(kw5, "green")) csp2a.cf = (ColorFunction *) GetGreenColor;
+			else if (!strcmp(kw5, "blue")) csp2a.cf = (ColorFunction *) GetBlueColor;
+			else if (!strcmp(kw5, "rb1")) csp2a.cf = (ColorFunction *) GetRBRange1;
+			else if (!strcmp(kw5, "rb2")) csp2a.cf = (ColorFunction *) GetRBRange2;
+			else { PrintToLog("ERROR : add surf2 : unknown colour function 1.\n"); return; }
+			
+			if (!strcmp(kw6, "red")) csp2b.cf = (ColorFunction *) GetRedColor;
+			else if (!strcmp(kw6, "green")) csp2b.cf = (ColorFunction *) GetGreenColor;
+			else if (!strcmp(kw6, "blue")) csp2b.cf = (ColorFunction *) GetBlueColor;
+			else if (!strcmp(kw6, "rb1")) csp2b.cf = (ColorFunction *) GetRBRange1;
+			else if (!strcmp(kw6, "rb2")) csp2b.cf = (ColorFunction *) GetRBRange2;
+			else { PrintToLog("ERROR : add surf2 : unknown colour function 2.\n"); return; }
+			
+			f64 sscale1 = strtod(kw7, endptr);
+			f64 sscale2 = strtod(kw8, endptr);
+			f64 cscale = strtod(kw9, endptr);
+			
+			f64 dim = strtod(kwA, endptr);
+			
+			i32s res = strtol(kwB, endptr, 10);
+			if (res < 4) res = 4;
+			
+			i32s solid = strtol(kwC, endptr, 10);
+			if (solid < 0) solid = 0; if (solid > 1) solid = 1;
+
+			i32s tp = strtol(kwD, endptr, 10);
+			if (tp < 0) tp = 0; if (tp > 1) tp = 1;
+			
+			f64 alpha = strtod(kwE, endptr);
+			
+			static fGL dim_arr[3];
+			dim_arr[0] = dim_arr[1] = dim_arr[2] = dim;
+			
+			static i32s res_arr[3];
+			res_arr[0] = res_arr[1] = res_arr[2] = res;
+			
+			csp2a.svalue = sscale1; csp2a.cvalue = cscale;
+			csp2a.dim = dim_arr; csp2a.np = res_arr; csp2a.wireframe = !solid;
+			csp2a.transparent = tp; csp2a.alpha = alpha; csp2a.toler = fabs(1.0e-6 * sscale1); csp2a.maxc = 250;
+			
+			csp2b.svalue = sscale2; csp2b.cvalue = cscale;
+			csp2b.dim = dim_arr; csp2b.np = res_arr; csp2b.wireframe = !solid;
+			csp2b.transparent = tp; csp2b.alpha = alpha; csp2b.toler = fabs(1.0e-6 * sscale2); csp2b.maxc = 250;
+			
+			ostrstream strN(object_name, sizeof(object_name));
+			strN << kw3 << "-" << kw4 << "-" << ends;
+			
+			AddObject(new color_surface_object(ol_static(), csp2a, object_name));
+			UpdateAllGraphicsViews();
+			
+			ostrstream strR(mbuff1, sizeof(mbuff1));
+			strR << "Added a new object : surf2 (" << kw3 << " " << kw4 << " " << kw5 << " " << kw6 << ")." << endl << ends;
+			PrintToLog(mbuff1);
+			return;
+		}
+		
+		PrintToLog("ERROR : could not process a command : \"add\".\n");
+	}
+	
+	if (!strcmp("random_search", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// the 2nd keyword; cycles.
+		char kw3[32]; istr >> kw3;	// the 3rd keyword; optsteps.
+		
+		char ** endptr = NULL;
+		i32s cycles = strtol(kw2, endptr, 10);
+		i32s optsteps = strtol(kw3, endptr, 10);
+		
+		DoRandomSearch(cycles, optsteps);
+		return;
+	}
+
+	if (!strcmp("systematic_search", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// the 2nd keyword; divisions.
+		char kw3[32]; istr >> kw3;	// the 3rd keyword; optsteps.
+		
+		char ** endptr = NULL;
+		i32s divisions = strtol(kw2, endptr, 10);
+		i32s optsteps = strtol(kw3, endptr, 10);
+		
+		DoSystematicSearch(divisions, optsteps);
+		return;
+	}
+
+	if (!strcmp("montecarlo_search", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// the 2nd keyword; n_init_steps.
+		char kw3[32]; istr >> kw3;	// the 3rd keyword; n_simul_steps.
+		char kw4[32]; istr >> kw4;	// the 4th keyword; optsteps.
+		
+		char ** endptr = NULL;
+		i32s n_init_steps = strtol(kw2, endptr, 10);
+		i32s n_simul_steps = strtol(kw3, endptr, 10);
+		i32s optsteps = strtol(kw4, endptr, 10);
+		
+		DoMonteCarloSearch(n_init_steps, n_simul_steps, optsteps);
+		return;
+	}
+	
+	if (!strcmp("make_plot1", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// A
+		char kw3[32]; istr >> kw3;	// B
+		char kw4[32]; istr >> kw4;	// C
+		char kw5[32]; istr >> kw5;	// D
+		char kw6[32]; istr >> kw6;	// div
+		char kw7[32]; istr >> kw7;	// start_ang
+		char kw8[32]; istr >> kw8;	// end_ang
+		char kw9[32]; istr >> kw9;	// optsteps
+		char kwA[32]; istr >> kwA;	// qmeng
+		char kwB[32]; istr >> kwB;	// qmc
+		char ** endptr = NULL;
+		
+		i32s ia = strtol(kw2, endptr, 10);
+		i32s ib = strtol(kw3, endptr, 10);
+		i32s ic = strtol(kw4, endptr, 10);
+		i32s id = strtol(kw5, endptr, 10);
+		i32s div1 = strtol(kw6, endptr, 10);
+		fGL start1 = strtod(kw7, endptr);
+		fGL end1 = strtod(kw8, endptr);
+		
+		i32s optsteps = strtol(kw9, endptr, 10);
+		
+		i32s qmeng = strtol(kwA, endptr, 10); if (!strlen(kwA)) qmeng = -1;
+		i32s qmc = strtol(kwB, endptr, 10);
+		
+		DoEnergyPlot1D(ia, ib, ic, id, div1, start1, end1, optsteps, qmeng, qmc);
+		return;
+	}
+	
+	if (!strcmp("make_plot2", kw1))
+	{
+		char kw2[32]; istr >> kw2;	// A
+		char kw3[32]; istr >> kw3;	// B
+		char kw4[32]; istr >> kw4;	// C
+		char kw5[32]; istr >> kw5;	// D
+		char kw6[32]; istr >> kw6;	// div
+		char kw7[32]; istr >> kw7;	// start_ang
+		char kw8[32]; istr >> kw8;	// end_ang
+		char kw9[32]; istr >> kw9;	// I
+		char kwA[32]; istr >> kwA;	// J
+		char kwB[32]; istr >> kwB;	// K
+		char kwC[32]; istr >> kwC;	// L
+		char kwD[32]; istr >> kwD;	// div
+		char kwE[32]; istr >> kwE;	// start_ang
+		char kwF[32]; istr >> kwF;	// end_ang
+		char kwG[32]; istr >> kwG;	// optsteps
+		char kwH[32]; istr >> kwH;	// qmeng
+		char kwI[32]; istr >> kwI;	// qmc
+		char ** endptr = NULL;
+		
+		i32s ia = strtol(kw2, endptr, 10);
+		i32s ib = strtol(kw3, endptr, 10);
+		i32s ic = strtol(kw4, endptr, 10);
+		i32s id = strtol(kw5, endptr, 10);
+		i32s div1 = strtol(kw6, endptr, 10);
+		fGL start1 = strtod(kw7, endptr);
+		fGL end1 = strtod(kw8, endptr);
+		
+		i32s ii = strtol(kw9, endptr, 10);
+		i32s ij = strtol(kwA, endptr, 10);
+		i32s ik = strtol(kwB, endptr, 10);
+		i32s il = strtol(kwC, endptr, 10);
+		i32s div2 = strtol(kwD, endptr, 10);
+		fGL start2 = strtod(kwE, endptr);
+		fGL end2 = strtod(kwF, endptr);
+		
+		i32s optsteps = strtol(kwG, endptr, 10);
+		
+		i32s qmeng = strtol(kwH, endptr, 10); if (!strlen(kwH)) qmeng = -1;
+		i32s qmc = strtol(kwI, endptr, 10);
+		
+		DoEnergyPlot2D(ia, ib, ic, id, div1, start1, end1, ii, ij, ik, il, div2, start2, end2, optsteps, qmeng, qmc);
+		return;
+	}
+	
+	if (!strcmp("build_amino", kw1))
+	{
+		char kw2[4096]; istr >> kw2;	// sequence
+		char kw3[32]; istr >> kw3;	// helix/sheet (optional)
+		
+		f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
+		f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
+		
+		mm1_mdl::amino_builder.Build(this, kw2, kw3[0] == 'h' || kw3[0] == 'H' ? aab_ah : aab_bs);
+		UpdateAllGraphicsViews();
+		
+		ostrstream strR(mbuff1, sizeof(mbuff1));
+		strR << "built a sequence : " << kw2 << endl << ends;
+		PrintToLog(mbuff1);
+		return;
+	}
+	
+	if (!strcmp("build_nucleic", kw1))
+	{
+		char kw2[4096]; istr >> kw2;	// sequence
+		
+		f64 nab[] =
+		{
+			44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
+			273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
+			88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
+		};
+		
+		mm1_mdl::nucleic_builder.Build(this, kw2, nab);
+		UpdateAllGraphicsViews();
+		
+		ostrstream strR(mbuff1, sizeof(mbuff1));
+		strR << "built a sequence : " << kw2 << endl << ends;
+		PrintToLog(mbuff1);
+		return;
+	}
+	
+	// error message...
+	ostrstream strE(mbuff1, sizeof(mbuff1)); strE << "ERROR : Unknown command : " << command << endl << ends;
+	PrintToLog(mbuff1); PrintToLog("The \"help\" command will give more information about command strings.\n");
+}
+
+void mm1_docv::DoEnergyPlot1D(i32s inda, i32s indb, i32s indc, i32s indd, i32s div1, fGL start1, fGL end1, i32s optsteps, i32s qmeng, i32s qmc)
+{
+	if (qmeng != -1)
+	{
+		char mbuff9[256]; strstream str9(mbuff9, sizeof(mbuff9));
+		str9 << "QM method was requested: using " << qm1_mdl::engtab1[qmeng] << " method with charge = " << qmc << endl << ends;
+		PrintToLog(mbuff9);
+	}
 	
 	i32s molnum = 0; i32s in_crdset = 0;
 	
-	f64 range1[2] = { 0.0, 2.0 * M_PI }; i32s steps1 = 36;
-	i32s atmi1[4]; mm1_atom * atmr1[4];
-cout << "give the atom ID numbers (separated by spaces) for torsion : ";
-cin >> atmi1[0] >> atmi1[1] >> atmi1[2] >> atmi1[3];
-cout << "do you want to change the default settings (y=yes)? "; cin >> question;
-if (question == 'y' || question == 'Y')
-{
-	cout << "give the number of steps (-1 = cancel, default = " << steps1 << ")? "; cin >> steps1;
-	if (steps1 < 0) return;
+	i32s atmi1[4] = { inda, indb, indc, indd };
+	mm1_atom * atmr1[4]; f64 range1[2];
+	range1[0] = M_PI * start1 / 180.0;
+	range1[1] = M_PI * end1 / 180.0;
 	
-	f64 range[2] = { 180.0 * range1[0] / M_PI, 180.0 * range1[0] / M_PI };
-	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
-	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
-	
-	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
-	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
-	
-	range1[0] = M_PI * range[0] / 180.0;
-	range1[1] = M_PI * range[1] / 180.0;
-}
-
-cout << "give the optimization steps (-1 to skip optimization, default = " << osteps << ")? "; cin >> osteps;
-cout << "should the final energy be calculated using a QM method (y=yes)? "; cin >> question;
-if (question == 'y' || question == 'Y')
-{
-	i32s index = 0;
-	while (true)
-	{
-		if (qm1_mdl::engtab1[index] == NULL) break;
-		
-		cout << index << " " << qm1_mdl::engtab1[index] << endl;
-		index++;
-	}
-	
-	cout << "give the method from the above list (or -1 to cancel)? "; cin >> qm_eng_index;
-	if (qm_eng_index >= index) qm_eng_index = NOT_DEFINED;	// check the index...
-	
-	cout << "give the total charge for the QM model (default = " << qm_total_charge << ")? "; cin >> qm_total_charge;
-}
-
 	for (i32s n1 = 0;n1 < 4;n1++)
 	{
 		iter_mm1al it1;
 		
 		it1 = FindAtomByIndex(atmi1[n1]);
-		if (it1 == GetAtomsEnd()) cout << "atom not found!" << endl;
+		if (it1 == GetAtomsEnd()) { PrintToLog("ERROR : atom not found!\n"); return; }
 		atmr1[n1] = & (* it1);
 	}
 	
 	GatherGroups();		// for internal coordinates...
 	mm1_intcrd * ic = new mm1_intcrd((* this), molnum, in_crdset);
 	i32s ict1 = ic->FindTorsion(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
-if (ict1 < 0) { cout << "could not find ic tor1." << endl; return; }
+if (ict1 < 0) { PrintToLog("ERROR : could not find ic for tor1.\n"); return; }
 	
 	v3d<fGL> v1a(atmr1[1]->crd_vector[in_crdset].data, atmr1[0]->crd_vector[in_crdset].data);
 	v3d<fGL> v1b(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
@@ -868,14 +1357,14 @@ if (ict1 < 0) { cout << "could not find ic tor1." << endl; return; }
 	
 	mm1_eng * eng1 = CreateDefaultEngine();
 	i32s fft1 = eng1->FindTorsion(atmr1[0], atmr1[1], atmr1[2], atmr1[3]);
-if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
+if (ict1 < 0) { PrintToLog("ERROR : could not find tor-term for tor1.\n"); return; }
 
 	mm1_eng * eng2 = CreateDefaultEngine();
 	
 	plot1d_view * plot = graphics_factory->ProducePlot1DView(this, PLOT_USERDATA_STRUCTURE, 0);
 	
 	f64 tor1 = range1[0];
-	for (i32s s1 = 0;s1 < (steps1 + 1);s1++)
+	for (i32s s1 = 0;s1 < (div1 + 1);s1++)
 	{
 		ic->SetTorsion(ict1, tor1 - oldt1);
 		ic->UpdateCartesian();
@@ -887,12 +1376,12 @@ if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
 		
 		mm1_geomopt * opt = new mm1_geomopt(eng1, 100, 0.025);		// optimal settings?!?!?
 		
-		for (i32s n1 = 0;n1 < osteps;n1++)
+		for (i32s n1 = 0;n1 < optsteps;n1++)
 		{
 			opt->TakeCGStep(conjugate_gradient::Newton2An);
 			cout << n1 << " " << opt->optval << " " << opt->optstp << endl;
 			
-			if (!(n1 % 10))
+			if (!(n1 % 50))
 			{
 				CopyCRD(eng1, this, 0); CenterCRDSet(0);
 				UpdateAllGraphicsViews(true);
@@ -905,7 +1394,7 @@ if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
 		// compute energy for final structure...
 		
 		f64 value;
-		if (qm_eng_index < 0)
+		if (qmeng < 0)
 		{
 			CopyCRD(this, eng2, 0);
 			eng2->Compute(0);
@@ -917,8 +1406,7 @@ if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
 			qm1_mdl * mdl9 = new qm1_mdl(& cout, * console_class_factory::GetInstance());
 			mdl9->aai_MakeCopy(this);
 			
-			mdl9->default_eng = qm_eng_index;
-			mdl9->SetTotalCharge(qm_total_charge);
+			mdl9->default_eng = qmeng; mdl9->SetTotalCharge(qmc);
 			qm1_eng * eng9 = mdl9->CreateDefaultEngine();
 			
 			if (eng9 != NULL)
@@ -943,7 +1431,12 @@ if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
 		void * udata = mm1_convert_cset_to_plotting_udata(this, 0);
 		plot->AddData(180.0 * tor1 / M_PI, value, udata);
 		
-		tor1 += (range1[1] - range1[0]) / (f64) steps1;
+		char mbuff1[256];
+		strstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "tor = " << (180.0 * tor1 / M_PI) << " deg, energy = " << value << " kJ/mol." << endl << ends;
+		PrintToLog(mbuff1);
+		
+		tor1 += (range1[1] - range1[0]) / (f64) div1;
 	}
 	
 	plotting_view_vector.push_back(plot);
@@ -955,93 +1448,46 @@ if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
 	delete ic;
 }
 
-void mm1_docv::DoEnergyPlot2D(void)
+void mm1_docv::DoEnergyPlot2D(i32s inda, i32s indb, i32s indc, i32s indd, i32s div1, fGL start1, fGL end1, i32s indi, i32s indj, i32s indk, i32s indl, i32s div2, fGL start2, fGL end2, i32s optsteps, i32s qmeng, i32s qmc)
 {
-	char question; i32s osteps = 100;
-	i32s qm_eng_index = NOT_DEFINED; i32s qm_total_charge = 0;
+	if (qmeng != -1)
+	{
+		char mbuff9[256]; strstream str9(mbuff9, sizeof(mbuff9));
+		str9 << "QM method was requested: using " << qm1_mdl::engtab1[qmeng] << " method with charge = " << qmc << endl << ends;
+		PrintToLog(mbuff9);
+	}
 	
 	i32s molnum = 0; i32s in_crdset = 0;
 	
-	f64 range1[2] = { 0.0, 2.0 * M_PI }; i32s steps1 = 18;
-	i32s atmi1[4]; mm1_atom * atmr1[4];
-cout << "give the atom ID numbers (separated by spaces) for torsion1 : ";
-cin >> atmi1[0] >> atmi1[1] >> atmi1[2] >> atmi1[3];
-cout << "do you want to change the default settings (y=yes)? "; cin >> question;
-if (question == 'y' || question == 'Y')
-{
-	cout << "give the number of steps (-1 = cancel, default = " << steps1 << ")? "; cin >> steps1;
-	if (steps1 < 0) return;
+	i32s atmi1[4] = { inda, indb, indc, indd };
+	mm1_atom * atmr1[4]; f64 range1[2];
+	range1[0] = M_PI * start1 / 180.0;
+	range1[1] = M_PI * end1 / 180.0;
 	
-	f64 range[2] = { 180.0 * range1[0] / M_PI, 180.0 * range1[0] / M_PI };
-	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
-	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
+	i32s atmi2[4] = { indi, indj, indk, indl };
+	mm1_atom * atmr2[4]; f64 range2[2];
+	range2[0] = M_PI * start2 / 180.0;
+	range2[1] = M_PI * end2 / 180.0;
 	
-	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
-	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
-	
-	range1[0] = M_PI * range[0] / 180.0;
-	range1[1] = M_PI * range[1] / 180.0;
-}
-
-	f64 range2[2] = { 0.0, 2.0 * M_PI }; i32s steps2 = 18;
-	i32s atmi2[4]; mm1_atom * atmr2[4];
-cout << "give the atom ID numbers (separated by spaces) for torsion2 : ";
-cin >> atmi2[0] >> atmi2[1] >> atmi2[2] >> atmi2[3];
-cout << "do you want to change the default settings (y=yes)? "; cin >> question;
-if (question == 'y' || question == 'Y')
-{
-	cout << "give the number of steps (-1 = cancel, default = " << steps2 << ")? "; cin >> steps2;
-	if (steps2 < 0) return;
-	
-	f64 range[2] = { 180.0 * range2[0] / M_PI, 180.0 * range2[0] / M_PI };
-	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
-	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
-	
-	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
-	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
-	
-	range2[0] = M_PI * range[0] / 180.0;
-	range2[1] = M_PI * range[1] / 180.0;
-}
-
-cout << "give the optimization steps (-1 to skip optimization, default = " << osteps << ")? "; cin >> osteps;
-cout << "should the final energy be calculated using a QM method (y=yes)? "; cin >> question;
-if (question == 'y' || question == 'Y')
-{
-	i32s index = 0;
-	while (true)
-	{
-		if (qm1_mdl::engtab1[index] == NULL) break;
-		
-		cout << index << " " << qm1_mdl::engtab1[index] << endl;
-		index++;
-	}
-	
-	cout << "give the method from the above list (or -1 to cancel)? "; cin >> qm_eng_index;
-	if (qm_eng_index >= index) qm_eng_index = NOT_DEFINED;	// check the index...
-	
-	cout << "give the total charge for the QM model (default = " << qm_total_charge << ")? "; cin >> qm_total_charge;
-}
-
 	for (i32s n1 = 0;n1 < 4;n1++)
 	{
 		iter_mm1al it1;
 		
 		it1 = FindAtomByIndex(atmi1[n1]);
-		if (it1 == GetAtomsEnd()) cout << "atom not found #1!" << endl;
+		if (it1 == GetAtomsEnd()) { PrintToLog("ERROR : tor1 atom not found!\n"); return; }
 		atmr1[n1] = & (* it1);
 		
 		it1 = FindAtomByIndex(atmi2[n1]);
-		if (it1 == GetAtomsEnd()) cout << "atom not found #2!" << endl;
+		if (it1 == GetAtomsEnd()) { PrintToLog("ERROR : tor2 atom not found!\n"); return; }
 		atmr2[n1] = & (* it1);
 	}
 	
 	GatherGroups();		// for internal coordinates...
 	mm1_intcrd * ic = new mm1_intcrd((* this), molnum, in_crdset);
 	i32s ict1 = ic->FindTorsion(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
-if (ict1 < 0) { cout << "could not find ic tor1." << endl; return; }
+if (ict1 < 0) { PrintToLog("ERROR : could not find ic for tor1.\n"); return; }
 	i32s ict2 = ic->FindTorsion(atmr2[1]->crd_vector[in_crdset].data, atmr2[2]->crd_vector[in_crdset].data);
-if (ict2 < 0) { cout << "could not find ic tor2." << endl; return; }
+if (ict2 < 0) { PrintToLog("ERROR : could not find ic for tor2.\n"); return; }
 	
 	v3d<fGL> v1a(atmr1[1]->crd_vector[in_crdset].data, atmr1[0]->crd_vector[in_crdset].data);
 	v3d<fGL> v1b(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
@@ -1055,19 +1501,19 @@ if (ict2 < 0) { cout << "could not find ic tor2." << endl; return; }
 	
 	mm1_eng * eng1 = CreateDefaultEngine();
 	i32s fft1 = eng1->FindTorsion(atmr1[0], atmr1[1], atmr1[2], atmr1[3]);
-if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
+if (ict1 < 0) { PrintToLog("ERROR : could not find tor-term for tor1.\n"); return; }
 	i32s fft2 = eng1->FindTorsion(atmr2[0], atmr2[1], atmr2[2], atmr2[3]);
-if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
+if (ict2 < 0) { PrintToLog("ERROR : could not find tor-term for tor2.\n"); return; }
 
 	mm1_eng * eng2 = CreateDefaultEngine();
 	
 	plot2d_view * plot = graphics_factory->ProducePlot2DView(this, PLOT_USERDATA_STRUCTURE, 0);
 	
 	f64 tor1 = range1[0];
-	for (i32s s1 = 0;s1 < (steps1 + 1);s1++)
+	for (i32s s1 = 0;s1 < (div1 + 1);s1++)
 	{
 		f64 tor2 = range2[0];
-		for (i32s s2 = 0;s2 < (steps2 + 1);s2++)
+		for (i32s s2 = 0;s2 < (div2 + 1);s2++)
 		{
 			ic->SetTorsion(ict1, tor1 - oldt1);
 			ic->SetTorsion(ict2, tor2 - oldt2);
@@ -1081,12 +1527,12 @@ if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
 			
 			mm1_geomopt * opt = new mm1_geomopt(eng1, 100, 0.025);		// optimal settings?!?!?
 			
-			for (i32s n1 = 0;n1 < osteps;n1++)
+			for (i32s n1 = 0;n1 < optsteps;n1++)
 			{
 				opt->TakeCGStep(conjugate_gradient::Newton2An);
 				cout << n1 << " " << opt->optval << " " << opt->optstp << endl;
 				
-				if (!(n1 % 10))
+				if (!(n1 % 50))
 				{
 					CopyCRD(eng1, this, 0); CenterCRDSet(0);
 					UpdateAllGraphicsViews(true);
@@ -1099,7 +1545,7 @@ if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
 			// compute energy for final structure...
 			
 			f64 value;
-			if (qm_eng_index < 0)
+			if (qmeng < 0)
 			{
 				CopyCRD(this, eng2, 0);
 				eng2->Compute(0);
@@ -1111,8 +1557,7 @@ if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
 				qm1_mdl * mdl9 = new qm1_mdl(& cout, * console_class_factory::GetInstance());
 				mdl9->aai_MakeCopy(this);
 				
-				mdl9->default_eng = qm_eng_index;
-				mdl9->SetTotalCharge(qm_total_charge);
+				mdl9->default_eng = qmeng; mdl9->SetTotalCharge(qmc);
 				qm1_eng * eng9 = mdl9->CreateDefaultEngine();
 				
 				if (eng9 != NULL)
@@ -1137,10 +1582,15 @@ if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
 			void * udata = mm1_convert_cset_to_plotting_udata(this, 0);
 			plot->AddData(180.0 * tor1 / M_PI, 180.0 * tor2 / M_PI, value, udata);
 			
-			tor2 += (range2[1] - range2[0]) / (f64) steps2;
+			char mbuff1[256];
+			strstream str1(mbuff1, sizeof(mbuff1));
+			str1 << "tor1 = " << (180.0 * tor1 / M_PI) << " deg, tor2 = " << (180.0 * tor2 / M_PI) << " deg, energy = " << value << " kJ/mol." << endl << ends;
+			PrintToLog(mbuff1);
+			
+			tor2 += (range2[1] - range2[0]) / (f64) div2;
 		}
 		
-		tor1 += (range1[1] - range1[0]) / (f64) steps1;
+		tor1 += (range1[1] - range1[0]) / (f64) div1;
 	}
 	
 	plotting_view_vector.push_back(plot);
