@@ -80,12 +80,14 @@ class mm1_docv : virtual public docview, public mm1_mdl
 	void SelectEvent(graphics_view *, vector<iGLu> &);	// virtual
 	void MeasureEvent(graphics_view *, vector<iGLu> &);	// virtual
 	
+	void ProcessCommandString(graphics_view *, const char *);	// virtual
+	
 	// here are some Do???() functions similar to those in mm1_mdl class.
 	// however, the classes here are "interactive" and are directly dependent on graphics.
 	// so you don't want to use any of these classes in the console version...
 	
-	void DoEnergyPlot1D(void);
-	void DoEnergyPlot2D(void);
+	void DoEnergyPlot1D(i32s, i32s, i32s, i32s, i32s, fGL, fGL, i32s, i32s, i32s);
+	void DoEnergyPlot2D(i32s, i32s, i32s, i32s, i32s, fGL, fGL, i32s, i32s, i32s, i32s, i32s, fGL, fGL, i32s, i32s, i32s);
 };
 
 /*################################################################################################*/
