@@ -51,8 +51,11 @@ class mm2_docv : virtual public docview, public mm2_mdl
 	void SelectAll(void);				// virtual
 	void InvertSelection(void);			// virtual
 //	void SelectNone(void);				// virtual	// see the base class...
+
+	bool TestAtom(i32s, i32s, rmode);
+	bool TestBond(i32s, i32s, i32s, i32s, rmode rm);
 	
-	void DrawCylinder1(graphics_view *, mm2_res **, i32s *, i32s);
+	void DrawCylinder1(graphics_view *, i32s **, i32s);
 	
 	void Render(graphics_view *, rmode);		// virtual
 	
@@ -63,14 +66,16 @@ class mm2_docv : virtual public docview, public mm2_mdl
 	void EraseEvent(graphics_view *, vector<iGLu> &);	// virtual
 	void SelectEvent(graphics_view *, vector<iGLu> &);	// virtual
 	void MeasureEvent(graphics_view *, vector<iGLu> &);	// virtual
+	
+	void ProcessCommandString(graphics_view *, const char *);	// virtual
 };
 
 /*################################################################################################*/
 
 struct mm2_cdata
 {
-	mm2_res * res;
-	i32s index;
+	mm2_docv * docv;
+	i32s index[3];
 };
 
 class mm2_cm_state : public color_mode
