@@ -103,9 +103,8 @@ void mm2_ribbon::MakeRibbon(void)
 	data5 = new fGL_a4[length];
 	for (i32s n1 = 0;n1 < length;n1++)
 	{
-		mm2_cdata cdata;
-		cdata.res = & docv->chn_vector[chn].res_vector[n1];
-		cdata.index = 0;
+		mm2_cdata cdata; cdata.docv = docv;
+		cdata.index[0] = chn; cdata.index[1] = n1; cdata.index[2] = 0;
 		
 		cmode->GetColor(& cdata, data5[n1]);
 	}
