@@ -63,10 +63,15 @@ void select_tool::ButtonEvent(ogl_view * oglv, i32s x, i32s y)
 	
 	graphics_view * gv = dynamic_cast<graphics_view *>(oglv);
 	if (gv != NULL) gv->cam->RenderWindow(gv, camera::Select, x, y);
+	
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 }
 
-void select_tool::MotionEvent(ogl_view *, i32s, i32s)
+void select_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 {
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 }
 
 /*################################################################################################*/
@@ -117,12 +122,8 @@ void zoom_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 		}
 	}
 	
-	enlevdiag_view * eldv = dynamic_cast<enlevdiag_view *>(oglv);
-	if (eldv != NULL)
-	{
-		eldv->scale += dist_sensitivity * eldv->scale * (fGL) (latest_y - y) / (fGL) eldv->size[1];
-		eldv->Update();
-	}
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->ZoomEvent(latest_y - y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -213,12 +214,8 @@ void translate_xy_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 		}
 	}
 	
-	enlevdiag_view * eldv = dynamic_cast<enlevdiag_view *>(oglv);
-	if (eldv != NULL)
-	{
-		eldv->center -= 0.5 * eldv->scale * (fGL) (latest_y - y) / (fGL) eldv->size[1];
-		eldv->Update();
-	}
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->TransEvent(latest_y - y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -366,6 +363,9 @@ void orbit_xy_tool::ButtonEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -391,6 +391,9 @@ void orbit_xy_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -450,6 +453,9 @@ void orbit_z_tool::ButtonEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -474,6 +480,9 @@ void orbit_z_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -533,6 +542,9 @@ void rotate_xy_tool::ButtonEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -558,6 +570,9 @@ void rotate_xy_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -617,6 +632,9 @@ void rotate_z_tool::ButtonEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
@@ -641,6 +659,9 @@ void rotate_z_tool::MotionEvent(ogl_view * oglv, i32s x, i32s y)
 			}
 		}
 	}
+
+	plotting_view * pv = dynamic_cast<plotting_view *>(oglv);
+	if (pv != NULL) pv->UserEvent(x, y);
 	
 	latest_x = x;
 	latest_y = y;
