@@ -27,8 +27,6 @@ color_surface::color_surface(cs_param & p1)
 	transparent = p1.transparent;
 	wireframe = p1.wireframe;
 	
-	ostr = p1.ostr;
-	
 	for (i32s n1 = 0;n1 < 3;n1++)
 	{
 		np[n1] = p1.np[n1];
@@ -64,16 +62,18 @@ color_surface::~color_surface(void)
 
 void color_surface::Update(void)
 {
+	char mbuff1[256];
+	
 	xdir = data->ydir.vpr(data->zdir);
 	
 	vdata.resize(0);
 	tdata.resize(0);
 	ldata.resize(0);
 	
-	if (ostr != NULL) (* ostr) << "evaluating grid" << flush;
+	docv->PrintToLog("evaluating grid");
 	for (i32s n1 = 0;n1 < np[0];n1++)
 	{
-		if (ostr != NULL) (* ostr) << "." << flush;
+		docv->PrintToLog(".");
 		
 		for (i32s n2 = 0;n2 < np[1];n2++)
 		{
@@ -85,40 +85,42 @@ void color_surface::Update(void)
 		}
 	}
 	
-	if (ostr != NULL) (* ostr) << " done" << endl;
+	docv->PrintToLog(" done\n");
 	
 	for (i32s n1 = 0;n1 < 3;n1++)
 	{
 		i32s tmp1[2]; i32s tmp2[2] = { 0, 0 };
-		if (ostr != NULL) (* ostr) << "interpolating in ";
+		docv->PrintToLog("interpolating in ");
 		switch (n1)
 		{
 			case 0:
 			tmp1[0] = 1; tmp1[1] = 2;
-			if (ostr != NULL) (* ostr) << "x";
+			docv->PrintToLog("x");
 			break;
 			
 			case 1:
 			tmp1[0] = 2; tmp1[1] = 0;
-			if (ostr != NULL) (* ostr) << "y";
+			docv->PrintToLog("y");
 			break;
 			
 			case 2:
 			tmp1[0] = 0; tmp1[1] = 1;
-			if (ostr != NULL) (* ostr) << "z";
+			docv->PrintToLog("z");
 			break;
 		}
 		
-		if (ostr != NULL) (* ostr) << "-direction... " << flush;
+		docv->PrintToLog("-direction... ");
 		
 		range[n1][0] = vdata.size();
 		Interpolate(n1, tmp1, tmp2);
 		range[n1][1] = vdata.size();
 		
-		if (ostr != NULL) (* ostr) << tmp2[0] << " points, " << tmp2[1] << " cycles" << endl;
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << tmp2[0] << " points, " << tmp2[1] << " cycles" << endl << ends;
+		docv->PrintToLog(mbuff1);
 	}
 	
-	if (ostr != NULL) (* ostr) << "generating image... " << flush;
+	docv->PrintToLog("generating image... ");
 	
 	for (i32s n1 = 0;n1 < np[0] - 1;n1++)
 	{
@@ -148,7 +150,7 @@ void color_surface::Update(void)
 								{
 					i32s tmp1 = min(id1[0], id1[n7 + 1]); id2[n7] = range[n7][0];
 					while (id2[n7] < range[n7][1] && vdata[id2[n7]].id != tmp1) id2[n7]++;
-					if (id2[n7] == range[n7][1] && ostr != NULL) (* ostr) << "color_surface problem #1" << endl;
+					if (id2[n7] == range[n7][1]) cout << "color_surface problem #1" << endl;
 								}
 							}
 							
@@ -198,7 +200,7 @@ void color_surface::Update(void)
 						if (grid[id1a[n5]] != grid[id1b[n5]])
 						{
 			id2[n5] = range[n4][0]; while (id2[n5] < range[n4][1] && vdata[id2[n5]].id != id1a[n5]) id2[n5]++;
-			if (id2[n5] == range[n4][1] && ostr != NULL) (* ostr) << "color_surface problem #2" << endl;
+			if (id2[n5] == range[n4][1]) cout << "color_surface problem #2" << endl;
 						}
 					}
 					
@@ -297,7 +299,9 @@ void color_surface::Update(void)
 		}
 	}
 	
-	if (ostr != NULL) (* ostr) << ldata.size() << " lines, " << tdata.size() << " triangles." << endl;
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << ldata.size() << " lines, " << tdata.size() << " triangles." << endl << ends;
+	docv->PrintToLog(mbuff1);
 	
 	// add/update the primitives, if this is a transparent object...
 	// add/update the primitives, if this is a transparent object...
