@@ -412,7 +412,11 @@ void plot1d_view::UserEvent(i32s x, i32s)
 	mm1_docv * mm1dv = dynamic_cast<mm1_docv *>(mdl);
 	if (mm1dv != NULL)
 	{
-		cout << "tor = " << dv[index].c1 << " energy = " << dv[index].v << " kJ/mol" << endl;
+		char mbuff1[256];
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "tor = " << dv[index].c1 << " energy = " << dv[index].v << " kJ/mol" << endl << ends;
+		mdl->PrintToLog(mbuff1);
+		
 		mm1_apply_udata_as_cset(mm1dv, dv[index].udata);
 	}
 }
@@ -486,7 +490,7 @@ void plot2d_view::Render(void)
 
 	glInitNames();
 	
-	i32s zzz = (i32s) sqrt(dv.size());	// support square datasets only!!!
+	i32s zzz = (i32s) sqrt((double) dv.size());	// support square datasets only!!!
 	for (i32s n1 = 0;n1 < zzz - 1;n1++)
 	{
 		for (i32s n2 = 0;n2 < zzz - 1;n2++)
@@ -520,7 +524,12 @@ void plot2d_view::ZoomEvent(i32s dy)
 {
 	f64 zzz = maxv - minv;
 	maxv += mouse_tool::dist_sensitivity * zzz * (fGL) dy / (fGL) size[1];
-	cout << "now scaling the colours to range " << (maxv - minv) << " kJ/mol" << endl;
+	
+	char mbuff1[256];
+	ostrstream str1(mbuff1, sizeof(mbuff1));
+	str1 << "now scaling the colours to range " << (maxv - minv) << " kJ/mol" << endl << ends;
+	mdl->PrintToLog(mbuff1);
+	
 	Update();
 }
 
@@ -542,7 +551,7 @@ void plot2d_view::UserEvent(i32s x, i32s y)
 	if (sy < 0.0) return;
 	if (sy > 1.0) return;
 	
-	i32s zzz = (i32s) sqrt(dv.size());	// support square datasets only!!!
+	i32s zzz = (i32s) sqrt((double) dv.size());	// support square datasets only!!!
 	i32s index = (i32s) (((i32s) (sx * zzz)) * zzz + sy * zzz);
 	if (index >= (i32s) dv.size()) return;
 	
@@ -557,7 +566,11 @@ void plot2d_view::UserEvent(i32s x, i32s y)
 	mm1_docv * mm1dv = dynamic_cast<mm1_docv *>(mdl);
 	if (mm1dv != NULL)
 	{
-		cout << "tor1 = " << dv[index].c1 << " tor2 = " << dv[index].c2 << " energy = " << dv[index].v << " kJ/mol" << endl;
+		char mbuff1[256];
+		ostrstream str1(mbuff1, sizeof(mbuff1));
+		str1 << "tor1 = " << dv[index].c1 << " tor2 = " << dv[index].c2 << " energy = " << dv[index].v << " kJ/mol" << endl << ends;
+		mdl->PrintToLog(mbuff1);
+		
 		mm1_apply_udata_as_cset(mm1dv, dv[index].udata);
 	}
 }
@@ -769,6 +782,8 @@ glVertex3f(right - 0.3 * width + diagram_triangle_width, energy + one_pixel_heig
 		delete[] del_first;
 		delete[] del_count;
 	}
+	
+	delete[] label_color;
 }
 
 void enlevdiag_view::ZoomEvent(i32s dy)
