@@ -211,6 +211,9 @@ class graphics_view : virtual public ogl_view
 	friend class mm2_docv;
 	friend class glut_mm2_docv;
 	
+	friend class mm3_docv;
+	friend class glut_mm3_docv;
+	
 	friend class gnome_graphics_view;	// attach/detach needs this...
 	
 	public:
