@@ -11,11 +11,14 @@
 #include "notice.h"
 
 #include "mm1mdl.h"
-#include "mm1eng9.h"
+#include "mm1eng1.h"
 #include "mm1alg.h"
 
 /*################################################################################################*/
 
+//#include "t1m_mm1param_cpp.h"
+//#include "t1m_mm2param_cpp.h"
+
 int main(int, char **)
 {
 	print_full_copyright_notice(cout);
@@ -31,7 +34,7 @@ int main(int, char **)
 	cout << "trying to open a file... ";
 	
 	ifstream ifile;
-	ifile.open("examples/2-chlorobutane.mm1gp", ios::in);
+	ifile.open("examples/2-chlorobutane.gpr", ios::in);
 	
 	if (ifile.fail())
 	{
@@ -51,7 +54,7 @@ int main(int, char **)
 	// if everything went fine, then create a computation engine object, initialize it,
 	// and create a geometry optimizer object using the computation engine object:
 	
-	mm1_eng * eng = new mm1_eng_exp9(* mdl);
+	mm1_eng * eng = new mm1_eng_exp1(* mdl);
 	
 	CopyCRD(mdl, eng, 0);
 	
