@@ -73,14 +73,14 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 	cpp.np = 30; cpp.docv = this; cpp.ref = (mm1_docv *) this;
 	
 	cs_param csp1;
-	csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-	csp1.docv = this; csp1.ref = (mm1_docv *) this; csp1.next = NULL;
+	csp1.np = sz; csp1.dim = dm; csp1.docv = this;
+	csp1.ref = (mm1_docv *) this; csp1.next = NULL;
 	
 	cs_param csp2a; cs_param csp2b;
-	csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-	csp2a.docv = this; csp2a.ref = (mm1_docv *) this; csp2a.next = & csp2b;
-	csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-	csp2b.docv = this; csp2b.ref = (mm1_docv *) this; csp2b.next = NULL;
+	csp2a.np = sz; csp2a.dim = dm; csp2a.docv = this;
+	csp2a.ref = (mm1_docv *) this; csp2a.next = & csp2b;
+	csp2b.np = sz; csp2b.dim = dm; csp2b.docv = this;
+	csp2b.ref = (mm1_docv *) this; csp2b.next = NULL;
 	
 	mm1_geomopt_param goparam;
 	mm1_moldyn_param mdparam;
@@ -99,20 +99,20 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		cout << "i         -> import ENT files" << endl;
 		cout << "O         -> open a file" << endl;
 		cout << "S         -> save to a file" << endl;
+
+		cout << "s         -> process a command string" << endl;
 		
 		cout << "H         -> hydrogens, add or remove" << endl;
 		cout << "b B       -> build sequences (amino/nucleic acids)" << endl;
 		cout << "p         -> add (or \"push\") a new coordinate set" << endl;
 		cout << "0         -> superimpose the crd-sets and measure RMS-error" << endl;
 		
-//	cout << "W         -> change the default_eng" << endl;
+		cout << "W         -> change the default_eng" << endl;
 		cout << "z         -> compare numerical and analytical gradient" << endl;
 		cout << "E         -> compute energy [kJ/mol]" << endl;
 		cout << "o         -> geometry optimization..." << endl;
 		cout << "d         -> molecular dynamics..." << endl;
 		cout << "R         -> random conformational search" << endl;
-		cout << "f         -> PLOT 1D" << endl;
-		cout << "F         -> PLOT 2D" << endl;
 
 		cout << "t         -> build a protein ribbon model" << endl;
 		cout << "e w n m   -> make some nice planes/surfaces..." << endl;
@@ -216,7 +216,7 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		break;
 		
 		case 'i':
-		cout << "name of ENT file to import ??? "; cin >> txtbuff;
+		cout << "name of ENT file to import ??? "; cin.getline(txtbuff, sizeof(txtbuff));
 		
 		mdata = readpdb_ReadMData(txtbuff);
 		cout << "give the index of chain to be imported, or -1 to import all ??? "; cin >> n1;
@@ -226,15 +226,20 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		break;
 		
 		case 'O':
-		cout << "name of file to open ??? "; cin >> buffer;
+		cout << "name of file to open ??? "; cin.getline(buffer, sizeof(buffer));
 		ifile.open(buffer, ios::in); ReadStream(ifile); ifile.close();
 		UpdateAllGraphicsViews();
 		break;
 		
 		case 'S':
-		cout << "name of file to save ??? "; cin >> buffer;
+		cout << "name of file to save ??? "; cin.getline(buffer, sizeof(buffer));
 		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
 		break;
+
+		case 's':
+		cout << "command string ??? "; cin.getline(buffer, sizeof(buffer));
+		ProcessCommandString(gv, buffer);
+		break;
 		
 		case 'H':
 		cout << "hydrogens, add or remove ? (a/r) ";
@@ -269,7 +274,7 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		case 'p':
 		PushCRDSets(1); t1a = cs_vector.size() - 1;
 		while (t1a > 0) { CopyCRDSet(t1a - 1, t1a); t1a -= 1; }
-		cs_vector.back().visible = true;
+		cs_vector.back()->visible = true;
 		UpdateAllGraphicsViews();
 		break;
 		
@@ -289,10 +294,10 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		UpdateAllGraphicsViews();
 		break;
 		
-	//	case 'W':
-	//	default_eng++; if (default_eng > 1) default_eng = 0;
-	//	cout << "default_eng = " << engtab1[default_eng] << endl;
-	//	break;
+		case 'W':
+		default_eng++; if (engtab1[default_eng] == NULL) default_eng = 0;
+		cout << "default_eng = " << engtab1[default_eng] << endl;
+		break;
 		
 		case 'z':
 /*	{	//LOCAL
@@ -329,15 +334,20 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		break;
 		
 		case 'R':
-		DoRandomSearch();
-		break;
+		i32s cycles; i32s optsteps;
+		cout << "give number of cycles (100 would be ok) or -1 to cancel : "; cin >> cycles; if (cycles < 0) return;
+		cout << "give number of optimization steps in a cycle (250 would be ok,\nor 0 to skip optimization) : "; cin >> optsteps;
+		DoRandomSearch(cycles, optsteps);
 		
-		case 'f':
-		DoEnergyPlot1D();
-		break;
+	/*	i32s divisions; i32s optsteps;
+		cout << "give number of divisions (for example 6 divisions ->\n360 deg / 6 = 60 deg) or -1 to cancel : "; cin >> divisions; if (divisions < 0) return;
+		cout << "give number of optimization steps in a cycle (250 would be ok,\nor 0 to skip optimization) : "; cin >> optsteps;
+		DoSystematicSearch(divisions, optsteps);	*/
 		
-		case 'F':
-		DoEnergyPlot2D();
+	/*	i32s n_init_steps; i32s n_simul_steps; i32s optsteps;
+		cout << "give number of init and simul steps (10 100 would be ok)\nor -1 to cancel : "; cin >> n_init_steps; cin >> n_simul_steps; if (n_init_steps < 0 || n_simul_steps < 0) return;
+		cout << "give number of optimization steps in a cycle (250 would be ok,\nor 0 to skip optimization) : "; cin >> optsteps;
+		DoMonteCarloSearch(n_init_steps, n_simul_steps, optsteps);	*/
 		break;
 		
 		case 't':
@@ -348,8 +358,10 @@ void glut_mm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 			vector<mm1_chn_info> & ci_vector = (* this->ref_civ);
 			for (i32u n1 = 0;n1 < ci_vector.size();n1++)
 			{
+				if (ci_vector[n1].type != mm1_chn_info::amino_acid) continue;
 				if (ci_vector[n1].length < 3) continue;
-				else AddObject(new mm1_ribbon(this, gv->colormode, n1, 4));	// min. order is 2!!!
+				
+				AddObject(new mm1_ribbon(this, gv->colormode, n1, 4));		// min. order is 2!!!
 			}
 		}
 		
