@@ -39,7 +39,7 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 	i32s t1a; i32s t1b;
 	fGL t2a; fGL t2b;
 	char input;
-		
+	
 	ifstream ifile;
 	ofstream ofile;
 	
@@ -55,14 +55,14 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 	cpp.np = 30; cpp.docv = this; cpp.ref = (mm2_docv *) this;
 	
 	cs_param csp1;
-	csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-	csp1.docv = this; csp1.ref = (mm2_docv *) this; csp1.next = NULL;
+	csp1.np = sz; csp1.dim = dm; csp1.docv = this;
+	csp1.ref = (mm2_docv *) this; csp1.next = NULL;
 	
 	cs_param csp2a; cs_param csp2b;
-	csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-	csp2a.docv = this; csp2a.ref = (mm2_docv *) this; csp2a.next = & csp2b;
-	csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-	csp2b.docv = this; csp2b.ref = (mm2_docv *) this; csp2b.next = NULL;
+	csp2a.np = sz; csp2a.dim = dm; csp2a.docv = this;
+	csp2a.ref = (mm2_docv *) this; csp2a.next = & csp2b;
+	csp2b.np = sz; csp2b.dim = dm; csp2b.docv = this;
+	csp2b.ref = (mm2_docv *) this; csp2b.next = NULL;
 
 	mm2_geomopt_param goparam;
 	mm2_moldyn_param mdparam;
@@ -72,15 +72,18 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		case 'h':
 		cout << "C         -> change color modes" << endl;
 		cout << "r         -> change the rendering mode (without updating view)" << endl;
+		cout << "Q         -> change labels" << endl;
 		
 		cout << "i         -> import ENT-files (Brookhaven PDB)" << endl;
 		cout << "O         -> open a file" << endl;
 		cout << "S         -> save to a file" << endl;
+
+		cout << "s         -> process a command string" << endl;
 		
 		cout << "p         -> add (or \"push\") a new coordinate set" << endl;
 		cout << "0         -> superimpose the crd-sets and measure RMS-error" << endl;
-		cout << "Q         -> change constraints for selected VA's" << endl;
-//	cout << "W         -> update the constraints" << endl;
+		cout << "T         -> change constraints for selected VA's" << endl;
+		cout << "W         -> update the constraints" << endl;
 		
 		cout << "z         -> compare numerical and analytical gradient" << endl;
 		cout << "E         -> compute energy [kJ/mol]" << endl;
@@ -139,6 +142,29 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		cout << endl;
 		break;
 		
+		case 'Q':
+		cout << "label = ";
+		switch (gv->label)
+		{
+			case LABEL_NOTHING:
+			gv->label = LABEL_RESIDUE;
+			cout << "LABEL_RESIDUE";
+			break;
+			
+		// LABEL_INDEX to display charges???
+			case LABEL_RESIDUE:
+			gv->label = LABEL_INDEX;
+			cout << "LABEL_INDEX";
+			break;
+			
+			case LABEL_INDEX:
+			gv->label = LABEL_NOTHING;
+			cout << "LABEL_NOTHING";
+			break;
+		}
+		cout << endl;
+		break;
+		
 		case 'i':
 		cout << "import ENT: filename ??? "; cin >> buffer;
 		cout << "give the index of chain to be imported, or -1 to import all ??? "; cin >> n1;
@@ -146,20 +172,25 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		break;
 		
 		case 'O':
-		cout << "name of file to open ??? "; cin >> buffer;
+		cout << "name of file to open ??? "; cin.getline(buffer, sizeof(buffer));
 		ifile.open(buffer, ios::in); ReadStream(ifile); ifile.close();
 		UpdateAllGraphicsViews();
 		break;
 		
 		case 'S':
-		cout << "name of file to save ??? "; cin >> buffer;
+		cout << "name of file to save ??? "; cin.getline(buffer, sizeof(buffer));
 		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
 		break;
+
+		case 's':
+		cout << "command string ??? "; cin.getline(buffer, sizeof(buffer));
+		ProcessCommandString(gv, buffer);
+		break;
 		
 		case 'p':
 		PushCRDSets(1); t1a = cs_vector.size() - 1;
 		while (t1a > 0) { CopyCRDSet(t1a - 1, t1a); t1a -= 1; }
-		cs_vector.back().visible = true;
+		cs_vector.back()->visible = true;
 		UpdateAllGraphicsViews();
 		break;
 		
@@ -179,7 +210,7 @@ void glut_mm2_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		UpdateAllGraphicsViews();
 		break;
 		
-		case 'Q':
+		case 'T':
 		for (n1 = 0;n1 < (i32s) chn_vector.size();n1++)
 		{
 			for (n2 = 0;n2 < (i32s) chn_vector[n1].res_vector.size();n2++)
