@@ -45,14 +45,14 @@ void glut_qm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 	cpp.np = 15; cpp.docv = this; cpp.ref = current_eng;
 	
 	cs_param csp1;
-	csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-	csp1.docv = this; csp1.ref = current_eng; csp1.next = NULL;
+	csp1.np = sz; csp1.dim = dm; csp1.docv = this;
+	csp1.ref = current_eng; csp1.next = NULL;
 	
 	cs_param csp2a; cs_param csp2b;
-	csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-	csp2a.docv = this; csp2a.ref = current_eng; csp2a.next = & csp2b;
-	csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-	csp2b.docv = this; csp2b.ref = current_eng; csp2b.next = NULL;
+	csp2a.np = sz; csp2a.dim = dm; csp2a.docv = this;
+	csp2a.ref = current_eng; csp2a.next = & csp2b;
+	csp2b.np = sz; csp2b.dim = dm; csp2b.docv = this;
+	csp2b.ref = current_eng; csp2b.next = NULL;
 	
 	char input;
 
@@ -68,6 +68,9 @@ void glut_qm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		cout << "r         -> change rendering modes (without updating view)" << endl;
 		
 		cout << "i         -> import MM1-files" << endl;
+		
+		cout << "s         -> process a command string" << endl;
+		
 		cout << "p         -> add (or \"push\") a new coordinate set" << endl;
 		
 		cout << "W         -> change the default_eng" << endl;
@@ -128,17 +131,19 @@ void glut_qm1_docv::KeyEvent(graphics_view * gv, char key, int, int)
 		break;
 		
 		case 'i':
-		cout << "name of file to import ??? ";
-		cin >> buffer;
-		
-		ImportMM1(buffer);
-		UpdateAllGraphicsViews();
+		cout << "name of file to import ??? "; cin.getline(buffer, sizeof(buffer));
+		ImportMM1(buffer); UpdateAllGraphicsViews();
+		break;
+
+		case 's':
+		cout << "command string ??? "; cin.getline(buffer, sizeof(buffer));
+		ProcessCommandString(gv, buffer);
 		break;
 		
 		case 'p':
 		PushCRDSets(1); t1a = cs_vector.size() - 1;
 		while (t1a > 0) { CopyCRDSet(t1a - 1, t1a); t1a -= 1; }
-		cs_vector.back().visible = true;
+		cs_vector.back()->visible = true;
 		UpdateAllGraphicsViews();
 		break;
 		
