@@ -50,7 +50,10 @@ void gnome_docv::InitGnomeMDIChild(void)
 	}
 	
 	child = gnome_mdi_generic_child_new(GetFullProjectFileName());
-	gtk_object_set_user_data(GTK_OBJECT(child), (docview *) this);
+	
+	gpointer userdata = (docview *) this;
+	gtk_object_set_user_data(GTK_OBJECT(child), userdata);
+	//cout << "SET_USER_DATA " << userdata << endl;
 	
 	gtk_signal_connect(GTK_OBJECT(child), "destroy", GTK_SIGNAL_FUNC(gnome_mdi_app::DestroyChildHandler), NULL);
 	
@@ -67,6 +70,16 @@ void gnome_docv::InitGnomeMDIChild(void)
 	
 	CreateTreeListView();
 	AddGraphicsView();
+	
+	char messagebuffer[256];
+	const char * ext = GetProjectFileNameExtension();
+	ostrstream str(messagebuffer, sizeof(messagebuffer));
+	str << "A new project created; " << ext << " (";
+	if (!strcmp("mm2gp", ext)) str << "simplified forcefield)." << endl << ends;
+	else if (dynamic_cast<qm1_mdl *>(this) != NULL) str << "quantum mechanics)." << endl << ends;
+	else if (dynamic_cast<mm1_mdl *>(this) != NULL) str << "molecular mechanics)." << endl << ends;
+	else str << "???)." << endl << ends;
+	PrintToLog(messagebuffer);
 }
 
 char * gnome_docv::GetConfigString(GnomeMDIChild * child, gpointer)
@@ -76,11 +89,37 @@ char * gnome_docv::GetConfigString(GnomeMDIChild * child, gpointer)
 
 GtkWidget * gnome_docv::GnomeViewCreator(GnomeMDIChild * child, gpointer)
 {
-	docview * docv1 = (docview *) gtk_object_get_user_data(GTK_OBJECT(child));
+	gpointer userdata = gtk_object_get_user_data(GTK_OBJECT(child));
+	//cout << "GET_USER_DATA " << userdata << endl;
+	
+	docview * docv1 = (docview *) userdata;
 	gnome_docv * docv2 = dynamic_cast<gnome_docv *>(docv1);
 	
+	if (docv2 == NULL)
+	{
+		cout << "fatal error : cast failed at gnome_docv::GnomeViewCreator()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	// the viewing widget is a vertically paned widget.
+	// it contains the notebook and the text log.
+	
+	docv2->paned_widget = GTK_WIDGET(gtk_vpaned_new());
+	
 	docv2->notebook = gtk_notebook_new();
-	return docv2->notebook;
+	
+	docv2->logpos = 0;	// the position counter for gtk_editable_insert_text.
+	docv2->scroll_widget = gtk_scrolled_window_new(NULL, NULL);
+//docv2->log_buffer = gtk_text_buffer_new(NULL);	in GTK2
+//gtk_text_view_new_with_buffer(docv2->log_buffer);	in GTK2
+	docv2->log_widget = gtk_text_new(NULL, NULL);
+	gtk_container_add(GTK_CONTAINER(docv2->scroll_widget), GTK_WIDGET(docv2->log_widget));
+	gtk_widget_show(GTK_WIDGET(docv2->log_widget)); gtk_widget_show(GTK_WIDGET(docv2->scroll_widget));
+	
+	gtk_paned_add1(GTK_PANED(docv2->paned_widget), GTK_WIDGET(docv2->notebook));
+	gtk_paned_add2(GTK_PANED(docv2->paned_widget), GTK_WIDGET(docv2->scroll_widget));
+	
+	return docv2->paned_widget;
 }
 
 void gnome_docv::UpdateAllWindowTitles()
@@ -88,7 +127,15 @@ void gnome_docv::UpdateAllWindowTitles()
 	// mdi_child_name and view titles are somewhat different issues now; check this!!!
 	// we should change mdi_child_name when the project name changes, but this function is for setting the view titles.
 	
-	gnome_mdi_child_set_name(GNOME_MDI_CHILD(child), GetProjectFileName(true));
+	static char childnamebuff[256];
+	ostrstream cnstr(childnamebuff, sizeof(childnamebuff));
+	
+	cnstr << GetProjectFileName(true);
+	if (dynamic_cast<qm1_mdl *>(this) != NULL) cnstr << " (a QM project)";
+	if (dynamic_cast<mm1_mdl *>(this) != NULL) cnstr << " (a MM project)";
+	cnstr << ends;
+	
+	gnome_mdi_child_set_name(GNOME_MDI_CHILD(child), childnamebuff);
 	docview::UpdateAllWindowTitles();
 	
 	event_SelectedObjectChanged();		// update the statusbar too
@@ -115,6 +162,11 @@ void gnome_docv::event_SelectedObjectChanged(void)
 	}
 }
 
+void gnome_docv::PrintToLog(const char * message)
+{
+	gtk_editable_insert_text(GTK_EDITABLE(log_widget), message, strlen(message), & logpos);
+}
+
 /*################################################################################################*/
 
 // eof
