@@ -45,7 +45,11 @@ class gnome_docv : virtual public docview
 	
 	GnomeMDIGenericChild * child;
 	
+	GtkWidget * paned_widget;
 	GtkWidget * notebook;
+	//GtkTextBuffer * log_buffer;	// in GTK2
+	GtkWidget * scroll_widget; GtkWidget * log_widget; int logpos;
+	
 	GtkWidget * popupmenu;
 	
 	friend class gnome_class_factory;
@@ -59,6 +63,8 @@ class gnome_docv : virtual public docview
 	friend class gnome_mm1_docv;	// soon to be obsolete???
 //	friend class gnome_mm2_docv;	// is obsolete
 	
+	friend class qm1_file_open_dialog;
+	friend class qm1_file_save_dialog;
 	friend class mm1_file_open_dialog;
 	friend class mm1_file_save_dialog;
 	
@@ -84,6 +90,8 @@ class gnome_docv : virtual public docview
 	
 	void UpdateAllWindowTitles(void);		// virtual
 	void event_SelectedObjectChanged(void);		// virtual
+	
+	void PrintToLog(const char *);			// virtual
 };
 
 /*################################################################################################*/
