@@ -7,16 +7,20 @@
 
 /*################################################################################################*/
 
+#define ENABLE_NLS	// supress the "gcc-3.2 and libintl.h" problem...
 #include "t3main.h"
 
 #include "t3errutil.h"
 #include "t3dialog.h"
 #include "notice.h"
 
+//#include <popt.h>			// For parsing command-line; OLD
+//#include <libgnome/gnome-popt.h>	// For parsing command-line; OLD
+#include <popt-gnome.h>			// For parsing command-line.
+
+#include <glade/glade.h>
+
 #include <strstream>
-//#include <popt.h>		// For parsing command-line
-#include <popt-gnome.h>			// For parsing command-line (Debian 2.2r3???)
-//#include <libgnome/gnome-popt.h>	// For parsing command-line (Debian 2.2r3???)
 using namespace std;
 
 /*################################################################################################*/
@@ -27,13 +31,15 @@ i32s gnome_app_data::current_tool_index = 6;	// 6 is the index for orbit_xy tool
 
 GnomeUIInfo gnome_mdi_app::file_new_submenu[] =
 {
-	GNOMEUIINFO_MENU_NEW_ITEM("qm1gp - quantum mechanics", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Quantum Mechanics (.gpr)", NULL,
 		gnome_mdi_app::menu_FileNewQM1, NULL),
 	
-	GNOMEUIINFO_MENU_NEW_ITEM("mm1gp - molecular mechanics", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Molecular Mechanics (.gpr)", NULL,
 		gnome_mdi_app::menu_FileNewMM1, NULL),
+
+	GNOMEUIINFO_SEPARATOR,
 	
-	GNOMEUIINFO_MENU_NEW_ITEM("mm2gp - reduced protein model", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Simplified Protein Model (.mm2gp)", NULL,
 		gnome_mdi_app::menu_FileNewMM2, NULL),
 	
 	GNOMEUIINFO_END
@@ -41,13 +47,15 @@ GnomeUIInfo gnome_mdi_app::file_new_submenu[] =
 
 GnomeUIInfo gnome_mdi_app::file_open_submenu[] =
 {
-	GNOMEUIINFO_MENU_NEW_ITEM("qm1gp - quantum mechanics", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Quantum Mechanics (.gpr)", NULL,
 		gnome_mdi_app::menu_FileOpenQM1, NULL),
 	
-	GNOMEUIINFO_MENU_NEW_ITEM("mm1gp - molecular mechanics", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Molecular Mechanics (.gpr)", NULL,
 		gnome_mdi_app::menu_FileOpenMM1, NULL),
+
+	GNOMEUIINFO_SEPARATOR,
 		
-	GNOMEUIINFO_MENU_NEW_ITEM("mm2gp - reduced protein model", NULL,
+	GNOMEUIINFO_MENU_NEW_ITEM("Simplified Protein Model (.mm2gp)", NULL,
 		gnome_mdi_app::menu_FileOpenMM2, NULL),
 		
 	GNOMEUIINFO_END
@@ -260,8 +268,10 @@ gnome_mdi_app::gnome_mdi_app(void)
 	// setup mdi-menubar...
 	
 	gnome_mdi_set_menubar_template(mdi, menubar_info);
-	gnome_mdi_set_child_menu_path(mdi, GNOME_MENU_FILE_STRING);
-	gnome_mdi_set_child_list_path(mdi, GNOME_MENU_WINDOWS_PATH);
+//	gnome_mdi_set_child_menu_path(mdi, GNOME_MENU_FILE_STRING);
+//	gnome_mdi_set_child_list_path(mdi, GNOME_MENU_WINDOWS_PATH);
+	gnome_mdi_set_child_menu_path(mdi, "_File");
+	gnome_mdi_set_child_list_path(mdi, "_Windows/");
 	
 	// ready...
 	
@@ -305,6 +315,8 @@ void gnome_mdi_app::HelpAbout(void)
 		"Tommi Hassinen",
 		"Geoff Hutchison",
 		"Mike Cruz",
+		"Michael Banck",
+		"Christopher Rowley",
 		NULL
 	};
 	
@@ -340,25 +352,17 @@ void gnome_mdi_app::menu_FileNewMM2(GtkWidget *, gpointer)
 
 void gnome_mdi_app::menu_FileOpenQM1(GtkWidget *, gpointer)
 {
-err_util * err = gnome_class_factory::GetInstance()->ProduceErrUtil();
-err->Message("Please remember, that the preferred way of using QM models\nis to start with MM model and then convert it to QM model!");
-delete err;
-
-menu_FileNewQM1(NULL, NULL);	// just open an empty file, until we have specified the file format here...
-
-	// the above message is not displayed???
-	// this is not very critical message, but perhaps the err_util dialogs should be forced to modal ones?!?!?!
-//	new qm1_file_open_dialog(NULL);		// memory leak???	THIS IS NOT YET READY...
+	new qm1_file_open_dialog(NULL);
 }
 
 void gnome_mdi_app::menu_FileOpenMM1(GtkWidget *, gpointer)
 {
-	new mm1_file_open_dialog(NULL);		// memory leak???
+	new mm1_file_open_dialog(NULL);
 }
 
 void gnome_mdi_app::menu_FileOpenMM2(GtkWidget *, gpointer)
 {
-	new mm2_file_open_dialog(NULL);		// memory leak???
+	new mm2_file_open_dialog(NULL);
 }
 	
 void gnome_mdi_app::menu_FileExit(GtkWidget *, gpointer)
@@ -520,12 +524,12 @@ void gnome_mdi_app::ToolButtonHandler(i32s tool_index)
 
 void gnome_mdi_app::shortcut_Element(GtkWidget *, gpointer)
 {
-	new element_dialog();		// memory leak???
+	new element_dialog();		// will call delete itself...
 }
 
 void gnome_mdi_app::shortcut_BondType(GtkWidget *, gpointer)
 {
-	new bondtype_dialog();		// memory leak???
+	new bondtype_dialog();		// will call delete itself...
 }
 
 // this same operation is done separately for the popup menu!!!
@@ -690,16 +694,18 @@ char *importname = NULL;
 
 int main(int argc, char ** argv)
 {
-        char **args;
+        //char **args;
         poptContext options_context;
         const struct poptOption options_table[] = {
 	  {"file",'f', POPT_ARG_STRING, &filename, 0, "open file", "filename"},
-	  {"import",'i', POPT_ARG_STRING, &importname, 0, "import file as mm1gp", "filename"},
+	  {"import",'i', POPT_ARG_STRING, &importname, 0, "import file as gpr", "filename"},
 	  {NULL,'\0', 0, NULL, 0, NULL, NULL}
 	};
 	options_context = poptGetContext(NULL, argc, argv, options_table, 0);
 	gnome_init_with_popt_table("ghemical", VERSION, argc, argv, options_table, 0, &options_context);
 
+	glade_gnome_init();
+
 	app_cleaner.SetInstance(gnome_mdi_app::GetInstance());
 
 	if (filename != NULL && strlen(filename) != 0)
@@ -708,7 +714,7 @@ int main(int argc, char ** argv)
 	    docv = new gnome_mm1_docv(& cout, * gnome_class_factory::GetInstance());
 	    ifstream ifile;
 	    ifile.open(filename, ios::in);
-	    if (docv->ReadStream(ifile) != false)
+	    if (docv->ReadStream(ifile, false) != false)
 	      {
 		docv->InitGnomeMDIChild();
 		docv->ParseProjectFileNameAndPath(filename);
