@@ -9,7 +9,13 @@
 
 #include "t3mm1docv.h"
 
-#include "glade_interface.h"
+#include "file_import_dialog.h"
+#include "file_export_dialog.h"
+
+#include "setup_mm1_dialog.h"
+#include "geomopt_dialog.h"
+#include "moldyn_dialog.h"
+#include "trajview_dialog.h"
 
 #include "t3main.h"
 #include "t3dialog.h"
@@ -39,7 +45,7 @@ using namespace std;
 
 /*################################################################################################*/
 
-const char mm1_file_open_dialog::title[] = "Open Project File - mm1gp";
+const char mm1_file_open_dialog::title[] = "Open Project File";
 gnome_mm1_docv * mm1_file_open_dialog::docv = NULL;
 
 mm1_file_open_dialog::mm1_file_open_dialog(gnome_mm1_docv * p1) : file_dialog(title, NULL, NULL)
@@ -92,7 +98,7 @@ void mm1_file_open_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
-const char mm1_file_save_dialog::title[] = "Save Project File - mm1gp";
+const char mm1_file_save_dialog::title[] = "Save Project File";
 gnome_mm1_docv * mm1_file_save_dialog::docv = NULL;
 
 mm1_file_save_dialog::mm1_file_save_dialog(gnome_mm1_docv * p1) :
@@ -185,10 +191,11 @@ bool mm1_trajfile_dialog::OkEvent(const char * filename)
 	
 	gnome_docv::glade_modal_dialog_owner = (gnome_mm1_docv *) docv;		// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_t1_dialog_trajectory();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new trajview_dialog();	// will call delete and CloseTrajectory!
+	//GtkWidget * dialog = create_t1_dialog_trajectory();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
+	//docv->CloseTrajectory();			// is handled by dialog...
 	
-	docv->CloseTrajectory();
 	return false;
 }
 
@@ -198,6 +205,51 @@ void mm1_trajfile_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
+const char mm1_importpdb_dialog::title[] = "Import File - Brookhaven PDB/ENT";
+gnome_mm1_docv * mm1_importpdb_dialog::docv = NULL;
+
+mm1_importpdb_dialog::mm1_importpdb_dialog(gnome_mm1_docv * p1) : file_dialog(title, NULL, NULL)
+{
+	docv = p1;
+}
+
+mm1_importpdb_dialog::~mm1_importpdb_dialog(void)
+{
+}
+
+bool mm1_importpdb_dialog::OkEvent(const char * filename)
+{
+	if (docv != NULL)
+	{
+		docv->err->WarningMessage("This alternative PDB reader is designed to remove any gaps that might exist in the structure data.\nTherefore the sequence might be altered! The process can also take several minutes to complete.");
+		
+		docv->ParseProjectFileNameAndPath(filename);
+		
+		mm1_readpdb_mdata * mdata = docv->readpdb_ReadMData(filename);
+		docv->readpdb_ReadData(filename, mdata, -1);
+		delete mdata;
+		
+		docv->UpdateAllWindowTitles();
+		docv->UpdateAllGraphicsViews();
+	}
+	
+	return true;
+}
+
+void mm1_importpdb_dialog::CancelEvent(void)
+{
+}
+
+/*################################################################################################*/
+
+GnomeUIInfo gnome_mm1_docv::filext_submenu[] =
+{
+	GNOMEUIINFO_ITEM("Import PDB...", "a special tool for importing PDB files.",
+		popup_FileExtra1, NULL),
+		
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo gnome_mm1_docv::file_submenu[] =
 {
 	GNOMEUIINFO_MENU_OPEN_ITEM(popup_FileOpen, NULL),
@@ -217,6 +269,10 @@ GnomeUIInfo gnome_mm1_docv::file_submenu[] =
 	
 	GNOMEUIINFO_SEPARATOR,
 	
+	GNOMEUIINFO_SUBTREE("Extra Tools", filext_submenu),
+
+	GNOMEUIINFO_SEPARATOR,
+	
 	GNOMEUIINFO_MENU_CLOSE_ITEM(gnome_graphics_view::popup_FileClose, NULL),
 	
 	GNOMEUIINFO_END
@@ -253,9 +309,9 @@ GnomeUIInfo gnome_mm1_docv::select_submenu[] =
 	GNOMEUIINFO_ITEM("Invert Selection", "invert the selection...",
 		gnome_graphics_view::popup_InvertSelection, NULL),
 	
-	GNOMEUIINFO_SEPARATOR,
-	
-	GNOMEUIINFO_SUBTREE("_Selection Mode", selmodes_submenu),
+//	GNOMEUIINFO_SEPARATOR,
+//	
+//	GNOMEUIINFO_SUBTREE("_Selection Mode", selmodes_submenu),	// move this stuff into post-v1.00 stage?!?
 	
 	GNOMEUIINFO_END
 };
@@ -304,10 +360,10 @@ GnomeUIInfo gnome_mm1_docv::lights_submenu[] =
 		
 	GNOMEUIINFO_SEPARATOR,
 	
-	GNOMEUIINFO_TOGGLEITEM("Display Local Lights", "loc",
+	GNOMEUIINFO_TOGGLEITEM("Enable Local Lights", "loc",
 		gnome_graphics_view::popup_LightsSwitchLoc, NULL),
 		
-	GNOMEUIINFO_TOGGLEITEM("Display Global Lights", "glob",
+	GNOMEUIINFO_TOGGLEITEM("Enable Global Lights", "glob",
 		gnome_graphics_view::popup_LightsSwitchGlob, NULL),
 		
 	GNOMEUIINFO_END
@@ -333,6 +389,17 @@ GnomeUIInfo gnome_mm1_docv::rendermode_style_radio[] =
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo gnome_mm1_docv::rendermode_color_radio[] =
+{
+	GNOMEUIINFO_ITEM("Color Elements", "color-el",
+		popup_CModeElement, NULL),
+
+	GNOMEUIINFO_ITEM("Color State", "color-st",
+		popup_CModeState, NULL),
+
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo gnome_mm1_docv::rendermode_label_radio[] =
 {
 	GNOMEUIINFO_ITEM("Label Nothing", "lab-null",
@@ -356,24 +423,13 @@ GnomeUIInfo gnome_mm1_docv::rendermode_label_radio[] =
 	GNOMEUIINFO_END
 };
 
-GnomeUIInfo gnome_mm1_docv::rendermode_color_radio[] =
-{
-	GNOMEUIINFO_ITEM("Color Elements", "color-el",
-		popup_CModeElement, NULL),
-
-	GNOMEUIINFO_ITEM("Color State", "color-st",
-		popup_CModeState, NULL),
-
-	GNOMEUIINFO_END
-};
-
 GnomeUIInfo gnome_mm1_docv::rendermode_submenu[] =
 {
         GNOMEUIINFO_RADIOLIST(rendermode_style_radio),
 	GNOMEUIINFO_SEPARATOR,
-        GNOMEUIINFO_RADIOLIST(rendermode_label_radio),
-	GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_RADIOLIST(rendermode_color_radio),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_RADIOLIST(rendermode_label_radio),
 	GNOMEUIINFO_END
 };
 
@@ -457,6 +513,12 @@ GnomeUIInfo gnome_mm1_docv::compute_submenu[] =
 	GNOMEUIINFO_ITEM("Random Conformational Search...", "...",
 		popup_CompRandomSearch, NULL),
 	
+	GNOMEUIINFO_ITEM("Systematic Conformational Search...", "...",
+		popup_CompSystematicSearch, NULL),
+	
+	GNOMEUIINFO_ITEM("Monte Carlo Search...", "...",
+		popup_CompMonteCarloSearch, NULL),
+	
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_ITEM("Plot Energy vs. 1 Torsion Angle...", "...",
@@ -495,8 +557,11 @@ GnomeUIInfo gnome_mm1_docv::build_submenu[] =
 	
 	GNOMEUIINFO_SUBTREE("Library", library_submenu),
 
-	GNOMEUIINFO_ITEM("Sequence Builder...", "...",
-		popup_Builder, NULL),
+	GNOMEUIINFO_ITEM("Sequence Builder (amino)...", "...",
+		popup_BuilderAmino, NULL),
+		
+	GNOMEUIINFO_ITEM("Sequence Builder (nucleic)...", "...",
+		popup_BuilderNucleic, NULL),
 		
 	GNOMEUIINFO_SEPARATOR,
 	
@@ -535,8 +600,11 @@ GnomeUIInfo gnome_mm1_docv::popupmenu_info[] =
 	GNOMEUIINFO_SEPARATOR,
 
 	GNOMEUIINFO_SUBTREE("_Build", build_submenu),
+
+	GNOMEUIINFO_ITEM("Enter a Command...", "Type in a command string and process it.",
+		popup_EnterCommand, NULL),
 	
-	GNOMEUIINFO_ITEM("Convert to QM project", "Convert this project into a qm1gp project.",
+	GNOMEUIINFO_ITEM("Convert to QM project", "Convert this project into a Quantum Mechanics project.",
 		popup_ConvertQM, NULL),
 	
 	GNOMEUIINFO_END
@@ -573,11 +641,11 @@ gnome_mm1_docv::gnome_mm1_docv(ostream * p1, gnome_class_factory & p2) :
   str << PRIMARY_PATH << "library" << DIR_SEPARATOR << ends;
   if ( stat(buffer, &stat_buf) == 0 && S_ISDIR(stat_buf.st_mode) )
     MakeLibraryWidgets(mi,
-		     model_prefs->Find("MM1Interface/LibraryDir", buffer),
+		     model_prefs->Find("MM1Interface/LibraryDir", buffer),		// memleak? never deleted???
 		     model_prefs->Value("MM1Interface/LibraryLevels", 1));
   else
     MakeLibraryWidgets(mi,
-		       model_prefs->Find("MM1Interface/LibraryDir", "library/"),
+		       model_prefs->Find("MM1Interface/LibraryDir", "library/"),	// memleak? never deleted???
 		       model_prefs->Value("MM1Interface/LibraryLevels", 1));
 }
 
@@ -596,7 +664,8 @@ void gnome_mm1_docv::popup_FileOpen(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
-	if (docv) new mm1_file_open_dialog(docv);	// memory leak???
+	
+	if (docv) new mm1_file_open_dialog(docv);
 }
 
 void gnome_mm1_docv::popup_FileImport(GtkWidget *, gpointer data)
@@ -605,8 +674,7 @@ void gnome_mm1_docv::popup_FileImport(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	glade_modal_dialog_owner = docv;	// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_f1_dialog_mm1_import();
-	gtk_widget_show(dialog);
+	if (docv) new file_import_dialog();
 }
 
 void gnome_mm1_docv::popup_FileExport(GtkWidget *, gpointer data)
@@ -615,22 +683,31 @@ void gnome_mm1_docv::popup_FileExport(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	glade_modal_dialog_owner = docv;	// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_f2_dialog_mm1_export();
-	gtk_widget_show(dialog);
+	if (docv) new file_export_dialog();
 }
 
 void gnome_mm1_docv::popup_FileExportGraphic(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
-	if (docv) new mm1_file_save_graphics_dialog(docv);	// memory leak???
+	if (docv) new mm1_file_save_graphics_dialog(docv);
 }
 
 void gnome_mm1_docv::popup_FileSaveAs(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
-	if (docv) new mm1_file_save_dialog(docv);	// memory leak???
+	if (docv) new mm1_file_save_dialog(docv);
+}
+
+void gnome_mm1_docv::popup_FileExtra1(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
+	if (docv)
+	{
+		new mm1_importpdb_dialog(docv);
+	}
 }
 
 void gnome_mm1_docv::popup_RModeWireFrame(GtkWidget *, gpointer data)
@@ -723,8 +800,10 @@ void gnome_mm1_docv::popup_ObjRibbon(GtkWidget *, gpointer data)
 		vector<mm1_chn_info> & ci_vector = (* docv->ref_civ);
 		for (i32u n1 = 0;n1 < ci_vector.size();n1++)
 		{
+			if (ci_vector[n1].type != mm1_chn_info::amino_acid) continue;
 			if (ci_vector[n1].length < 3) continue;
-			else docv->AddObject(new mm1_ribbon(docv, gv->colormode, n1, 4));	// min. order is 2!!!
+			
+			docv->AddObject(new mm1_ribbon(docv, gv->colormode, n1, 4));		// min. order is 2!!!
 		}
 		
 		docv->UpdateAllGraphicsViews();
@@ -745,18 +824,8 @@ void gnome_mm1_docv::popup_ObjEPlane(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("MM1Graphics/PlaneResolution", 30); 
-		cpp.docv = docv; cpp.ref = (mm1_docv *) docv;
-		
-		cout << "give dimension of the plane [nm] (1.0 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = (ColorFunction *) GetRBRange1;
-		cpp.value = 138.9354518 * 1.0; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane esp rb1 138.0 1.0 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -766,18 +835,8 @@ void gnome_mm1_docv::popup_ObjEVolume(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("MM1Graphics/SurfaceResolution", 30);
-		cpp.docv = docv; cpp.ref = (mm1_docv *) docv;
-		
-		cout << "give dimension of the volume [nm] (1.0 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = (ColorFunction *) GetRBRange2;
-		cpp.value = 138.9354518 * 1.0; cpp.alpha = 0.25;
-		
-		docv->AddObject(new volume_rendering_object(ol_static(), cpp, 20, tmp1 / 2.0, (* gv->cam), "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add volrend esp rb2 138.0 1.0 25 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -787,37 +846,8 @@ void gnome_mm1_docv::popup_ObjESurface(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
-		
-		i32s size = docv->model_prefs->Value("MM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp2a; cs_param csp2b;
-		csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-		csp2a.docv = docv; csp2a.ref = (mm1_docv *) docv; csp2a.next = & csp2b;
-		csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-		csp2b.docv = docv; csp2b.ref = (mm1_docv *) docv; csp2b.next = NULL;
-		
-		tmp1 = 138.9354518 / 4.0;
-		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp2a.wireframe = csp2b.wireframe = (input == 'w' || input == 'W');
-		
-		csp2a.transparent = !true;
-		csp2a.vf1 = (ValueFunction *) mm1_GetESPValue; csp2a.vf2 = (ValueFunction *) GetUnity;
-		csp2a.cvalue = 1.0; csp2a.alpha = 0.25; csp2a.toler = 1.0e-6 * tmp1; csp2a.maxc = 500;
-		csp2a.cf = (ColorFunction *) GetRedColor; csp2a.svalue = +tmp1;
-		
-		csp2b.transparent = !true;
-		csp2b.vf1 = (ValueFunction *) mm1_GetESPValue; csp2b.vf2 = (ValueFunction *) GetUnity;
-		csp2b.cvalue = 1.0; csp2b.alpha = 0.25; csp2b.toler = 1.0e-6 * tmp1; csp2b.maxc = 500;
-		csp2b.cf = (ColorFunction *) GetBlueColor; csp2b.svalue = -tmp1;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp2a, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf2 esp unity red blue +35.0 -35.0 1.0 2.0 50 0 0 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -827,29 +857,8 @@ void gnome_mm1_docv::popup_ObjVDWSurface(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
-		
-		i32s size = docv->model_prefs->Value("MM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp1;
-		csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-		csp1.docv = docv; csp1.ref = (mm1_docv *) docv; csp1.next = NULL;
-		
-		tmp1 = 138.9354518 / 2.0;
-		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp1.wireframe = (input == 'w' || input == 'W');
-		
-		csp1.transparent = true; csp1.cf = (ColorFunction *) GetRBRange1;
-		csp1.vf1 = (ValueFunction *) mm1_GetVDWSValue; csp1.vf2 = (ValueFunction *) mm1_GetESPValue;
-		csp1.svalue = 1.0; csp1.cvalue = tmp1; csp1.alpha = 0.50; csp1.toler = 1.0e-6 * tmp1; csp1.maxc = 500;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp1, "ESP-colored VDW-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf1 vdws esp rb1 1.0 70.0 2.0 50 1 1 0.65";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -859,8 +868,9 @@ void gnome_mm1_docv::popup_CompSetup(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	glade_modal_dialog_owner = docv;	// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_e2_dialog_mm1_setup();
-	gtk_widget_show(dialog);
+	new setup_mm1_dialog();
+	//GtkWidget * dialog = create_e2_dialog_mm1_setup();	// obsolete
+	//gtk_widget_show(dialog);				// obsolete
 }
 
 void gnome_mm1_docv::popup_CompEnergy(GtkWidget *, gpointer data)
@@ -906,7 +916,30 @@ void gnome_mm1_docv::popup_CompRandomSearch(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		docv->DoRandomSearch();
+		static const char command[] = "random_search 100 250";
+		new command_dialog(docv, gv, command);
+	}
+}
+
+void gnome_mm1_docv::popup_CompSystematicSearch(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
+	if (docv)
+	{
+		static const char command[] = "systematic_search 6 250";
+		new command_dialog(docv, gv, command);
+	}
+}
+
+void gnome_mm1_docv::popup_CompMonteCarloSearch(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
+	if (docv)
+	{
+		static const char command[] = "montecarlo_search 10 100 250";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -914,14 +947,26 @@ void gnome_mm1_docv::popup_CompEPlot1D(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
-	if (docv) docv->DoEnergyPlot1D();
+	if (docv)
+	{
+		docv->err->Message("PLEASE NOTE!\nThe command string, which is displayed in the next dialog, is incomplete.\nYou should replace the letters A-D with atom indices that define the torsion.");
+		
+		static const char command[] = "make_plot1 A B C D 36 0.0 360.0 250";
+		new command_dialog(docv, gv, command);
+	}
 }
 
 void gnome_mm1_docv::popup_CompEPlot2D(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
-	if (docv) docv->DoEnergyPlot2D();
+	if (docv)
+	{
+		docv->err->Message("PLEASE NOTE!\nThe command string, which is displayed in the next dialog, is incomplete.\nYou should replace the letters A-D and I-L with atom indices that define the torsions.");
+		
+		static const char command[] = "make_plot2 A B C D 36 0.0 360.0 I J K L 36 0.0 360.0 250";
+		new command_dialog(docv, gv, command);
+	}
 }
 
 void gnome_mm1_docv::popup_CompFormula(GtkWidget *, gpointer data)
@@ -937,7 +982,7 @@ void gnome_mm1_docv::popup_TrajView(GtkWidget *, gpointer data)
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (!docv->GetTrajectoryFile())
 	{
-		if (docv) new mm1_trajfile_dialog(docv);	// memory leak???
+		if (docv) new mm1_trajfile_dialog(docv);
 	}
 	else docv->err->ErrorMessage("Trajectory already open?!?!?!");
 }
@@ -1007,40 +1052,29 @@ cout << "preparing to open a library file : " << buffer << endl;	// just for deb
   ifile.close();
 }
 
-void gnome_mm1_docv::popup_Builder(GtkWidget *, gpointer data)
+void gnome_mm1_docv::popup_BuilderAmino(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
 	if (docv)
 	{
-		char seq[1024]; char input;
+docv->err->Message("PLEASE NOTE!\nThe command string, which is displayed in the next dialog, is incomplete.\nYou should replace the default sequence AAA with the sequence to be built.");
 		
-		f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
-		f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
+		static const char command[] = "build_amino AAA";
+		new command_dialog(docv, gv, command);
+	}
+}
+
+void gnome_mm1_docv::popup_BuilderNucleic(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
+	if (docv)
+	{
+docv->err->Message("PLEASE NOTE!\nThe command string, which is displayed in the next dialog, is incomplete.\nYou should replace the default sequence GGG with the sequence to be built.");
 		
-		f64 nab[] =
-		{
-			44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
-			273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
-			88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
-		};
-		
-		cout << "build amino/nucleic acid sequences ? (a/n) "; cin >> input;
-		
-		if (input == 'a')
-		{
-			cout << "give the sequence using symbols ARNDCQEGHILKMFPSTWYV ???"; cin >> seq;
-			cout << "please select the conformation : H = helix, S = sheet ???"; cin >> input;
-			mm1_mdl::amino_builder.Build(docv, seq, input == 'h' || input == 'H' ? aab_ah : aab_bs);
-			docv->UpdateAllGraphicsViews();
-		}
-		
-		if (input == 'n')
-		{
-			cout << "give the sequence using symbols AGTC and aguc ???"; cin >> seq;
-			mm1_mdl::nucleic_builder.Build(docv, seq, nab);
-			docv->UpdateAllGraphicsViews();
-		}
+		static const char command[] = "build_nucleic GGG";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -1071,6 +1105,16 @@ void gnome_mm1_docv::popup_Center(GtkWidget *, gpointer data)
 	}
 }
 
+void gnome_mm1_docv::popup_EnterCommand(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm1_docv * docv = dynamic_cast<gnome_mm1_docv *>(gv->docv);
+	if (docv)
+	{
+		new command_dialog(docv, gv, NULL);
+	}
+}
+
 void gnome_mm1_docv::popup_ConvertQM(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
@@ -1096,7 +1140,7 @@ qm_docv->camera_vector.front()->CopySettings(gv->cam);
 qm_docv->RemoveLight(qm_docv->light_vector.front());
 qm_docv->AddLocalLight(new directional_light(ol_static(qm_docv->camera_vector.front()->GetLocData())), qm_docv->camera_vector.front());
 
-		// convert mm1gp -> qm1gp using all_atoms_interface...
+		// convert mm1mdl -> qm1mdl using all_atoms_interface...
 		
 		qm_docv->aai_MakeCopy(docv);
 		
@@ -1118,14 +1162,13 @@ void gnome_mm1_docv::GeomOptGetParam(mm1_geomopt_param & param)
 	
 	goparam = & param;
 	
-	GtkWidget * dialog = create_a1_dialog_geomopt();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new geomopt_dialog();	// the object will call delete itself...
+	//GtkWidget * dialog = create_a1_dialog_geomopt();	// obsolete...
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete...
 	
 	goparam = NULL;
 }
 
-#define START 10000	// see mm1docv.cpp!!!!!!!!
-
 void gnome_mm1_docv::MolDynGetParam(mm1_moldyn_param & param)
 {
 	glade_modal_dialog_owner = this;	// requires modal dialogs!!!
@@ -1135,8 +1178,9 @@ void gnome_mm1_docv::MolDynGetParam(mm1_moldyn_param & param)
 	
 	mdparam = & param;
 	
-	GtkWidget * dialog = create_a2_dialog_moldyn();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new moldyn_dialog();	// the object will call delete itself...
+	//GtkWidget * dialog = create_a2_dialog_moldyn();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
 	
 	mdparam = NULL;
 }
@@ -1166,7 +1210,7 @@ void gnome_mm1_docv::MakeLibraryWidgets(GtkMenuItem *menu,
 	       
 	       if ( namelist->d_name[0] != '.' &&
 		    stat(filename, &stat_buf) == 0 &&
-		    strstr(namelist->d_name, ".mm1gp") != NULL)
+		    strstr(namelist->d_name, ".gpr") != NULL)
 		 {
 		   library_item = gtk_menu_item_new_with_label(namelist->d_name);
 		   
