@@ -24,7 +24,7 @@ class gnome_mm1_docv;
 
 /*################################################################################################*/
 
-#define MM1_FILTER	"*.mm1gp"
+#define MM1_FILTER	"*.gpr"
 
 /*################################################################################################*/
 
@@ -83,6 +83,24 @@ class mm1_file_save_graphics_dialog : public file_dialog
 
 /*################################################################################################*/
 
+class mm1_importpdb_dialog : public file_dialog
+{
+	private:
+	
+	static const char title[];
+	static gnome_mm1_docv * docv;
+	
+	public:
+	
+	mm1_importpdb_dialog(gnome_mm1_docv *);
+	~mm1_importpdb_dialog(void);
+	
+	bool OkEvent(const char *);	// virtual
+	void CancelEvent(void);		// virtual
+};
+
+/*################################################################################################*/
+
 class mm1_trajfile_dialog : public file_dialog
 {
 	private:
@@ -107,6 +125,7 @@ class gnome_mm1_docv : public gnome_docv, public mm1_docv
 {
 	protected:
 	
+	static GnomeUIInfo filext_submenu[];		// file
 	static GnomeUIInfo file_submenu[];
 	
 	static GnomeUIInfo selmodes_radio[];
@@ -119,8 +138,8 @@ class gnome_mm1_docv : public gnome_docv, public mm1_docv
 	static GnomeUIInfo views_submenu[];		// render
 	static GnomeUIInfo lights_submenu[];		// render
 	static GnomeUIInfo rendermode_style_radio[];
-	static GnomeUIInfo rendermode_label_radio[];
 	static GnomeUIInfo rendermode_color_radio[];
+	static GnomeUIInfo rendermode_label_radio[];
 	static GnomeUIInfo rendermode_submenu[];	// render
 	static GnomeUIInfo projection_radio[];
 	static GnomeUIInfo projection_submenu[];	// render
@@ -161,16 +180,15 @@ class gnome_mm1_docv : public gnome_docv, public mm1_docv
 	static void popup_FileExport(GtkWidget *, gpointer);
 	static void popup_FileExportGraphic(GtkWidget *, gpointer);
 	static void popup_FileSaveAs(GtkWidget *, gpointer);
+	static void popup_FileExtra1(GtkWidget *, gpointer);
 	
 	static void popup_RModeWireFrame(GtkWidget *, gpointer);
 	static void popup_RModeBallAndStick(GtkWidget *, gpointer);
 	static void popup_RModeVanDerWaals(GtkWidget *, gpointer);
 	static void popup_RModeCylinders(GtkWidget *, gpointer);
 	static void popup_RModeNothing(GtkWidget *, gpointer);
-
 	static void popup_CModeState(GtkWidget *, gpointer);
 	static void popup_CModeElement(GtkWidget *, gpointer);
-	
 	static void popup_LModeNothing(GtkWidget *, gpointer);
 	static void popup_LModeIndex(GtkWidget *, gpointer);
 	static void popup_LModeCharge(GtkWidget *, gpointer);
@@ -189,6 +207,8 @@ class gnome_mm1_docv : public gnome_docv, public mm1_docv
 	static void popup_CompGeomOpt(GtkWidget *, gpointer);
 	static void popup_CompMolDyn(GtkWidget *, gpointer);
 	static void popup_CompRandomSearch(GtkWidget *, gpointer);
+	static void popup_CompSystematicSearch(GtkWidget *, gpointer);
+	static void popup_CompMonteCarloSearch(GtkWidget *, gpointer);
 	static void popup_CompEPlot1D(GtkWidget *, gpointer);
 	static void popup_CompEPlot2D(GtkWidget *, gpointer);
 	static void popup_CompFormula(GtkWidget *, gpointer);
@@ -198,8 +218,10 @@ class gnome_mm1_docv : public gnome_docv, public mm1_docv
 	static void popup_HRemove(GtkWidget *, gpointer);
 	static void popup_Clear(GtkWidget *, gpointer);
 	static void popup_Library(GtkWidget *, gpointer);
-	static void popup_Builder(GtkWidget *, gpointer);
+	static void popup_BuilderAmino(GtkWidget *, gpointer);
+	static void popup_BuilderNucleic(GtkWidget *, gpointer);
 	static void popup_Center(GtkWidget *, gpointer);
+	static void popup_EnterCommand(GtkWidget *, gpointer);
 	static void popup_ConvertQM(GtkWidget *, gpointer);
 	
 	void GeomOptGetParam(mm1_geomopt_param &);	// virtual
