@@ -9,7 +9,9 @@
 
 #include "t3mm2docv.h"
 
-#include "glade_interface.h"
+#include "geomopt_dialog.h"
+#include "moldyn_dialog.h"
+#include "trajview_dialog.h"
 
 #include "t3main.h"
 #include "t3dialog.h"
@@ -54,7 +56,7 @@ bool mm2_file_open_dialog::OkEvent(const char * filename)
 	
 	ifstream ifile;
 	ifile.open(filename, ios::in);
-	docv->ReadStream(ifile);
+	docv->ReadStream(ifile, false);
 	ifile.close();
 	
 	docv->ParseProjectFileNameAndPath(filename);
@@ -135,10 +137,11 @@ bool mm2_trajfile_dialog::OkEvent(const char * filename)
 	
 	gnome_docv::glade_modal_dialog_owner = (gnome_mm2_docv *) docv;		// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_t1_dialog_trajectory();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new trajview_dialog();	// will call delete and CloseTrajectory!
+	//GtkWidget * dialog = create_t1_dialog_trajectory();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
+	//docv->CloseTrajectory();			// is handled by dialog...
 	
-	docv->CloseTrajectory();
 	return false;
 }
 
@@ -148,10 +151,49 @@ void mm2_trajfile_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
-GnomeUIInfo gnome_mm2_docv::fileimport_submenu[] =
+const char mm2_importpdb_dialog::title[] = "Import File - Brookhaven PDB/ENT";
+gnome_mm2_docv * mm2_importpdb_dialog::docv = NULL;
+
+mm2_importpdb_dialog::mm2_importpdb_dialog(gnome_mm2_docv * p1) : file_dialog(title, NULL, NULL)
 {
-	GNOMEUIINFO_ITEM("pdb...", "pdb",
-		popup_FileImport, NULL),
+	docv = p1;
+}
+
+mm2_importpdb_dialog::~mm2_importpdb_dialog(void)
+{
+}
+
+bool mm2_importpdb_dialog::OkEvent(const char * filename)
+{
+	if (docv != NULL)
+	{
+		docv->err->WarningMessage("This alternative PDB reader is designed to remove any gaps that might exist in the structure data.\nTherefore the sequence might be altered! The process can also take several minutes to complete.");
+		
+		docv->ParseProjectFileNameAndPath(filename);
+		
+//cout << "give name of the PDB file to import ??? "; cin >> docv->buffer;
+//cout << "give the chain index to be imported, or -1 to import all chains ??? "; i32s ci; cin >> ci;
+//docv->ImportPDB(docv->buffer, ci);
+
+		docv->ImportPDB(filename, -1);
+		
+		docv->UpdateAllWindowTitles();
+		docv->UpdateAllGraphicsViews();
+	}
+	
+	return true;
+}
+
+void mm2_importpdb_dialog::CancelEvent(void)
+{
+}
+
+/*################################################################################################*/
+
+GnomeUIInfo gnome_mm2_docv::filext_submenu[] =
+{
+	GNOMEUIINFO_ITEM("Import PDB...", "a special tool for importing PDB files.",
+		popup_FileExtra1, NULL),
 	
 	GNOMEUIINFO_END
 };
@@ -159,12 +201,15 @@ GnomeUIInfo gnome_mm2_docv::fileimport_submenu[] =
 GnomeUIInfo gnome_mm2_docv::file_submenu[] =
 {
 	GNOMEUIINFO_MENU_OPEN_ITEM(popup_FileOpen, NULL),
-	GNOMEUIINFO_SUBTREE("_Import", fileimport_submenu),
 	
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_MENU_SAVE_AS_ITEM(popup_FileSaveAs, NULL),
 	
+	GNOMEUIINFO_SEPARATOR,
+
+	GNOMEUIINFO_SUBTREE("Extra Tools", filext_submenu),
+
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_MENU_CLOSE_ITEM(gnome_graphics_view::popup_FileClose, NULL),
@@ -216,10 +261,10 @@ GnomeUIInfo gnome_mm2_docv::lights_submenu[] =
 		
 	GNOMEUIINFO_SEPARATOR,
 	
-	GNOMEUIINFO_TOGGLEITEM("Display Local Lights", "loc",
+	GNOMEUIINFO_TOGGLEITEM("Enable Local Lights", "loc",
 		gnome_graphics_view::popup_LightsSwitchLoc, NULL),
 		
-	GNOMEUIINFO_TOGGLEITEM("Display Global Lights", "glob",
+	GNOMEUIINFO_TOGGLEITEM("Enable Global Lights", "glob",
 		gnome_graphics_view::popup_LightsSwitchGlob, NULL),
 		
 	GNOMEUIINFO_END
@@ -248,9 +293,41 @@ GnomeUIInfo gnome_mm2_docv::rendermode_style_radio[] =
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo gnome_mm2_docv::rendermode_color_radio[] =
+{
+	GNOMEUIINFO_ITEM("Color State", "color-st",
+		popup_CModeState, NULL),
+		
+	GNOMEUIINFO_ITEM("Color Residue", "color-res",
+		popup_CModeResidue, NULL),
+		
+	GNOMEUIINFO_ITEM("Color SASA", "color-sasa",
+		popup_CModeSASA, NULL),
+		
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_mm2_docv::rendermode_label_radio[] =
+{
+	GNOMEUIINFO_ITEM("Label Nothing", "lab-null",
+		popup_LModeNothing, NULL),
+		
+	GNOMEUIINFO_ITEM("Label Residue", "lab-res",
+		popup_LModeResidue, NULL),
+
+//	GNOMEUIINFO_ITEM("Label DEBUG", "lab-???",
+//		popup_LModeDEBUG, NULL),
+		
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo gnome_mm2_docv::rendermode_submenu[] =
 {
         GNOMEUIINFO_RADIOLIST(rendermode_style_radio),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_RADIOLIST(rendermode_color_radio),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_RADIOLIST(rendermode_label_radio),
 	GNOMEUIINFO_END
 };
 
@@ -345,6 +422,11 @@ GnomeUIInfo gnome_mm2_docv::popupmenu_info[] =
 	
 	GNOMEUIINFO_ITEM("MD Trajectory Viewer...", "...",
 		popup_TrajView, NULL),
+
+	GNOMEUIINFO_SEPARATOR,
+
+	GNOMEUIINFO_ITEM("Enter a Command...", "Type in a command string and process it.",
+		popup_EnterCommand, NULL),
 		
 	GNOMEUIINFO_END
 };
@@ -374,26 +456,24 @@ void gnome_mm2_docv::popup_FileOpen(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
-	docv->err->Message("Sorry, this feature is not yet implemented.");
-}
-
-void gnome_mm2_docv::popup_FileImport(GtkWidget *, gpointer data)
-{
-	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
-	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
-	if (docv)
-	{
-cout << "give name of the PDB file to import ??? "; cin >> docv->buffer;
-cout << "give the chain index to be imported, or -1 to import all chains ??? "; i32s ci; cin >> ci;
-docv->ImportPDB(docv->buffer, ci); docv->UpdateAllGraphicsViews();
-	}
+	docv->err->Message("Sorry, this feature is not yet implemented.\nPlease use File/New + File/ExtraTools/ImportPDB instead.");
 }
 
 void gnome_mm2_docv::popup_FileSaveAs(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
-	if (docv) new mm2_file_save_dialog(docv);	// memory leak???
+	if (docv) new mm2_file_save_dialog(docv);
+}
+
+void gnome_mm2_docv::popup_FileExtra1(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
+	if (docv)
+	{
+		new mm2_importpdb_dialog(docv);
+	}
 }
 
 void gnome_mm2_docv::popup_RModeWireFrame(GtkWidget *, gpointer data)
@@ -432,6 +512,42 @@ void gnome_mm2_docv::popup_RModeNothing(GtkWidget *, gpointer data)
 	gv->render = RENDER_NOTHING; gv->docv->UpdateAllGraphicsViews();
 }
 
+void gnome_mm2_docv::popup_LModeNothing(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->label = LABEL_NOTHING; gv->docv->UpdateAllGraphicsViews();
+}
+
+void gnome_mm2_docv::popup_LModeResidue(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->label = LABEL_RESIDUE; gv->docv->UpdateAllGraphicsViews();
+}
+
+void gnome_mm2_docv::popup_LModeDEBUG(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->label = LABEL_CHARGE; gv->docv->UpdateAllGraphicsViews();			// DEBUG!!!
+}
+
+void gnome_mm2_docv::popup_CModeState(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->colormode = & mm2_docv::cm_state; gv->docv->UpdateAllGraphicsViews();
+}
+
+void gnome_mm2_docv::popup_CModeResidue(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->colormode = & mm2_docv::cm_residue; gv->docv->UpdateAllGraphicsViews();
+}
+
+void gnome_mm2_docv::popup_CModeSASA(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->colormode = & mm2_docv::cm_sasa; gv->docv->UpdateAllGraphicsViews();
+}
+
 void gnome_mm2_docv::popup_ObjRibbon(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
@@ -463,18 +579,8 @@ void gnome_mm2_docv::popup_ObjEPlane(GtkWidget *, gpointer data)
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("MM2Graphics/PlaneResolution", 30); 
-		cpp.docv = docv; cpp.ref = (mm2_docv *) docv;
-		
-		cout << "give dimension of the plane [nm] (2.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) mm2_GetESPValue; cpp.cf = (ColorFunction *) GetRBRange1;
-		cpp.value = 138.9354518 / 10.0; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane esp rb1 14.0 2.5 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -484,18 +590,8 @@ void gnome_mm2_docv::popup_ObjEVolume(GtkWidget *, gpointer data)
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("MM2Graphics/SurfaceResolution", 30);
-		cpp.docv = docv; cpp.ref = (mm2_docv *) docv;
-		
-		cout << "give dimension of the volume [nm] (2.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) mm2_GetESPValue; cpp.cf = (ColorFunction *) GetRBRange2;
-		cpp.value = 138.9354518 / 10.0; cpp.alpha = 0.25;
-		
-		docv->AddObject(new volume_rendering_object(ol_static(), cpp, 20, tmp1 / 2.0, (* gv->cam), "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add volrend esp rb2 14.0 2.5 25 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -505,37 +601,8 @@ void gnome_mm2_docv::popup_ObjESurface(GtkWidget *, gpointer data)
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
-		
-		i32s size = docv->model_prefs->Value("MM2Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp2a; cs_param csp2b;
-		csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-		csp2a.docv = docv; csp2a.ref = (mm2_docv *) docv; csp2a.next = & csp2b;
-		csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-		csp2b.docv = docv; csp2b.ref = (mm2_docv *) docv; csp2b.next = NULL;
-		
-		tmp1 = 138.9354518 / 10.0;
-		cout << "give dimension of the surface [nm] (5.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp2a.wireframe = csp2b.wireframe = (input == 'w' || input == 'W');
-		
-		csp2a.transparent = !true;
-		csp2a.vf1 = (ValueFunction *) mm2_GetESPValue; csp2a.vf2 = (ValueFunction *) GetUnity;
-		csp2a.cvalue = 1.0; csp2a.alpha = 0.25; csp2a.toler = 1.0e-6 * tmp1; csp2a.maxc = 500;
-		csp2a.cf = (ColorFunction *) GetRedColor; csp2a.svalue = +tmp1;
-		
-		csp2b.transparent = !true;
-		csp2b.vf1 = (ValueFunction *) mm2_GetESPValue; csp2b.vf2 = (ValueFunction *) GetUnity;
-		csp2b.cvalue = 1.0; csp2b.alpha = 0.25; csp2b.toler = 1.0e-6 * tmp1; csp2b.maxc = 500;
-		csp2b.cf = (ColorFunction *) GetBlueColor; csp2b.svalue = -tmp1;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp2a, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf2 esp unity red blue +14.0 -14.0 1.0 5.0 50 0 0 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -545,29 +612,8 @@ void gnome_mm2_docv::popup_ObjVDWSurface(GtkWidget *, gpointer data)
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
-		
-		i32s size = docv->model_prefs->Value("MM2Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp1;
-		csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-		csp1.docv = docv; csp1.ref = (mm2_docv *) docv; csp1.next = NULL;
-		
-		tmp1 = 138.9354518 / 10.0;
-		cout << "give dimension of the surface [nm] (5.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp1.wireframe = (input == 'w' || input == 'W');
-		
-		csp1.transparent = true; csp1.cf = (ColorFunction *) GetRBRange1;
-		csp1.vf1 = (ValueFunction *) mm2_GetSASValue; csp1.vf2 = (ValueFunction *) mm2_GetESPValue;
-		csp1.svalue = 1.0; csp1.cvalue = tmp1; csp1.alpha = 0.50; csp1.toler = 1.0e-6 * tmp1; csp1.maxc = 500;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp1, "ESP-colored VDW-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf1 vdws esp rb1 1.0 14.0 5.0 50 1 1 0.65";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -614,11 +660,21 @@ void gnome_mm2_docv::popup_TrajView(GtkWidget *, gpointer data)
 	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
 	if (!docv->GetTrajectoryFile())
 	{
-		if (docv) new mm2_trajfile_dialog(docv);	// memory leak???
+		if (docv) new mm2_trajfile_dialog(docv);
 	}
 	else docv->err->ErrorMessage("Trajectory already open?!?!?!");
 }
 
+void gnome_mm2_docv::popup_EnterCommand(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
+	if (docv)
+	{
+		new command_dialog(docv, gv, NULL);
+	}
+}
+
 /*##############################################*/
 /*##############################################*/
 
@@ -631,8 +687,9 @@ void gnome_mm2_docv::GeomOptGetParam(mm2_geomopt_param & param)
 	
 	goparam = & param;
 	
-	GtkWidget * dialog = create_a1_dialog_geomopt();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new geomopt_dialog();	// the object will call delete itself...
+	//GtkWidget * dialog = create_a1_dialog_geomopt();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
 	
 	goparam = NULL;
 }
@@ -648,8 +705,9 @@ void gnome_mm2_docv::MolDynGetParam(mm2_moldyn_param & param)
 	
 	mdparam = & param;
 	
-	GtkWidget * dialog = create_a2_dialog_moldyn();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new moldyn_dialog();	// the object will call delete itself...
+	//GtkWidget * dialog = create_a2_dialog_moldyn();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
 	
 	mdparam = NULL;
 }
