@@ -83,11 +83,29 @@ class mm2_trajfile_dialog : public file_dialog
 
 /*################################################################################################*/
 
+class mm2_importpdb_dialog : public file_dialog
+{
+	private:
+	
+	static const char title[];
+	static gnome_mm2_docv * docv;
+	
+	public:
+	
+	mm2_importpdb_dialog(gnome_mm2_docv *);
+	~mm2_importpdb_dialog(void);
+	
+	bool OkEvent(const char *);	// virtual
+	void CancelEvent(void);		// virtual
+};
+
+/*################################################################################################*/
+
 class gnome_mm2_docv : public gnome_docv, public mm2_docv
 {
 	protected:
 	
-	static GnomeUIInfo fileimport_submenu[];	// file
+	static GnomeUIInfo filext_submenu[];		// file
 	static GnomeUIInfo file_submenu[];
 	
 	static GnomeUIInfo select_submenu[];
@@ -96,6 +114,8 @@ class gnome_mm2_docv : public gnome_docv, public mm2_docv
 	static GnomeUIInfo views_submenu[];		// render
 	static GnomeUIInfo lights_submenu[];		// render
 	static GnomeUIInfo rendermode_style_radio[];
+	static GnomeUIInfo rendermode_color_radio[];
+	static GnomeUIInfo rendermode_label_radio[];
 	static GnomeUIInfo rendermode_submenu[];	// render
 	static GnomeUIInfo projection_radio[];
 	static GnomeUIInfo projection_submenu[];	// render
@@ -123,8 +143,8 @@ class gnome_mm2_docv : public gnome_docv, public mm2_docv
 	// the mm2-dependent popup-menu callbacks are here...
 	
 	static void popup_FileOpen(GtkWidget *, gpointer);
-	static void popup_FileImport(GtkWidget *, gpointer);
 	static void popup_FileSaveAs(GtkWidget *, gpointer);
+	static void popup_FileExtra1(GtkWidget *, gpointer);
 	
 	static void popup_RModeWireFrame(GtkWidget *, gpointer);
 	static void popup_RModeBallAndStick(GtkWidget *, gpointer);
@@ -132,6 +152,12 @@ class gnome_mm2_docv : public gnome_docv, public mm2_docv
 	static void popup_RModeAccessible(GtkWidget *, gpointer);
 	static void popup_RModeCylinders(GtkWidget *, gpointer);
 	static void popup_RModeNothing(GtkWidget *, gpointer);
+	static void popup_CModeState(GtkWidget *, gpointer);
+	static void popup_CModeResidue(GtkWidget *, gpointer);
+	static void popup_CModeSASA(GtkWidget *, gpointer);
+	static void popup_LModeNothing(GtkWidget *, gpointer);
+	static void popup_LModeResidue(GtkWidget *, gpointer);
+	static void popup_LModeDEBUG(GtkWidget *, gpointer);
 	
 	static void popup_ObjRibbon(GtkWidget *, gpointer);
 	static void popup_ObjEPlane(GtkWidget *, gpointer);
@@ -143,6 +169,7 @@ class gnome_mm2_docv : public gnome_docv, public mm2_docv
 	static void popup_CompGeomOpt(GtkWidget *, gpointer);
 	static void popup_CompMolDyn(GtkWidget *, gpointer);
 	static void popup_TrajView(GtkWidget *, gpointer);
+	static void popup_EnterCommand(GtkWidget *, gpointer);
 	
 	void GeomOptGetParam(mm2_geomopt_param &);	// virtual
 	void MolDynGetParam(mm2_moldyn_param &);	// virtual
