@@ -88,8 +88,17 @@ void gnome_prefs::Add(const char *key,
 char * gnome_prefs::Find(const char *key, const char * def)
 {
   ostrstream fullkey(keybuffer, sizeof(keybuffer));
-  fullkey << "/ghemical/" << key << "=" << def << ends;  
-  return gnome_config_get_string(keybuffer);
+  fullkey << "/ghemical/" << key << "=" << def << ends;
+  
+  // for consistency, so that gnome_prefs::Find() and console_prefs::Find() would work
+  // the same way, we must allocate the final memory block that we return using new.
+  
+  char * tmpbuff = gnome_config_get_string(keybuffer);
+  char * retbuff = new char[strlen(tmpbuff) + 1];
+  strcpy(retbuff, tmpbuff);
+  g_free(tmpbuff);
+  
+  return retbuff;
 }
 
 int    gnome_prefs::Value(const char *key, const int def)
@@ -116,36 +125,37 @@ bool   gnome_prefs::Boolean(const char *key, const bool def)
 // noticed at 0.80 that graphics have slowed down compared to 0.42 - at least partially
 // this seems to be related to colour prefs. calling prefs->ColorRGB() and prefs->ColorRGBA()
 // is a costly operation, and we need to store the colours to arrays (at element class???)
-// and move the function calls outside loops, as much as possible.
-
-// a temporary patch is applied here where the default values are just quickly returned.
-// 2001-10-03 TH
+// and move the function calls outside loops, as much as possible. NOT YET DONE!
 
 float *gnome_prefs::ColorRGB(const char *key, const float def[3])
 {
   float *temp = new float[3];
-  char *valbuffer = new char[1024];
+  
+  char valbuffer[1024];
   ostrstream fullkey(keybuffer, sizeof(keybuffer));
   sprintf(valbuffer, "%f %f %f", def[0], def[1], def[2]);
   fullkey << "/ghemical/" << key << "=" << valbuffer << ends;
-  delete [] valbuffer;
-
-  valbuffer = gnome_config_get_string(keybuffer);
-  sscanf(valbuffer, "%f %f %f", &temp[0], &temp[1], &temp[2]);
+  
+  char * tmpbuff = gnome_config_get_string(keybuffer);
+  sscanf(tmpbuff, "%f %f %f", &temp[0], &temp[1], &temp[2]);
+  g_free(tmpbuff);
+  
   return temp;
 }
 	
 float *gnome_prefs::ColorRGBA(const char *key, const float def[4])
 {
   float *temp = new float[4];
-  char *valbuffer = new char[1024];
+  
+  char valbuffer[1024];
   ostrstream fullkey(keybuffer, sizeof(keybuffer));
   sprintf(valbuffer, "%f %f %f %f", def[0], def[1], def[2], def[3]);
   fullkey << "/ghemical/" << key << "=" << valbuffer << ends;
-  delete [] valbuffer;
-
-  valbuffer = gnome_config_get_string(keybuffer);
-  sscanf(valbuffer, "%f %f %f %f", &temp[0], &temp[1], &temp[2], &temp[3]);
+  
+  char * tmpbuff = gnome_config_get_string(keybuffer);
+  sscanf(tmpbuff, "%f %f %f %f", &temp[0], &temp[1], &temp[2], &temp[3]);
+  g_free(tmpbuff);
+  
   return temp;
 }
 
