@@ -9,7 +9,8 @@
 
 #include "t3qm1docv.h"
 
-#include "glade_interface.h"
+#include "setup_qm1_dialog.h"
+#include "geomopt_dialog.h"
 
 #include "t3main.h"
 #include "t3dialog.h"
@@ -30,7 +31,7 @@ using namespace std;
 
 /*################################################################################################*/
 
-const char qm1_file_open_dialog::title[] = "Open Project File - qm1gp";
+const char qm1_file_open_dialog::title[] = "Open Project File";
 gnome_qm1_docv * qm1_file_open_dialog::docv = NULL;
 
 qm1_file_open_dialog::qm1_file_open_dialog(gnome_qm1_docv * p1) : file_dialog(title, NULL, NULL)
@@ -50,13 +51,30 @@ bool qm1_file_open_dialog::OkEvent(const char * filename)
 		docv->InitGnomeMDIChild();
 	}
 	
-//	ifstream ifile;				THIS IS NOT YET READY...
-//	ifile.open(filename, ios::in);
-//	docv->ReadStream(ifile);
-//	ifile.close();
-
-	docv->UpdateAllGraphicsViews();
+	bool open = !docv->atom_list.size();	// open, not insert
 	
+	ifstream ifile;
+	ifile.open(filename, ios::in);
+	docv->ReadStream(ifile, !open);		// was !open
+	ifile.close();
+	
+	if (open) 	// we're "switching" to this file
+	{
+		docv->ParseProjectFileNameAndPath(filename);
+		
+		gnome_mdi_child_set_name(GNOME_MDI_CHILD(docv->child), docv->GetProjectFileName(true));
+		docv->UpdateAllWindowTitles();
+	}
+	else		// we append this file to the current project
+	{
+		if (docv->selected_object != NULL)		// if an object was selected,
+		{						// remove that selection and leave
+			docv->selected_object = NULL;		// the newly added atoms as selection;
+			docv->event_SelectedObjectChanged();	// now the mouse tools also affect the
+		}						// newly added atoms!
+	}
+	
+	docv->UpdateAllGraphicsViews();
 	return true;
 }
 
@@ -66,43 +84,51 @@ void qm1_file_open_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
-const char qm1_importmm1_dialog::title[] = "Import File - mm1gp";
-gnome_qm1_docv * qm1_importmm1_dialog::docv = NULL;
+const char qm1_file_save_dialog::title[] = "Save Project File";
+gnome_qm1_docv * qm1_file_save_dialog::docv = NULL;
 
-qm1_importmm1_dialog::qm1_importmm1_dialog(gnome_qm1_docv * p1) : file_dialog(title, NULL, NULL)
+qm1_file_save_dialog::qm1_file_save_dialog(gnome_qm1_docv * p1) : file_dialog(title, NULL, NULL)
 {
 	docv = p1;
 }
 
-qm1_importmm1_dialog::~qm1_importmm1_dialog(void)
+qm1_file_save_dialog::~qm1_file_save_dialog(void)
 {
 }
 
-bool qm1_importmm1_dialog::OkEvent(const char * filename)
+bool qm1_file_save_dialog::OkEvent(const char * filename)
 {
-	if (docv != NULL)
+	if (docv == NULL)
 	{
-	  // Update titlebar
-		docv->ParseProjectFileNameAndPath(filename);
-		docv->ImportMM1(filename);
-
-		docv->UpdateAllWindowTitles();
-		docv->UpdateAllGraphicsViews();
+	  docv->err->ErrorMessage("qm1_file_save_dialog::OkEvent() failed");
+	  exit(EXIT_FAILURE);
 	}
 	
+  	// should check to see if file already exists...
+
+	docv->ParseProjectFileNameAndPath(filename);
+	
+	gnome_mdi_child_set_name(GNOME_MDI_CHILD(docv->child), docv->GetProjectFileName(true));
+	docv->UpdateAllWindowTitles();
+	
+	ofstream ofile;
+	ofile.open(docv->GetFullProjectFileName(), ios::out);
+	docv->WriteStream(ofile);
+	ofile.close();
+
 	return true;
 }
 
-void qm1_importmm1_dialog::CancelEvent(void)
+void qm1_file_save_dialog::CancelEvent(void)
 {
 }
 
 /*################################################################################################*/
 
-GnomeUIInfo gnome_qm1_docv::fileimport_submenu[] =
+GnomeUIInfo gnome_qm1_docv::filext_submenu[] =
 {
-	GNOMEUIINFO_ITEM("mm1gp...", "mm1gp",
-		popup_FileImport, NULL),
+//	GNOMEUIINFO_ITEM("no operation!", "???",
+//		popup_FileExtra1, NULL),
 		
 	GNOMEUIINFO_END
 };
@@ -110,12 +136,15 @@ GnomeUIInfo gnome_qm1_docv::fileimport_submenu[] =
 GnomeUIInfo gnome_qm1_docv::file_submenu[] =
 {
 	GNOMEUIINFO_MENU_OPEN_ITEM(popup_FileOpen, NULL),
-	GNOMEUIINFO_SUBTREE("_Import", fileimport_submenu),
 	
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_MENU_SAVE_AS_ITEM(popup_FileSaveAs, NULL),
 	
+	GNOMEUIINFO_SEPARATOR,
+
+	GNOMEUIINFO_SUBTREE("Extra Tools", filext_submenu),
+
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_MENU_CLOSE_ITEM(gnome_graphics_view::popup_FileClose, NULL),
@@ -192,7 +221,7 @@ GnomeUIInfo gnome_qm1_docv::lights_submenu[] =
 	GNOMEUIINFO_END
 };
 
-GnomeUIInfo gnome_qm1_docv::rendermode_submenu[] =
+GnomeUIInfo gnome_qm1_docv::rendermode_style_radio[] =
 {
 	GNOMEUIINFO_ITEM("Wireframe", "wf",
 		popup_RModeWireFrame, NULL),
@@ -206,6 +235,35 @@ GnomeUIInfo gnome_qm1_docv::rendermode_submenu[] =
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo gnome_qm1_docv::rendermode_color_radio[] =
+{
+	GNOMEUIINFO_ITEM("Color Element", "color-el",
+		popup_CModeElement, NULL),
+		
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_qm1_docv::rendermode_label_radio[] =
+{
+	GNOMEUIINFO_ITEM("Label Nothing", "lab-null",
+		popup_LModeNothing, NULL),
+		
+//	GNOMEUIINFO_ITEM("Label DEBUG", "lab-???",
+//		popup_LModeDEBUG, NULL),
+	
+	GNOMEUIINFO_END
+};
+
+GnomeUIInfo gnome_qm1_docv::rendermode_submenu[] =
+{
+        GNOMEUIINFO_RADIOLIST(rendermode_style_radio),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_RADIOLIST(rendermode_color_radio),
+	GNOMEUIINFO_SEPARATOR,
+        GNOMEUIINFO_RADIOLIST(rendermode_label_radio),
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo gnome_qm1_docv::projection_submenu[] =
 {
 	GNOMEUIINFO_ITEM("Orthographic", "ortho",
@@ -321,8 +379,11 @@ GnomeUIInfo gnome_qm1_docv::popupmenu_info[] =
 		popup_SetOrbital, NULL),
 	
 	GNOMEUIINFO_SEPARATOR,
+
+	GNOMEUIINFO_ITEM("Enter a Command...", "Type in a command string and process it.",
+		popup_EnterCommand, NULL),
 	
-	GNOMEUIINFO_ITEM("Convert to MM project", "Convert this project into a mm1gp project.",
+	GNOMEUIINFO_ITEM("Convert to MM project", "Convert this project into a Molecular Mechanics project.",
 		popup_ConvertMM, NULL),
 	
 	GNOMEUIINFO_END
@@ -353,28 +414,25 @@ void gnome_qm1_docv::popup_FileOpen(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
-	docv->err->Message("Sorry, this feature is not yet implemented.");
-}
-
-void gnome_qm1_docv::popup_FileImport(GtkWidget *, gpointer data)
-{
-	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
-	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
-	if (docv)
-	{
-		new qm1_importmm1_dialog(docv);
-//cout << "name of MM1 file to import ??? ";
-//cin >> docv->buffer; docv->ImportMM1(docv->buffer);
-//docv->UpdateAllGraphicsViews();
-	}
+	
+	if (docv) new qm1_file_open_dialog(docv);
 }
 
 void gnome_qm1_docv::popup_FileSaveAs(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
+	glade_modal_dialog_owner = docv;	// requires modal dialogs!!!
 	
-	docv->err->Message("Sorry, this feature is not yet implemented.");
+	if (docv) new qm1_file_save_dialog(docv);
+}
+
+void gnome_qm1_docv::popup_FileExtra1(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
+	
+	// no operation...
 }
 
 void gnome_qm1_docv::popup_ViewsNewEnLevDiag(GtkWidget *, gpointer data)
@@ -403,26 +461,35 @@ void gnome_qm1_docv::popup_RModeNothing(GtkWidget *, gpointer data)
 	gv->render = RENDER_NOTHING; gv->docv->UpdateAllGraphicsViews();
 }
 
+void gnome_qm1_docv::popup_LModeNothing(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->label = LABEL_NOTHING; gv->docv->UpdateAllGraphicsViews();
+}
+
+void gnome_qm1_docv::popup_LModeDEBUG(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gv->label = LABEL_CHARGE; gv->docv->UpdateAllGraphicsViews();			// DEBUG!!!
+}
+
+void gnome_qm1_docv::popup_CModeElement(GtkWidget *, gpointer data)
+{
+// color modes not yet implemented at all here...
+//	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+//	gv->colormode = & mm2_docv::cm_element; gv->docv->UpdateAllGraphicsViews();
+}
+
 void gnome_qm1_docv::popup_ObjEPlane(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/PlaneResolution", 30); 
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getESP; cpp.cf = (ColorFunction *) GetRBRange1;
-		cpp.value = 138.9354518 * 1.0; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane esp rb1 138.0 0.75 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -432,39 +499,10 @@ void gnome_qm1_docv::popup_ObjESurface(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-		i32s size = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp2a; cs_param csp2b;
-		csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-		csp2a.docv = docv; csp2a.ref = docv->current_eng; csp2a.next = & csp2b;
-		csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-		csp2b.docv = docv; csp2b.ref = docv->current_eng; csp2b.next = NULL;
-		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		tmp1 = 138.9354518 / 4.0;
-		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp2a.wireframe = csp2b.wireframe = (input == 'w' || input == 'W');
-		
-		csp2a.transparent = !true;
-		csp2a.vf1 = (ValueFunction *) qm1_eng::getESP; csp2a.vf2 = (ValueFunction *) GetUnity;
-		csp2a.cvalue = 1.0; csp2a.alpha = 0.50; csp2a.toler = 1.0e-6 * tmp1; csp2a.maxc = 50;	// slow!!!
-			csp2a.cf = (ColorFunction *) GetRedColor; csp2a.svalue = +tmp1;
-			
-		csp2b.transparent = !true;
-		csp2b.vf1 = (ValueFunction *) qm1_eng::getESP; csp2b.vf2 = (ValueFunction *) GetUnity;
-		csp2b.cvalue = 1.0; csp2b.alpha = 0.50; csp2b.toler = 1.0e-6 * tmp1; csp2b.maxc = 50;	// slow!!!
-			csp2b.cf = (ColorFunction *) GetBlueColor; csp2b.svalue = -tmp1;
-			
-		docv->AddObject(new color_surface_object(ol_static(), csp2a, "ESP-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf2 esp unity red blue +35.0 -35.0 1.0 1.5 50 0 0 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -474,31 +512,10 @@ void gnome_qm1_docv::popup_ObjVDWSurface(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-		i32s size = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp1;
-		csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-		csp1.docv = docv; csp1.ref = docv->current_eng; csp1.next = NULL;
-		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		tmp1 = 138.9354518 / 2.0;
-		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp1.wireframe = (input == 'w' || input == 'W');
-		
-		csp1.transparent = true; csp1.cf = (ColorFunction *) GetRBRange1;
-		csp1.vf1 = (ValueFunction *) qm1_eng::getVDWSurface; csp1.vf2 = (ValueFunction *) qm1_eng::getESP;
-		csp1.svalue = 1.0; csp1.cvalue = tmp1; csp1.alpha = 0.50; csp1.toler = 1.0e-6 * tmp1; csp1.maxc = 250;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp1, "ESP-colored VDW-"));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf1 vdws esp rb1 1.0 70.0 1.5 50 1 1 0.65";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -508,20 +525,10 @@ void gnome_qm1_docv::popup_ObjEDPlane(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/PlaneResolution", 30); 
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getElDens; cpp.cf = (ColorFunction *) GetGreenColor;
-		cpp.value = 0.05; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "E-density "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane eldens rb1 0.05 0.75 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -531,31 +538,10 @@ void gnome_qm1_docv::popup_ObjEDSurface(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-		i32s size = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp1;
-		csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-		csp1.docv = docv; csp1.ref = docv->current_eng; csp1.next = NULL;
-		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		tmp1 = 0.05;
-		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp1.wireframe = (input == 'w' || input == 'W');
-		
-		csp1.transparent = true; csp1.cf = (ColorFunction *) GetGreenColor;
-		csp1.vf1 = (ValueFunction *) qm1_eng::getElDens; csp1.vf2 = (ValueFunction *) GetUnity;
-		csp1.svalue = tmp1; csp1.cvalue = 1.0; csp1.alpha = 0.50; csp1.toler = 1.0e-6 * tmp1; csp1.maxc = 250;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp1, "E-density "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf1 eldens unity red 0.01 1.0 1.5 50 0 0 0.65";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -565,20 +551,10 @@ void gnome_qm1_docv::popup_ObjMOPlane(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/PlaneResolution", 30);
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getOrbital; cpp.cf = (ColorFunction *) GetRBRange1;
-		cpp.value = 0.05; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "molecular orbital "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane mo rb1 0.05 0.75 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -588,20 +564,10 @@ void gnome_qm1_docv::popup_ObjMOVolume(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 30);
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the volume [nm] (1.0 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getOrbital; cpp.cf = (ColorFunction *) GetRBRange2;
-		cpp.value = 0.05; cpp.alpha = 0.25;
-		
-		docv->AddObject(new volume_rendering_object(ol_static(), cpp, 20, tmp1 / 2.0, (* gv->cam), "MO "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add volrend mo rb2 0.025 1.5 25 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -611,39 +577,10 @@ void gnome_qm1_docv::popup_ObjMOSurface(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-		i32s size = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 50);
-		i32s sz[3] = { size, size, size };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp2a; cs_param csp2b;
-		csp2a.np = sz; csp2a.dim = dm; csp2a.ostr = & cout;
-		csp2a.docv = docv; csp2a.ref = docv->current_eng; csp2a.next = & csp2b;
-		csp2b.np = sz; csp2b.dim = dm; csp2b.ostr = & cout;
-		csp2b.docv = docv; csp2b.ref = docv->current_eng; csp2b.next = NULL;
-		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		tmp1 = 0.05;
-		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp2a.wireframe = csp2b.wireframe = (input == 'w' || input == 'W');
-		
-		csp2a.transparent = !true;
-		csp2a.vf1 = (ValueFunction *) qm1_eng::getOrbital; csp2a.vf2 = (ValueFunction *) GetUnity;
-		csp2a.cvalue = 1.0; csp2a.alpha = 0.50; csp2a.toler = 1.0e-6 * tmp1; csp2a.maxc = 250;
-			csp2a.cf = (ColorFunction *) GetRedColor; csp2a.svalue = +tmp1;
-			
-		csp2b.transparent = !true;
-		csp2b.vf1 = (ValueFunction *) qm1_eng::getOrbital; csp2b.vf2 = (ValueFunction *) GetUnity;
-		csp2b.cvalue = 1.0; csp2b.alpha = 0.50; csp2b.toler = 1.0e-6 * tmp1; csp2b.maxc = 250;
-			csp2b.cf = (ColorFunction *) GetBlueColor; csp2b.svalue = -tmp1;
-			
-		docv->AddObject(new color_surface_object(ol_static(), csp2a, "MO "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf2 mo unity red blue +0.025 -0.025 1.0 1.5 50 0 0 0.50";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -653,20 +590,10 @@ void gnome_qm1_docv::popup_ObjMODPlane(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/PlaneResolution", 30); 
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getOrbDens; cpp.cf = (ColorFunction *) GetGreenColor;
-		cpp.value = 0.0125; cpp.alpha = 0.75;
-		
-		docv->AddObject(new color_plane_object(ol_static(), cpp, "MO-density "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add plane mod rb1 0.005 0.75 50 1 0.75";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -676,20 +603,10 @@ void gnome_qm1_docv::popup_ObjMODVolume(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1;
-		cp_param cpp; cpp.np = docv->model_prefs->Value("QM1Graphics/SurfaceResolution", 30);
-		cpp.docv = docv; cpp.ref = docv->current_eng;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		cout << "give dimension of the volume [nm] (1.0 might be ok) ??? "; cin >> tmp1;
-		
-		cpp.dim = tmp1; cpp.transparent = true;
-		cpp.vf = (ValueFunction *) qm1_eng::getOrbDens; cpp.cf = (ColorFunction *) GetRBRange2;
-		cpp.value = 0.0125; cpp.alpha = 0.25;
-		
-		docv->AddObject(new volume_rendering_object(ol_static(), cpp, 20, tmp1 / 2.0, (* gv->cam), "MO "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add volrend mod rb2 0.0025 1.5 25 0.35";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -699,30 +616,10 @@ void gnome_qm1_docv::popup_ObjMODSurface(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		fGL tmp1; fGL tmp2; char input;
+if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
 		
-		i32s sz[3] = { 25, 25, 25 };
-		fGL dm[3] = { 0.55, 0.50, 0.45 };
-		
-		cs_param csp1;
-		csp1.np = sz; csp1.dim = dm; csp1.ostr = & cout;
-		csp1.docv = docv; csp1.ref = docv->current_eng; csp1.next = NULL;
-		
-	if (!docv->current_eng) { docv->err->ErrorMessage("Calculate energy first!!!"); return; }
-	
-		tmp1 = 0.0125;
-		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
-		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
-		
-		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
-		cin >> input; csp1.wireframe = (input == 'w' || input == 'W');
-		
-		csp1.transparent = true; csp1.cf = (ColorFunction *) GetGreenColor;
-		csp1.vf1 = (ValueFunction *) qm1_eng::getOrbDens; csp1.vf2 = (ValueFunction *) GetUnity;
-		csp1.svalue = tmp1; csp1.cvalue = 1.0; csp1.alpha = 0.50; csp1.toler = 1.0e-6 * tmp1; csp1.maxc = 250;
-		
-		docv->AddObject(new color_surface_object(ol_static(), csp1, "MO-density "));
-		docv->UpdateAllGraphicsViews();
+		static const char command[] = "add surf1 mod unity red 0.0025 1.0 1.5 50 0 0 0.65";
+		new command_dialog(docv, gv, command);
 	}
 }
 
@@ -732,8 +629,9 @@ void gnome_qm1_docv::popup_CompSetup(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	glade_modal_dialog_owner = docv;	// requires modal dialogs!!!
 	
-	GtkWidget * dialog = create_e1_dialog_qm1_setup();
-	gtk_widget_show(dialog);
+	new setup_qm1_dialog();
+	//GtkWidget * dialog = create_e1_dialog_qm1_setup();	// obsolete
+	//gtk_widget_show(dialog);				// obsolete
 }
 
 void gnome_qm1_docv::popup_CompEnergy(GtkWidget *, gpointer data)
@@ -764,12 +662,20 @@ void gnome_qm1_docv::popup_SetOrbital(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		cout << "current orbital is now " << docv->current_orbital << endl;
-		cout << "enter new value (BE CAREFUL, THERE'S NO BOUNDS CHECKING YET!!!) : ";
+		docv->err->Message("PLEASE NOTE!\nThe command string, which is displayed in the next dialog, is incomplete.\nYou should replace the letter X with the orbital index that will become the current orbital.");
 		
-		cin >> docv->current_orbital;
-		
-		cout << "current orbital now set to " << docv->current_orbital << endl;
+		static const char command[] = "set_current_orbital X";
+		new command_dialog(gv->docv, gv, command);
+	}
+}
+
+void gnome_qm1_docv::popup_EnterCommand(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
+	if (docv)
+	{
+		new command_dialog(docv, gv, NULL);
 	}
 }
 
@@ -798,7 +704,7 @@ mm_docv->camera_vector.front()->CopySettings(gv->cam);
 mm_docv->RemoveLight(mm_docv->light_vector.front());
 mm_docv->AddLocalLight(new directional_light(ol_static(mm_docv->camera_vector.front()->GetLocData())), mm_docv->camera_vector.front());
 
-		// convert qm1gp -> mm1gp using all_atoms_interface...
+		// convert qm1mdl -> mm1mdl using all_atoms_interface...
 		
 		mm_docv->aai_MakeCopy(docv);
 		
@@ -820,8 +726,9 @@ void gnome_qm1_docv::GeomOptGetParam(qm1_geomopt_param & param)
 	
 	goparam = & param;
 	
-	GtkWidget * dialog = create_a1_dialog_geomopt();
-	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	new geomopt_dialog();	// the object will call delete itself...
+	//GtkWidget * dialog = create_a1_dialog_geomopt();	// obsolete
+	//gnome_dialog_run_and_close(GNOME_DIALOG(dialog));	// obsolete
 	
 	goparam = NULL;
 }
