@@ -24,14 +24,10 @@ class gnome_qm1_docv;
 
 /*################################################################################################*/
 
-#define QM1_FILTER	"*.qm1gp"
+#define QM1_FILTER	"*.gpr"
 
 /*################################################################################################*/
 
-// save
-// import
-// export ???
-
 class qm1_file_open_dialog : public file_dialog
 {
 	private:
@@ -48,7 +44,7 @@ class qm1_file_open_dialog : public file_dialog
 	void CancelEvent(void);		// virtual
 };
 
-class qm1_importmm1_dialog : public file_dialog
+class qm1_file_save_dialog : public file_dialog
 {
 	private:
 	
@@ -57,8 +53,8 @@ class qm1_importmm1_dialog : public file_dialog
 	
 	public:
 	
-	qm1_importmm1_dialog(gnome_qm1_docv *);
-	~qm1_importmm1_dialog(void);
+	qm1_file_save_dialog(gnome_qm1_docv *);
+	~qm1_file_save_dialog(void);
 	
 	bool OkEvent(const char *);	// virtual
 	void CancelEvent(void);		// virtual
@@ -72,7 +68,7 @@ class gnome_qm1_docv : public gnome_docv, public qm1_docv
 {
 	protected:
 	
-	static GnomeUIInfo fileimport_submenu[];	// file
+	static GnomeUIInfo filext_submenu[];		// file
 	static GnomeUIInfo file_submenu[];
 	
 	static GnomeUIInfo select_submenu[];
@@ -82,6 +78,9 @@ class gnome_qm1_docv : public gnome_docv, public qm1_docv
 	static GnomeUIInfo viewscreate_submenu[];	// render/views
 	static GnomeUIInfo views_submenu[];		// render
 	static GnomeUIInfo lights_submenu[];		// render
+	static GnomeUIInfo rendermode_style_radio[];
+	static GnomeUIInfo rendermode_color_radio[];
+	static GnomeUIInfo rendermode_label_radio[];
 	static GnomeUIInfo rendermode_submenu[];	// render
 	static GnomeUIInfo projection_submenu[];	// render
 	static GnomeUIInfo render_submenu[];
@@ -108,14 +107,17 @@ class gnome_qm1_docv : public gnome_docv, public qm1_docv
 	// the qm1-dependent popup-menu callbacks are here...
 	
 	static void popup_FileOpen(GtkWidget *, gpointer);
-	static void popup_FileImport(GtkWidget *, gpointer);
 	static void popup_FileSaveAs(GtkWidget *, gpointer);
+	static void popup_FileExtra1(GtkWidget *, gpointer);
 
 	static void popup_ViewsNewEnLevDiag(GtkWidget *, gpointer);
 	
 	static void popup_RModeWireFrame(GtkWidget *, gpointer);
 	static void popup_RModeBallAndStick(GtkWidget *, gpointer);
 	static void popup_RModeNothing(GtkWidget *, gpointer);
+	static void popup_CModeElement(GtkWidget *, gpointer);
+	static void popup_LModeNothing(GtkWidget *, gpointer);
+	static void popup_LModeDEBUG(GtkWidget *, gpointer);
 	
 	static void popup_ObjEPlane(GtkWidget *, gpointer);
 	static void popup_ObjESurface(GtkWidget *, gpointer);
@@ -135,6 +137,7 @@ class gnome_qm1_docv : public gnome_docv, public qm1_docv
 	
 	static void popup_SetOrbital(GtkWidget *, gpointer);
 	
+	static void popup_EnterCommand(GtkWidget *, gpointer);
 	static void popup_ConvertMM(GtkWidget *, gpointer);
 	
 	public:
