@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#define ENABLE_NLS	// supress the "gcc-3.2 and libintl.h" problem...
 #include "t3views.h"
 
 #include "t3main.h"
@@ -510,29 +511,8 @@ void gnome_graphics_view::popup_LightsNewLight(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = GetGV((GtkWidget *) data);
 	
-	char input; light * new_light;
-	
-	cout << "add light:" << endl;
-	cout << "directional- or spotlight ? (d/s) "; cin >> input;
-	
-	if (input == 'd') new_light = new directional_light(ol_static());
-	else if (input == 's')
-	{
-		new_light = new spot_light(ol_static());
-		
-		const fGL trans[3] = { 0.0, 0.0, -1.0 };
-		new_light->TranslateObject((const fGL *) trans, gv->cam->GetLocData());
-	}
-	else return;
-	
-	cout << "global or local light ? (g/l) "; cin >> input;
-	
-	if (input == 'g') gv->docv->AddGlobalLight(new_light);
-	else if (input == 'l') gv->docv->AddLocalLight(new_light, gv->cam);
-	else delete new_light;
-	
-	if (input == 'g' || graphics_view::draw_info) gv->docv->UpdateAllGraphicsViews();
-	else if (input == 'l') gv->docv->UpdateGraphicsViews(gv->cam);
+	static const char command[] = "add light";
+	new command_dialog(gv->docv, gv, command);
 }
 
 void gnome_graphics_view::popup_LightsSwitchLoc(GtkWidget *, gpointer data)
@@ -544,7 +524,7 @@ void gnome_graphics_view::popup_LightsSwitchLoc(GtkWidget *, gpointer data)
 void gnome_graphics_view::popup_LightsSwitchGlob(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = GetGV((GtkWidget *) data);
-	gv->docv->DoSwitchLocalLights(gv->cam, false);
+	gv->docv->DoSwitchGlobalLights(gv->cam, false);
 }
 
 void gnome_graphics_view::popup_ObjectsDeleteCurrent(GtkWidget *, gpointer data)
@@ -814,6 +794,8 @@ void gnome_treelist_view::SelectChildHandler(GtkTree * subt, GtkWidget * item, g
 
 void gnome_treelist_view::SelectChildEvent(GtkTree * subt, GtkWidget * item)
 {
+	char mbuff1[256];
+	
 	if (subt == GTK_TREE(subt_lights))
 	{
 		i32u index = 0;
@@ -822,8 +804,9 @@ void gnome_treelist_view::SelectChildEvent(GtkTree * subt, GtkWidget * item)
 			if (light_vector[index].item != item) index++;
 			else
 			{
-				cout << "treelist: selected light ";
-				cout << light_vector[index].ref->GetObjectName() << endl;
+				ostrstream str1(mbuff1, sizeof(mbuff1));
+				str1 << "treelist: selected light " << light_vector[index].ref->GetObjectName() << "." << endl << ends;
+				docv->PrintToLog(mbuff1);
 				
 				docv->SelectLight(light_vector[index].ref);
 				return;
@@ -839,8 +822,9 @@ void gnome_treelist_view::SelectChildEvent(GtkTree * subt, GtkWidget * item)
 			if (object_vector[index].item != item) index++;
 			else
 			{
-				cout << "treelist: selected object ";
-				cout << object_vector[index].ref->GetObjectName() << endl;
+				ostrstream str1(mbuff1, sizeof(mbuff1));
+				str1 << "treelist: selected object " << object_vector[index].ref->GetObjectName() << "." << endl << ends;
+				docv->PrintToLog(mbuff1);
 				
 				docv->SelectObject(object_vector[index].ref);
 				return;
@@ -899,6 +883,8 @@ graphics_view * gnome_class_factory::ProduceGraphicsView(docview * docv1, camera
 	{
 		gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), gv->view_widget, gv->label_widget);
 		gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
+		
+		gtk_widget_show(GTK_WIDGET(docv2->notebook));
 	}
 	
 	return gv;
@@ -912,6 +898,7 @@ plot1d_view * gnome_class_factory::ProducePlot1DView(docview * docv1, i32s ud1,
 	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), plotv->view_widget, plotv->label_widget);
 	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
 	
+	gtk_widget_show(GTK_WIDGET(docv2->notebook));
 	return plotv;
 }
 
@@ -923,17 +910,19 @@ plot2d_view * gnome_class_factory::ProducePlot2DView(docview * docv1, i32s ud1,
 	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), plotv->view_widget, plotv->label_widget);
 	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
 	
+	gtk_widget_show(GTK_WIDGET(docv2->notebook));
 	return plotv;
 }
 
 enlevdiag_view * gnome_class_factory::ProduceEnLevDiagView(qm1_mdl * mdl)
 {
-	gnome_docv * docv = dynamic_cast<gnome_docv *>(mdl);
+	gnome_docv * docv2 = dynamic_cast<gnome_docv *>(mdl);
 	
 	gnome_enlevdiag_view * eldv = new gnome_enlevdiag_view(mdl);
-	gtk_notebook_append_page(GTK_NOTEBOOK(docv->notebook), eldv->view_widget, eldv->label_widget);
-	gtk_notebook_set_page(GTK_NOTEBOOK(docv->notebook), -1);	// activate the last page.
+	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), eldv->view_widget, eldv->label_widget);
+	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
 	
+	gtk_widget_show(GTK_WIDGET(docv2->notebook));
 	return eldv;
 }
 
@@ -944,7 +933,8 @@ treelist_view * gnome_class_factory::ProduceTreeListView(docview * docv1)
 	gnome_treelist_view * tlv = new gnome_treelist_view(docv2);
 	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), tlv->view_widget, tlv->label_widget);
 //	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page. // currently has no effect...
-	
+
+	gtk_widget_show(GTK_WIDGET(docv2->notebook));
 	return tlv;
 }
 
